#ifndef __DATASET_H
#define __DATASET_H     1


/* Symbole fuer physikalische Einheiten */
typedef enum _DS_units_e
{
    DS_unit_ndef    = 0;    /* Einheit nicht definiert */
    DS_unit_degree  = 1;    /* Winkelgrad (z.B. fuer Himmelsausschnitt) */
    DS_unit_meter   = 2;    /* Meter (z.B. fuer Pixelkantenlaengen) */
}
DS_units_e;


 
/* Achsenbeschreibung */ 
typedef struct _DS_axis_t 
{ 
    unsigned long   length;     /* Achsenlaenge == Anzahl Messwerte */ 
    DS_units_e      unit;       /* Messwerteinheit */ 
    double          samplesize; /* Messwertgroesse */ 
    char            *name;      /* Achsenbezeichnung */ 
} 
DS_axis_t; 
 
 
 
/* Symbole fuer Datentypen */ 
typedef enum _DS_datatypes_e 
{ 
    DS_dt_byte      = 1;    /* 8 Bit Ganzzahl mit Vorzeichen */ 
    DS_dt_short     = 2;    /* 16 Bit Ganzzahl mit Vorzeichen */ 
    DS_dt_long      = 3;    /* 32 Bit Ganzzahl mit Vorzeichen */ 
    DS_dt_float     = 4;    /* 32 Bit Fliesskommazahl */ 
    DS_dt_double    = 5;    /* 64 Bit Fliesskommazahl */ 
    DS_dt_cbyte     = 129;  /* 2x 8 Bit Ganzzahl mit Vorzeichen (komplex) */ 
    DS_dt_cshort    = 130;  /* 2x 16 Bit Ganzzahl mit Vorzeichen (komplex) */ 
    DS_dt_clong     = 131;  /* 2x 32 Bit Ganzzahl mit Vorzeichen (komplex) */ 
    DS_dt_cfloat    = 132;  /* 2x 32 Bit Fliesskommazahl (komplex) */ 
    DS_dt_cdouble   = 133;  /* 2x 64 Bit Fliesskommazahl (komplex) */ 
                            /* bei komplexen Zahlen bildet das erste Datenwort den Realteil, das 
                               zweite den Imaginaerteil */ 
} 
DS_datatypes_e; 
 
 
 
/* Struktur fuer Zeitangaben, uebernommen aus der CCDLIB von Georg Dittie */ 
typedef struct _DS_time_t 
{ 
    short   day;        /* Tag 1 - 28/31 */ 
    short   month;      /* Monat 1 - 12 */ 
    short   year;       /* Jahr 4 stellig */ 
    short   hour;       /* Stunde 0 - 23 */ 
    short   min;        /* Minute 0 - 59 */ 
    short   sec;        /* Sekunde 0 - 59 */ 
    short   msec;       /* Millisekunden 0 - 999 */ 
    short   timezone;   /* Zeitzone -12 bis 12 bez. UT ( 0 = UT ) */ 
    char    name[12];   /* Tagesname */ 
    double  juldat;     /* Vollstaendiges julianisches Datum auf 15 Stellen */ 
} 
DS_time_t; 
 
 
 
/* Struktur fuer weitere Informationen, die einzelnen Felder ("Tags") muessen 
   nicht ausgefuellt werden. Leere Felder haben den Wert 0 bzw. NULL. */ 
typedef struct _DS_tags_t 
{ 
    DS_time_t       startobs;       /* Beginn der Messung */ 
    DS_time_t       endobs;         /* Ende der Messung */ 
    unsigned long   serialid;       /* ID der Serie */ 
    unsigned long   serialcount;    /* Laufende Nummer der Messung */ 
    char            *object;        /* Objektbezeichnung */ 
    char            *observer;      /* Beobachtername */ 
    char            *instrument;    /* Instrumentbezeichnung */ 
    char            *comment;       /* Kommentar */ 
    double          *sensortemp;    /* Sensortemperatur in Grad Celsius */ 
} 
DS_tags_t; 
 
 
 
/* Definition eines Datensatzes */ 
typedef struct _DS_dataset_t 
{ 
    unsigned char   naxis;          /* Achsenanzahl */ 
    DS_axis_t       **axistable;    /* Zeiger auf Zeigertabelle fuer Achsen */ 
    DS_datatypes_e  datatype;       /* Datentyp */ 
    double          offset;         /* Offset fuer Ganzzahlen */ 
    double          dynamic;        /* Dynamik fuer Ganzzahlen */ 
    void            **datatable;    /* Zeiger auf Zeigertabelle fuer Datenbloecke */ 
    DS_tags_t       *tags;          /* Zusatzinformationen ("Tags") */ 
} 
DS_dataset_t; 
 
 
 
#endif 
 
