/******************************************************************************
*                                                                             *
*   MATHCNST.H                                                                *
*   ==========                                                                *
*                                                                             *
*   This file includes definitions of mathematical constants.                 *
*                                                                             *
*   version:      1.0                                                         *
*   last change:  14.4.1999                                                   *
*                                                                             *
******************************************************************************/



#ifndef __MATHCNST_H__
#define __MATHCNST_H__  1



#ifndef MC_E
  #define MC_E        2.7182818284590452354       /* e */
#endif
#ifndef MC_LOG2E
  #define MC_LOG2E    1.4426950408889634074       /* log 2e */
#endif
#ifndef MC_LOG10E
  #define MC_LOG10E   0.43429448190325182765      /* log 10e */
#endif
#ifndef MC_LN2
  #define MC_LN2      0.69314718055994530942      /* log e2 */
#endif
#ifndef MC_LN10
  #define MC_LN10     2.30258509299404568402      /* log e10 */
#endif


/* some defines for the constant PI */
#ifndef MC_PI
  #define MC_PI       3.14159265358979323846      /* pi */
#endif
#ifndef MC_1dPI
  #define MC_1dPI     0.31830988618379067154      /* 1/pi */
#endif
#ifndef MC_2dPI
  #define MC_2dPI     0.63661977236758134308      /* 2/pi */
#endif
#ifndef MC_PId2
  #define MC_PId2     1.57079632679489661923      /* pi/2 */
#endif
#ifndef MC_PId3
  #define MC_PId3     MC_PI/3.                    /* pi/3 */
#endif
#ifndef MC_PId4
  #define MC_PId4     0.78539816339744830962      /* pi/4 */
#endif
#ifndef MC_PId6
  #define MC_PId6     MC_PI/6.                    /* pi/6 */
#endif
#ifndef MC_PId12
  #define MC_PId12    MC_PI/12.                   /* pi/12 */
#endif
#ifndef MC_PId36
  #define MC_PId36    MC_PI/36.                   /* pi/36 */
#endif
#ifndef MC_2d3PI
  #define MC_2d3PI    MC_PI/1.5                   /* 2/3 pi */
#endif
#ifndef MC_4d3PI
  #define MC_4d3PI    MC_PI/0.75                  /* 4/3 pi */
#endif
#ifndef MC_3d2PI
  #define MC_3d2PI    MC_PI*1.5                   /* 3/2 pi */
#endif
#ifndef MC_2PI
  #define MC_2PI      MC_PI*2.                    /* 2 pi */
#endif

/* angles in rad */
#define MC_DEG        MC_PI/180.
#define MC_MIN        MC_PI/10800.
#define MC_SEC        MC_PI/648000.
#define MC_270GRAD    MC_2d3PI
#define MC_180GRAD    MC_PI
#define MC_90GRAD     MC_PId2
#define MC_15GRAD     MC_PId12
#define MC_5GRAD      MC_PId36

/* angles in degree */
#define MC_RAD        180./MC_PI


#ifndef MC_2dSQRTPI
  #define MC_2dSQRTPI 1.12837916709551257390      /* 2/sqrt(pi) */
#endif
#ifndef MC_SQRT2
  #define MC_SQRT2    1.41421356237309504880      /* sqrt(2) */
#endif
#ifndef MC_SQRT1_2
  #define MC_SQRT1_2  0.70710678118654752440      /* 1/sqrt(2) */
#endif



/* the old definitions:

#ifndef PI
  #define PI          3.14159265358979323846
#endif

#ifndef PI2
  #define PI2         PI*2.
#endif

#define PID2          PI/2.
#define PID3          PI/3.
#define PID4          PI/4.
#define PID6          PI/6.
#define PID12         PI/12.
#define PID36         PI/36.
#define PID23         PI/1.5
#define PID43         PI/0.75
#define PI3D2         PI*1.5

#define PI_GRAD       PI/180.
#define PI_MIN        PI/10800.
#define PI_SEC        PI/648000.

#define PI_270GRAD    PI3D2
#define PI_180GRAD    PI
#define PI_90GRAD     PID2
#define PI_15GRAD     PID12
#define PI_5GRAD      PID36

#define RAD           (3.14159265358979323846/180.)
#define DEG           (180./3.14159265358979323846)
*/



#endif

