#pragma warn -waus

#include  <stdlib.h>
#include  <string.h>
#include  <math.h>
#include  <time.h>
#include  <stdio.h>


/******************************************************************************
* system dependent definitions                                                *
******************************************************************************/

#ifdef  __OS_WIN32
  #include  "cl_win32.h"
#endif

#ifdef _PROGRESS_
  extern long progresscounter;
#endif


/******************************************************************************
* header files of project                                                     *
******************************************************************************/

#include  "ccdlib.h"
#include  "cl_error.h"
#include  "mathcnst.h"



/* miscellaneous functions */

extern FILE *debug;
extern char debuglog[80];
extern void Debug( char * );


/***************************************************************************
*                                                                          *
*   CCDLIB_CopyRegion:  Routine zum Kopieren von Ausschnitten              *
*                                                                          *
*   Parameter:  * destbuffer  : Zielbild                                   *
*               * sourcebuffer: Quellbuffer                                *
*               * pictureparam: Systemparameter des zu bearbeitenden       *
*                               Bildes                                     *
*               * copyrect    : Auschnittrechteck                          *
*                                                                          *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung      *
*                                                                          *
*               CCDLIB_Successful:            Alles OK !                   *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen     *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse          *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.    *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen     *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher            *
*                                                                          *
*   Version:    1.0 (18.05.1996)                                           *
*   Written by: Georg Dittie, 1996                                         *
*                                                                          *
***************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_CopyRegion(
  short                     *destbuffer,
  short                     *sourcebuffer,
  CCDLIB_PictureparamStruct *picpar,
  CCDLIB_RectangleStruct    *copyrect
)
{
  long xs, xd, yss, yds, slinestep, dlinestep,
       xstart, xstop, ystart, ystop;

  /* check arguments */
  if( (picpar == NULL) || (copyrect == NULL) )
    return( CCDLIB_NoPointerError );
  if( (sourcebuffer == NULL) || (destbuffer == NULL) )
    return( CCDLIB_NoPicturePointerError );
  if( ! CCDLIB_CheckRectangleValues( copyrect, picpar ) )
    return( CCDLIB_WrongRectangleValuesError );

  /* calculate start and stop positions in array */  
  xstart  = copyrect->xpos;
  ystart  = copyrect->ypos;
  xstop   = copyrect->xpos + copyrect->xsize;
  ystop   = copyrect->ypos + copyrect->ysize;
  ystart *= picpar->linelength;
  ystop  *= picpar->linelength;

  /* for black-white images copy line after line in given box to           */
  /* targetbuffer                                                          */
  if( picpar->colorchannel != CCDLIB_RGBChannelColor )
  {
    slinestep = picpar->linelength;
    dlinestep = copyrect->xsize;
    yds = 0;
    for( yss = ystart; yss < ystop; yss += slinestep )
    {
      xd = 0;
        for(xs = xstart; xs < xstop; xs ++ )
        {
          destbuffer[ yds + xd ] = sourcebuffer[ yss + xs ];
          xd++;
        }
        yds += dlinestep;
    }
  }
 
  /* for color images do the same but 3 times */
  else
  {
    xstart *= 3;  ystart *= 3;
    xstop  *= 3;  ystop  *= 3;
    slinestep = picpar->linelength*3;
    dlinestep = copyrect->xsize*3;
    yds = 0;
    for( yss = ystart; yss < ystop; yss += slinestep )
    {
      xd = 0;
      for(xs = xstart; xs < xstop; xs ++ )
      {
        destbuffer[ yds + xd ] = sourcebuffer[ yss + xs ];
        xd++;
      }
      yds += dlinestep;
    }
  }

  return( CCDLIB_Successful );

}



/***************************************************************************
*                                                                          *
*   CCDLIB_InitRectangle :  Routine zum Setzen von Rechteckwerten          *
*                                                                          *
*   Parameter:  * picturrect: Zu fuellende Struktur                        *
*               pos,size    : Fuellwerte                                   *
*                                                                          *
*   Version:    1.0 (21.09.1997)                                           *
*   Written by: Georg Dittie, 1997                                         *
*                                                                          *
***************************************************************************/

void CCDLIB_InitRectangle(
  CCDLIB_RectangleStruct *picturerect,
  long                   xpos,
  long                   ypos,
  long                   xsize,
  long                   ysize 
)
{
  /* check pointer to box structure */
  if( picturerect == NULL)    return;

  /* set box values */
  picturerect->xpos  = xpos;
  picturerect->ypos  = ypos;
  picturerect->xsize = xsize;
  picturerect->ysize = ysize;
}


/**************************************************************************
*                                                                         *
*   CCDLIB_FlipMirror: Routine zum Spiegeln oder rechtwinkligen Drehen.   *
*                                                                         *
*   Parameter:  flipmethod:       Arbeitsanweisung                        *
*               *sourcepicparam:  Parameter der Quelle                    *
*               *destpicparam  :  Parameter des Ziels                     *
*               *sourcepic     :  32-Bit-Pointer auf die Bildquelle       *
*               *destpic       :  32-Bit-Pointer auf das Bildziel         *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   ACHTUNG: Source und Ziel duerfen nicht identisch sein !!!             *
*                                                                         *
*   Version:    1.0, (21.09.1997)                         32 Bit          *
*   Written by: Georg Dittie, 1997                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_FlipMirror
(
  enum CCDLIB_FlipMethodEnum flipmethod,
  CCDLIB_PictureparamStruct  *sourcepicparam,
  CCDLIB_PictureparamStruct  *destpicparam,
  short                      *sourcepic,
  short                      *destpic
)
{
  long xs, xd, ys, yd, xsstop, ysstop,
       xdstep, ydstep, ydstart, xdstart;

  /* argument checking */
  if( (sourcepic == NULL) || (destpic == NULL) )
    return( CCDLIB_NoPicturePointerError );
  if( (sourcepicparam == NULL) || (destpicparam == NULL) )
    return( CCDLIB_NoPointerError );
  if( (sourcepic == destpic) || (sourcepicparam == destpicparam) )
    return( CCDLIB_WrongParameterError );

  /* copy source parameters to destination parameter structure */
  *destpicparam = *sourcepicparam;

  /* for black and white images: */
  if( sourcepicparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    /* calculate stop column and stop row */
    xsstop = sourcepicparam->linelength;
    ysstop = sourcepicparam->columnheight*sourcepicparam->linelength;

    switch( flipmethod )
    {

      /* flip top-bottom */
      case CCDLIB_FlipTopdownConst:
        ydstep  = sourcepicparam->linelength;
        ydstart = ysstop-sourcepicparam->linelength;
        for( ys=0, yd=ydstart; ys<ysstop; ys+=ydstep, yd-=ydstep )
          for( xs=0; xs<xsstop; xs++ )
            destpic[ xs+yd ] = sourcepic[ xs+ys ];
        break;

      /* flip left-right */
      case CCDLIB_FlipMirrorConst:
        ydstep  = sourcepicparam->linelength;
        xdstart = sourcepicparam->linelength-1;
        for( ys=0; ys<ysstop; ys+=ydstep )
          for( xs=0, xd=xdstart; xs<xsstop; xs++, xd-- )
            destpic[ xd+ys ] = sourcepic[ xs+ys ];
        break;

      /* 180 degree rotation */
      case CCDLIB_FlipErectConst:
        ydstep  = sourcepicparam->linelength;
        xdstart = sourcepicparam->linelength-1;
        ydstart = ysstop-sourcepicparam->linelength;
        for( ys=0, yd=ydstart; ys<ysstop; ys+=ydstep, yd-=ydstep )
          for( xs=0, xd=xdstart; xs<xsstop; xs++, xd-- )
            destpic[ xd+yd ] = sourcepic[ xs+ys ];
        break;
  
      /* 90 degree clockwise rotation */  
      case CCDLIB_FlipRightturnConst:
        xdstep  = sourcepicparam->columnheight;
        ydstep  = sourcepicparam->linelength;
        ydstart = ysstop-sourcepicparam->columnheight;
        for( ys=0, xd=0; ys<ysstop; ys+=ydstep, xd++ )
          for( xs=0, yd=ydstart; xs<xsstop; xs++, yd-=xdstep )
            destpic[ xd+yd ] = sourcepic[ xs+ys ];
        destpicparam->linelength = sourcepicparam->columnheight;
        destpicparam->columnheight = sourcepicparam->linelength;
        break;

      /* 90 degree counterclockwise rotation */
      case CCDLIB_FlipLeftturnConst:
        xdstep  = sourcepicparam->columnheight;
        ydstep  = sourcepicparam->linelength;
        xdstart = sourcepicparam->columnheight-1;
        for( ys=0, xd=xdstart; ys<ysstop; ys+=ydstep, xd-- )
          for( xs=0, yd=0; xs<xsstop; xs++, yd+=xdstep )
        destpic[ xd+yd ] = sourcepic[ xs+ys ];
        destpicparam->linelength = sourcepicparam->columnheight;
        destpicparam->columnheight = sourcepicparam->linelength;
        break;
  
      /* wrong flip method */
      default:
        return( CCDLIB_WrongParameterError );
        break;
    }

  }

  /* for color images do the same but 3 times */
  else
  {
    /* calculate stop column and stop row */    
    xsstop = sourcepicparam->linelength*3;
    ysstop = sourcepicparam->columnheight*sourcepicparam->linelength*3;

    switch( flipmethod )
    {

      /* flip top-bottom */
      case CCDLIB_FlipTopdownConst:
        ydstep  = xsstop;
        ydstart = ysstop-xsstop;
        for( ys=0, yd=ydstart; ys<ysstop; ys+=ydstep, yd-=ydstep )
          for( xs=0; xs<xsstop; xs+=3 )
          {
            destpic[ xs+yd ]   = sourcepic[ xs+ys ];
            destpic[ xs+yd+1 ] = sourcepic[ xs+ys+1 ];
            destpic[ xs+yd+2 ] = sourcepic[ xs+ys+2 ];
          }
        break;

      /* flip left-right */
      case CCDLIB_FlipMirrorConst:
        ydstep  = xsstop;
        xdstart = xsstop-3;
        for( ys=0; ys<ysstop; ys+=ydstep )
          for( xs=0, xd=xdstart; xs<xsstop; xs+=3, xd-=3 )
          {
            destpic[ xd+ys ]   = sourcepic[ xs+ys ];
            destpic[ xd+ys+1 ] = sourcepic[ xs+ys+1 ];
            destpic[ xd+ys+2 ] = sourcepic[ xs+ys+2 ];
          }
        break;
  
      /* 180 degree rotation */
      case CCDLIB_FlipErectConst:
        ydstep  = xsstop;
        xdstart = xsstop-3;
        ydstart = ysstop-xsstop;
        for( ys=0, yd=ydstart; ys<ysstop; ys+=ydstep, yd-=ydstep )
          for( xs=0, xd=xdstart; xs<xsstop; xs+=3, xd-=3 )
          {
            destpic[ xd+yd ]   = sourcepic[ xs+ys ];
            destpic[ xd+yd+1 ] = sourcepic[ xs+ys+1 ];
            destpic[ xd+yd+2 ] = sourcepic[ xs+ys+2 ];
          }
        break;
  
      /* 90 degree clockwise rotation */  
      case CCDLIB_FlipRightturnConst:
        xdstep  = (sourcepicparam->columnheight)*3;
        ydstep  = (sourcepicparam->linelength)*3;
        ydstart = ysstop-sourcepicparam->columnheight*3;
        for( ys=0, xd=0; ys<ysstop; ys+=ydstep, xd+=3 )
          for( xs=0, yd=ydstart; xs<xsstop; xs+=3, yd-=xdstep )
          {
            destpic[ xd+yd ]   = sourcepic[ xs+ys ];
            destpic[ xd+yd+1 ] = sourcepic[ xs+ys+1 ];
            destpic[ xd+yd+2 ] = sourcepic[ xs+ys+2 ];
          }
        destpicparam->linelength = sourcepicparam->columnheight;
        destpicparam->columnheight = sourcepicparam->linelength;
        break;

      /* 90 degree counterclockwise rotation */
      case CCDLIB_FlipLeftturnConst:
        xdstep  = (sourcepicparam->columnheight)*3;
        ydstep  = (sourcepicparam->linelength)*3;
        xdstart = (sourcepicparam->columnheight-1)*3;
        for( ys=0, xd=xdstart; ys<ysstop; ys+=ydstep, xd-=3 )
          for( xs=0, yd=0; xs<xsstop; xs+=3, yd+=xdstep )
          {
            destpic[ xd+yd ]   = sourcepic[ xs+ys ];
            destpic[ xd+yd+1 ] = sourcepic[ xs+ys+1 ];
            destpic[ xd+yd+2 ] = sourcepic[ xs+ys+2 ];
          }
        destpicparam->linelength = sourcepicparam->columnheight;
        destpicparam->columnheight = sourcepicparam->linelength;
        break;

      /* wrong flip method */
      default:
        return( CCDLIB_WrongParameterError );
        break;
    }

  }

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_AddPictures : Routine zum anteilmaessigen Aufaddieren von      *
*                      Bildern. Als Resultat entsteht die Summe aller     *
*                      bisher aufaddierten Bilder. Hier kann in ein       *
*                      konstantes oder mitwachsendes Bildfeld addiert     *
*                      werden.                                            *
*                                                                         *
*   ACHTUNG:    Addpuffer, Addcount und Picparam werden bei jedem Schritt *
*               geaendert, wenn die Additionsflaeche mitwaechst ! Die Be- *
*               zugskoordinate bezieht sich immer auf den aktuellen Addi- *
*               tionspuffer !                                             *
*                                                                         *
*   Hinweis: Pixel mit negativen Grauwerten werden nicht mitaddiert !     *
*                                                                         *
*                                                                         *
*   Parameter:  *sourcepicrect: Arbeitsbereiches im Quellbild             *
*               *destpicpos   : Position der oberen/linken Zielecke       *
*               *pictureparam : Systemparameter des zu bearbeitenden      *
*                               Bildes                                    *
*               *picpuffer    : 32-Bit-Pointer auf das aktuelle Bild      *
*               **addpuffer   : 32-Bit-Pointer auf die Grauwertsumme      *
*               **addcount    : 32-Bit-Ptr auf Anzahl der add. Pixel      *
*               piccount      : Anzahl der bisher aufaddierten Bilder     *
*               constarea     : FALSE = mitwachsendes Bildfeld            *
*                               TRUE: Anfangsbildfeld bleibt konstant     *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    4.0, (10.04.1999)                                         *
*   Written by: Georg Dittie, 1996-1999                                   *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_AddPictures
(
  CCDLIB_PositionStruct     *destpicpos,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  CCDLIB_PictureparamStruct *addparam,
  long                      **addpuffer,
  long                      **addcount,
  long                      piccount,
  BOOL                      constarea
)
{
  long greyval, greyflag, xs, xd, xc, ys, yd, yc,
       xsstart, ysstart, xstop, ystop,
       xdstart, ydstart, ysstep, ydstep,
       istop, cstop, oldlinelength, oldcolumnheight;

  long *newaddpuffer, *newaddcount;

  /* argument checking */
  if( piccount < 0 )  return( CCDLIB_WrongParameterError );
  if( (destpicpos == NULL) || (pictureparam == NULL) )
    return( CCDLIB_NoPointerError );
  if( (picture == NULL) || (addpuffer == NULL) || (addcount == NULL) )
    return( CCDLIB_NoPicturePointerError );

  /* if it is the first image: allocate new add nd count buffer and clean up */
  if( piccount == 0 )
  {
    cstop = pictureparam->linelength * pictureparam->columnheight;
    if( pictureparam->colorchannel == CCDLIB_RGBChannelColor )
      istop = cstop * 3L;
    else
      istop = cstop;

    *addpuffer = (long *)malloc( istop*sizeof(long) );
    *addcount  = (long *)malloc( cstop*sizeof(long) );
    if( *addpuffer == NULL || *addcount == NULL )
    {
      if( *addpuffer != NULL )
      {
        free( *addpuffer );
        *addpuffer = NULL;
      }
      if( *addcount != NULL )
      {
        free( *addcount );
        *addcount = NULL;
      }
      return( CCDLIB_OutOfMemoryError );
    }
    memset( *addpuffer, 0, istop*sizeof(long) );
    memset( *addcount, 0, cstop*sizeof(long) );

    *addparam = *pictureparam;
  }

  else if( piccount != 0 && constarea == FALSE )
  {
    oldlinelength   = addparam->linelength;
    oldcolumnheight = addparam->columnheight;

    if( destpicpos->xpos < 0 )
    {
      addparam->linelength -= destpicpos->xpos;
      xdstart = -destpicpos->xpos;
    }
    else if( destpicpos->xpos+pictureparam->linelength > addparam->linelength )
    {
      addparam->linelength = pictureparam->linelength+destpicpos->xpos;
      xdstart = 0;
    }
    else
      xdstart = 0;

    xstop = xdstart + oldlinelength;

    if( destpicpos->ypos < 0 )
    {
      addparam->columnheight -= destpicpos->ypos;
      ydstart = -destpicpos->ypos;
    }
    else if( destpicpos->ypos+pictureparam->columnheight > addparam->columnheight )
    {
      addparam->columnheight = pictureparam->columnheight+destpicpos->ypos;
      ydstart = 0;
    }
    else
      ydstart = 0;

    ystop = ydstart + oldcolumnheight;

    cstop = addparam->linelength * addparam->columnheight;
    if( addparam->colorchannel == CCDLIB_RGBChannelColor )
      istop = cstop * 3L;
    else
      istop = cstop;

    newaddpuffer = (long *)malloc( istop*sizeof(long) );
    newaddcount  = (long *)malloc( cstop*sizeof(long) );

    if( newaddpuffer == NULL || newaddcount == NULL )
    {
      if( newaddpuffer != NULL ) free( newaddpuffer );
      if( newaddcount != NULL )  free( newaddcount );
      return( CCDLIB_OutOfMemoryError );
    }

    memset( newaddpuffer, 0, istop*sizeof(long) );
    memset( newaddcount, 0, cstop*sizeof(long) );

    ydstart *= addparam->linelength;
    ystop   *= addparam->linelength;

    for( yd=ydstart, ys=0; yd<ystop; yd+=addparam->linelength, ys+=oldlinelength )
      for( xd = xdstart, xs = 0; xd < xstop; xd++, xs ++ )
        newaddcount[ yd+xd ] = (*addcount)[ ys+xs ];

    if( addparam->colorchannel == CCDLIB_RGBChannelColor )
    {
      ydstart *= 3; xdstart *= 3; ystop *=3; xstop *= 3;
    }

    for( yd = ydstart, ys = 0; yd < ystop; yd += addparam->linelength, ys += oldlinelength )
      for( xd = xdstart, xs = 0; xd < xstop; xd++, xs ++ )
        newaddpuffer[ yd+xd ] = (*addpuffer)[ ys+xs ];

    free( *addpuffer );
    free( *addcount );
    *addpuffer = newaddpuffer;
    *addcount  = newaddcount;
  }

  xdstart = destpicpos->xpos;
  xsstart = 0;
  xstop   = xdstart + pictureparam->linelength;
  ydstart = destpicpos->ypos;
  ysstart = 0;
  ystop   = ydstart + pictureparam->columnheight;

  if( constarea == TRUE )
  {
    if( xdstart < 0 )
    { 
      xdstart = 0; xsstart = -destpicpos->xpos; 
    }
    if( ydstart < 0 )
    { 
      ydstart = 0; ysstart = -destpicpos->ypos; 
    }
    if( xstop >= addparam->linelength )
      xstop = addparam->linelength-1;
    if( ystop >= addparam->columnheight )
      ystop = addparam->columnheight-1;
  }

  ysstart *= addparam->linelength;
  ydstart *= addparam->linelength;
  ystop   *= addparam->linelength;

  /* for color images: recalc array positions */
  if( pictureparam->colorchannel == CCDLIB_RGBChannelColor )
  {
    xstop *= 3;
    ystop *= 3;
    ydstep = addparam->linelength*3;
    ysstep = pictureparam->linelength*3;

    for( yd = ydstart*3, ys = ysstart*3, yc = ydstart;
         yd < ystop;
         yd += ydstep, ys += ysstep, yc += addparam->linelength )
    {
      for( xd = xdstart*3, xs = xsstart*3, xc = xdstart;
           xd < xstop;
           xd += 3, xs += 3, xc++ )
      {
        greyflag = 0;
        greyval = (long)picture[ ys + xs ];
        if( greyval >= 0 )
          (*addpuffer)[ yd + xd ] += greyval;
        else
          greyflag++;

        greyval = (long)picture[ ys + xs + 1 ];
        if( greyval >= 0 )
          (*addpuffer)[ yd + xd + 1 ] += greyval;
        else
          greyflag++;

        greyval = (long)picture[ ys + xs + 2 ];

        if( greyval >= 0 )
          (*addpuffer)[ yd + xd + 2 ] += greyval;
        else
          greyflag++;

        if( greyflag == 0 )
          (*addcount)[ yc + xc ]++;
      }
    }
  }

  /* grey image */
  else
  {
    for( yd = ydstart, ys = ysstart;
         yd < ystop;
         yd += addparam->linelength, ys += pictureparam->linelength )
    {
      for( xd = xdstart, xs = xsstart;
           xd < xstop;
           xd ++, xs ++ )
      {
        greyval = (long)picture[ ys + xs ];
        if( greyval >= 0 )
        {
          (*addpuffer)[ yd + xd ] += greyval;
          (*addcount)[ yd + xd ]++;
        }
      }
    }
  }

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_MoveAddfieldToPicture : Routine zum normierten Uebertragen     *
*               von auffaddierten Bildern. Als Resultat entsteht der      *
*               arithmetische Mittelwert aller aufaddierten Bilder.       *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *picpuffer   : 32-Bit-Pointer auf das aktuelle Bild       *
*               *addpuffer   : 32-Bit-Pointer auf die Summe               *
*               *addcount    : Anzahl der bisher aufaddierten Bilder      *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    2.0 (11.09.1998)                                          *
*               2.1 (10.04.1999)                                          *
*   Written by: Georg Dittie, 1994-1999                                   *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_MoveAddfieldToPicture
(
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  long                      *addpuffer,
  long                      *addcount
)
{
  long x, xc, ys, ycs, xstop, ystop, linestep, count;

  xstop = pictureparam->linelength;
  ystop = pictureparam->columnheight * pictureparam->linelength;

  if( picture == NULL || addpuffer == NULL || addcount == NULL )
    return( CCDLIB_NoPointerError );

  linestep = pictureparam->linelength;

  if( pictureparam->colorchannel == CCDLIB_RGBChannelColor )
  {
    xstop *= 3;
    ystop *= 3;

    for(ys = 0, ycs = 0; ys < ystop; ys += xstop, ycs += linestep )
    {
      for(x = 0, xc = 0; x < xstop; x += 3, xc ++ )
      {
        count = addcount[ ycs+xc ];
        if( count > 0 )
        {
          picture[ ys+x   ] = (short)( addpuffer[ ys+x   ] / count );
          picture[ ys+x+1 ] = (short)( addpuffer[ ys+x+1 ] / count );
          picture[ ys+x+2 ] = (short)( addpuffer[ ys+x+2 ] / count );
        }
        else
        {
          picture[ ys+x   ] = (short)( addpuffer[ ys+x   ] );
          picture[ ys+x+1 ] = (short)( addpuffer[ ys+x+1 ] );
          picture[ ys+x+2 ] = (short)( addpuffer[ ys+x+2 ] );
        }
      }
    }
  }
  else
  {
    for(ys = 0; ys < ystop; ys += xstop )
    {
      for(x = 0; x < xstop; x++ )
      {
        if( addcount[ ys+x ] > 0 )
          picture[ ys+x ] = (short)( addpuffer[ ys+x ] / addcount[ ys+x ] );
        else
          picture[ ys+x ] = (short)( addpuffer[ ys+x ] );
      }
    }
  }
  return( CCDLIB_Successful );
}



/*** Bestimmung von Verschiebungskoordinaten ***/

/**************************************************************************
*                                                                         *
*   CCDLIB_GetCenterOfDisk : Routine zur Koordinatenbestimmung des        *
*                            geometrischen Schwerpunkts eines             *
*                            hellen Objekts.                              *
*                                                                         *
*   Parameter:  *picturerect : Arbeitsbereich im Bild                     *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               threshold    : Beruecksichtigungsschwelle Max/Untergr.    *
*               *picpuffer   : 32-Bit-Pointer auf das aktuelle Bild       *
*               *centercoord : Position des Schwerpunkts                  *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !      *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0, (03.10.1997, CCD-Praktikum)                32 Bit    *
*   Written by: Georg Dittie, 1997                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_GetCenterOfDisk
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  CCDLIB_PositionStruct     *centercoord
)
{
  long    pw, xmaxpos, ymaxpos, maxbright,
          xaltpos, yaltpos, xcenter, ycenter, yupos, ydpos, xlpos, xrpos,
          x, y, xstart, xstop, ystart, ystop, ystep, stepcount;

  double  histomaxpart, histoentropy;
  long    histomaximum, histomaxpos, fwhmlevel,
          histolowestpos, histohighestpos, histohighpos, histolowpos,
          *histogramfield;


  histogramfield = (long *)malloc( pictureparam->dynamik * sizeof(long) );

  if( histogramfield == NULL )
    return( CCDLIB_OutOfMemoryError );

  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL )
    return( CCDLIB_NoPointerError );

  if( CCDLIB_RetrieveHistogram( picturerect, pictureparam,
      picture, histogramfield ) != CCDLIB_Successful )
    return( CCDLIB_NoPointerError );

  CCDLIB_EvaluateHistogram(
    histogramfield,
    pictureparam->dynamik,
    CCDLIB_Histo99_8Const,
    0,
    0,
    &histolowestpos,
    &histohighestpos,
    &histolowpos,
    &histohighpos,
    &histomaxpos,
    &histomaximum,
    &histomaxpart,
    &histoentropy );

  fwhmlevel = (long)floor(((double)histohighpos - (double)histomaxpos) * 0.25
            + (double)histomaxpos + 0.5 );

  xstart  = picturerect->xpos;
  ystart  = picturerect->ypos;
  xstop   = picturerect->xpos + picturerect->xsize;
  ystop   = picturerect->ypos + picturerect->ysize;
  ystep   = pictureparam->linelength;

  maxbright = 0;

  if( pictureparam->colorchannel == CCDLIB_RGBChannelColor )
    ystep   *= 3;
    
  xmaxpos = xstart; ymaxpos = ystart;        /* initialize with 1st pixel  */

  for( y=ystart; y<ystop; y++ )
  {
    /*** hellsten Spot suchen (5x5 Pixel) ***/
    for( x=xstart; x<xstop; x++ )
    {
      pw = CCDLIB_GetMeanSample(
        x, y, ystep,
        pictureparam->linelength,
        pictureparam->columnheight,
        pictureparam->colorchannel,
        3, picture );

      if( pw > maxbright )
      {
        xmaxpos = x;
        ymaxpos = y;
        maxbright = pw;
      }
    }
  }

  /*** Iteratives Suchstrahlverfahren ***/
  xcenter = xmaxpos; ycenter = ymaxpos; stepcount = 0;
  do
  {
    xaltpos = xcenter; yaltpos = ycenter;
    for( xlpos=xcenter-3; xlpos>=xstart; xlpos-- )
      if( CCDLIB_GetMeanSample( xlpos, ycenter, ystep,
          pictureparam->linelength,
          pictureparam->columnheight,
          pictureparam->colorchannel,
          5, picture ) < fwhmlevel )    break;
    for( xrpos=xcenter+3; xrpos<xstop; xrpos++ )
      if( CCDLIB_GetMeanSample( xrpos, ycenter, ystep,
          pictureparam->linelength,
          pictureparam->columnheight,
          pictureparam->colorchannel,
          5, picture ) < fwhmlevel )    break;
    for( yupos=ycenter-3; yupos>=ystart; yupos-- )
      if( CCDLIB_GetMeanSample( xcenter, yupos, ystep,
          pictureparam->linelength,
          pictureparam->columnheight,
          pictureparam->colorchannel,
          5, picture ) < fwhmlevel )    break;
    for( ydpos=ycenter+3; ydpos<ystop; ydpos++ )
      if( CCDLIB_GetMeanSample( xcenter, ydpos, ystep,
          pictureparam->linelength,
          pictureparam->columnheight,
          pictureparam->colorchannel,
          5, picture ) < fwhmlevel )    break;

    xcenter = (xlpos+xrpos)/2; ycenter = (yupos+ydpos)/2;
    stepcount++;
  }
  while( (xaltpos != xcenter || yaltpos != ycenter) && stepcount < 10 );

  centercoord->xpos = xcenter;
  centercoord->ypos = ycenter;

  free( histogramfield );

  return( CCDLIB_Successful );
}

/*** Hilfsfunktionen ***/

long CCDLIB_GetMeanSample(
  long  xcoord,
  long  ycoord,
  long  ystep,
  long  picwidth,
  long  picheight,
  enum  CCDLIB_ColorEnum colorchannel,
  long  samplesize,
  short *picture
)
{
  long x, xstart, xstop, ys, ystart, ystop,
       samplehalf, samplenum, greyval=0;


  samplehalf = samplesize >> 1;
  samplenum  = samplesize*samplesize;
  xstart = xcoord-samplehalf;
  xstop  = xcoord+samplehalf;
  ystart = (ycoord-samplehalf)*ystep;
  ystop  = (ycoord+samplehalf)*ystep;
  if( xcoord < samplehalf ||
      ycoord < samplehalf ||
      xcoord >= picwidth-samplehalf ||
      ycoord >= picheight-samplehalf )
    return( (long)picture[ xcoord+ycoord*ystep ] );

  if( colorchannel != CCDLIB_RGBChannelColor )
  {
    for( ys=ystart; ys<ystop; ys+=ystep )
      for( x=xstart; x<xstop; x++ )
        greyval += (long)picture[x+ys];
  }
  else
  {
    for( ys=ystart; ys<ystop; ys+=ystep )
      for( x=xstart; x<xstop; x++ )
        greyval += (long)floor( 0.299 * (double)picture[ ys+x+2 ]
                + 0.587 * (double)picture[ ys+x+1 ]
                + 0.114 * (double)picture[ ys+x ] + 0.5 );
  }

  return( greyval/samplenum );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_GetCenterOfBrightness : Routine zur Koordinatenbestimmung      *
*                                  des Helligkeitsschwerpunkts            *
*                                  in einem Bild.                         *
*                                                                         *
*   Parameter:  *picturerect : Arbeitsbereich im Bild                     *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               threshold    : Beruecksichtigungsschwelle Max/Untergr.    *
*               *picpuffer   : 32-Bit-Pointer auf das aktuelle Bild       *
*               *centercoord : Position des Schwerpunkts                  *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0 (12.05.1996)                                          *
*   Written by: Georg Dittie, 1996                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_GetCenterOfBrightness
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  double                    threshold,
  short                     *picture,
  CCDLIB_PositionStruct     *centercoord
)
{
  double  xweight, yweight, brightsum, xd, yd, pw;
  long    x, ys, xstart, xstop, ystart, ystop, linestep;

  long    histomaximum, histomaxpos;
  double  histomaxpart, histoentropy, histolevel;
  long    histolowestpos, histohighestpos;

  long    *histogramfield;


  histogramfield = (long *)malloc( pictureparam->dynamik * sizeof(long) );
  if( histogramfield == NULL )
  {
    if( histogramfield != NULL ) free( histogramfield );
    return( CCDLIB_OutOfMemoryError );
  }

  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL )
    return( CCDLIB_NoPointerError );

  if( CCDLIB_RetrieveHistogram( picturerect, pictureparam,
      picture, histogramfield ) != CCDLIB_Successful )
    return( CCDLIB_NoPointerError );

  CCDLIB_GetMaximumInHistogram(
    histogramfield,
    pictureparam->dynamik,
    &histolowestpos,
    &histohighestpos,
    &histomaxpos,
    &histomaximum,
    &histomaxpart,
    &histoentropy );

  histolevel = ((double)histohighestpos - (double)histomaxpos) * threshold
             + (double)histomaxpos;

  /*** The picture ist to bright ! **/

  if( histolevel >= (double)histohighestpos-10. )
  {
    free( histogramfield );
    return( CCDLIB_WrongParameterError );
  }

  xstart = picturerect->xpos;
  ystart = picturerect->ypos * pictureparam->linelength;
  xstop  = picturerect->xpos + picturerect->xsize;
  ystop  = (picturerect->ypos + picturerect->ysize) * pictureparam->linelength;
  linestep = pictureparam->linelength;

  xweight   = 0.;
  yweight   = 0.;
  brightsum = 0.;

  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    yd = (double)(picturerect->ypos);
    for( ys = ystart; ys < ystop; ys += linestep )
      {
      xd = (double)xstart;
      for( x = xstart; x < xstop; x++ )
      {
        pw = (double)picture[ ys + x ];
        if( pw >= histolevel )
        {
          xweight   += (xd * pw);
          yweight   += (yd * pw);
          brightsum += pw;
        }
        xd++;
      }
      yd++;
    }
  }
  else
  {
    xstart *= 3;
    xstop  *= 3;
    ystart *= 3;
    ystop  *= 3;
    linestep *= 3;

    /*** Gewichtung nach der ISO-Helligkeitsnorm ***/

    yd = (double)(picturerect->ypos);
    for( ys = ystart; ys < ystop; ys += linestep )
    {
      xd = (double)(picturerect->xpos);
      for( x = xstart; x < xstop; x += 3 )
      {
        pw  = 0.299 * (double)picture[ ys+x+2 ]
            + 0.587 * (double)picture[ ys+x+1 ]
            + 0.114 * (double)picture[ ys+x ];
        if( pw >= histolevel )
        {
          xweight += (xd * pw);
          yweight += (yd * pw);
          brightsum += pw;
        }
        xd++;
      }
      yd++;
    }
  }
  if( brightsum > 0. )
  {
    centercoord->xpos = (long)floor( xweight / brightsum + 0.5 );
    centercoord->ypos = (long)floor( yweight / brightsum + 0.5 );
  }
  else
  {
    centercoord->xpos = pictureparam->linelength/2;
    centercoord->ypos = pictureparam->columnheight/2;
  }

  free( histogramfield );
  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_2DCrossCorrelation : Routine zur Koordinatenbestimmung         *
*                               des Zentralpixels eines Suchmusters       *
*                               in einem Bild.                            *
*                                                                         *
*   Diese Routine realisiert echte 2D-Kreuzkorrelation !                  *
*                                                                         *
*   Parameter:  *searchrect  : Absuchbereich fuer Centerpixel im Bild     *
*               *fitsize     : Groesse des Passmusters                    *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *picture     : 32-Bit-Pointer auf das aktuelle Bild       *
*               *fitpuffer   : 32-Bit-Pointer auf das Suchmuster          *
*               *centercoord : Position des Schwerpunkts                  *
*               *correlation : maximaler Wert der Kreuzkorrelation        *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Das Ergebnis steht in centercoord und correlation                     *
*                                                                         *
*   Version:    1.0 (12.05.1996)                                          *
*   Written by: Georg Dittie, 1996                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_2DCrossCorrelation
(
  CCDLIB_RectangleStruct    *searchrect,
  CCDLIB_RectsizeStruct     *fitsize,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  short                     *fitpuffer,
  CCDLIB_PositionStruct     *centercoord,
  double                    *correlation
)
{
  long    xp, ysp, xr, ysr, xpr, ypr,
          xstart, xstop, ystart, ystop, xrstop, yrstop,
          picrefstep, piclinestep, reflinestep,
          xfithalf, yfithalf, ylinefithalf;

  double  corrval, corrmax, divisor, samplenum;

  long    corrxpos, corrypos;

  double  refgreyval, refgreymean, refgreysqr, refsqrsum,
          picgreyval, picgreymean, picgreysqr, picsqrsum,
          picrefsqr, picrefsum;

  short   *pictureptr;
  short   *fitbufptr;


  CCDLIB_PictureparamStruct fitparam = *pictureparam;

  if( CCDLIB_CheckRectangleValues( searchrect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL )
    return( CCDLIB_NoPointerError );

  fitparam.linelength   = fitsize->xsize;
  fitparam.columnheight = fitsize->ysize;

  xfithalf = fitsize->xsize / 2;
  yfithalf = fitsize->ysize / 2;

  samplenum = (double)(fitsize->xsize/4+1) * (double)(fitsize->ysize/4+1);

  if( (xfithalf%4) != 0 || (yfithalf%4) != 0 )
    return( CCDLIB_WrongParameterError );

  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    pictureptr = picture;
    fitbufptr  = fitpuffer;
  }
  else
  {
    pictureptr = (short *)malloc(
      pictureparam->linelength * pictureparam->columnheight * sizeof(short) );
    fitbufptr = (short *)malloc(
      fitsize->xsize * fitsize->ysize * sizeof(short) );

    if( pictureptr == NULL || fitbufptr == NULL )
    {
      if( pictureptr != NULL ) free( pictureptr );
      if( fitbufptr != NULL ) free( fitbufptr );
      return( CCDLIB_OutOfMemoryError );
    }
    CCDLIB_ExtractLuminance( picture, pictureparam, pictureptr );
    CCDLIB_ExtractLuminance( fitpuffer, &fitparam, fitbufptr );
  }

  xstart = searchrect->xpos;
  ystart = searchrect->ypos;

  if( xstart < xfithalf )
  {
    searchrect->xsize -= (xfithalf - xstart);
    xstart = xfithalf;
  }
  if( ystart < yfithalf )
  {
    searchrect->ysize -= (yfithalf - ystart);
    ystart = yfithalf;
  }

  xstop = xstart + searchrect->xsize;
  ystop = ystart + searchrect->ysize;

  if( xstop >= pictureparam->linelength - xfithalf )
    xstop = pictureparam->linelength - xfithalf - 1;
  if( ystop >= pictureparam->columnheight - yfithalf )
    ystop = pictureparam->columnheight - yfithalf - 1;

  ystart *= pictureparam->linelength;
  ystop  *= pictureparam->linelength;

  yfithalf     *= fitsize->xsize;
  ylinefithalf  = fitsize->ysize/2 * pictureparam->linelength;

  /*** Grauwertsummen fuer die Referenz vorberechnen ***/

  picrefstep  = pictureparam->linelength * 4;
  reflinestep = fitsize->xsize * 4;

  xrstop = fitsize->xsize;
  yrstop = fitsize->xsize * fitsize->ysize;

  refgreymean = 0.;
  for( ysr = 0; ysr <= yrstop; ysr += reflinestep )
  {
    for( xr = 0; xr <= xrstop; xr += 4 )
    {
      refgreymean += (double)fitbufptr[ ysr+xr ];
    }
  }
  refgreymean /= samplenum;

  refsqrsum = 0.;
  for( ysr = 0; ysr <= yrstop; ysr += reflinestep )
  {
    for( xr = 0; xr <= xrstop; xr += 4 )
    {
      refgreyval = (double)fitbufptr[ ysr+xr ] - refgreymean;
      refgreysqr = refgreyval * refgreyval;
      refsqrsum += refgreysqr;
    }
  }

  /*** 1. Iterationsschritt: Suche in 2er Schritten ***/

  corrmax  = -1.;
  corrxpos = -1;
  corrypos = -1;

  piclinestep = pictureparam->linelength * 2;

  for( ysp = ystart; ysp < ystop; ysp += piclinestep )
  {
    for( xp = xstart; xp < xstop; xp += 2 )
    {
      picgreymean = 0.;
      ypr = ysp - ylinefithalf;

      for( ysr = 0; ysr <= yrstop; ysr += reflinestep )
      {
        xpr = xp - xfithalf;
        for( xr = 0; xr <= xrstop; xr += 4 )
        {
          picgreymean += (double)pictureptr[ ypr+xpr ];
          xpr += 4;
        }
        ypr += picrefstep;
      }
      picgreymean /= samplenum;

      picrefsum = 0.;
      picsqrsum = 0.;

      ypr = ysp - ylinefithalf;
      for( ysr = 0; ysr <= yrstop; ysr += reflinestep )
      {
        xpr = xp - xfithalf;
        for( xr = 0; xr <= xrstop; xr += 4 )
        {
          refgreyval = (double)fitbufptr[ ysr+xr ] - refgreymean;
          picgreyval = (double)pictureptr[ ypr+xpr ] - picgreymean;

          picgreysqr = picgreyval*picgreyval;
          picsqrsum += picgreysqr;

          picrefsqr  = picgreyval*refgreyval;
          picrefsum += picrefsqr;

          xpr += 4;
        }
        ypr += picrefstep;
      }

      divisor = refsqrsum * picsqrsum;
      if( divisor > 0. )
        corrval = picrefsum / sqrt( divisor );
      else
        corrval = -1.;

      if( corrval > corrmax )
      {
        corrmax   = corrval;
        corrxpos  = xp;
        corrypos  = ysp;
      }
    }
  }

  /*** Zweiter Iterationsschritt: pixelgenaue Suche ***/

  corrmax = -1.;

  piclinestep = pictureparam->linelength;

  xstart = corrxpos - 2;
  xstop  = corrxpos + 2;
  ystart = corrypos - 2*pictureparam->linelength;
  ystop  = corrypos + 2*pictureparam->linelength;

  for( ysp = ystart; ysp < ystop; ysp += piclinestep )
  {
    for( xp = xstart; xp < xstop; xp++ )
    {
      picgreymean = 0.;
      ypr         = ysp - ylinefithalf;
      for( ysr = 0; ysr <= yrstop; ysr += reflinestep )
      {
        xpr = xp - xfithalf;
        for( xr = 0; xr <= xrstop; xr += 4 )
        {
          picgreymean += (double)pictureptr[ ypr+xpr ];
          xpr         += 4;
        }
        ypr += picrefstep;
      }
      picgreymean /= samplenum;

      picrefsum = 0.;
      picsqrsum = 0.;

      ypr = ysp - ylinefithalf;
      for( ysr = 0; ysr <= yrstop; ysr += reflinestep )
      {
        xpr = xp - xfithalf;
        for( xr = 0; xr <= xrstop; xr += 4 )
        {
          refgreyval = (double)fitbufptr[ ysr+xr ] - refgreymean;
          picgreyval = (double)pictureptr[ ypr+xpr ] - picgreymean;

          picgreysqr = picgreyval*picgreyval;
          picsqrsum += picgreysqr;

          picrefsqr  = picgreyval*refgreyval;
          picrefsum += picrefsqr;

          xpr += 4;
        }
        ypr += picrefstep;
      }

      divisor = refsqrsum * picsqrsum;
      if( divisor > 0. )
        corrval = picrefsum / sqrt( divisor );
      else
        corrval = -1.;

      if( corrval > corrmax )
      {
        corrmax   = corrval;
        corrxpos  = xp;
        corrypos  = ysp;
      }
    }
  }

  centercoord->xpos = corrxpos;
  centercoord->ypos = corrypos/pictureparam->linelength;

  *correlation = corrmax;

  if( pictureparam->colorchannel == CCDLIB_RGBChannelColor )
  {
    free( pictureptr );
    free( fitbufptr );
  }

  return( CCDLIB_Successful );
}



/*** Algorithmen zum Sussmann-Operator ***/

/**************************************************************************
*                                                                         *
*   CCDLIB_CreateMask erzeugt in einem Maskenpuffer eine Bearbeitungs-    *
*   maske fuer Bildverarbeitungsprozeduren an Planetenaufnahmen           *
*                                                                         *
*   Version:    1.0 (20.09.1998 Mariazell)                                *
*   Written by: Georg Dittie, 1998                                        *
*                                                                         *
**************************************************************************/

/** Liebe Leute, Hier wird noch entwickelt .... **/

/*
enum CCDLIB_ErrorEnum CCDLIB_CreateMask(
  char **mask,
  short *picbuffer,
  CCDLIB_PictureparamStruct *picparam,
  double masklevel )
{
  long brightval, buflength;
  long x, y, xs, ys, yd, xstart, xstop, ystart, ystop,
      ystep, ysstep;

  long histomaximum, histomaxpos, fwhmlevel;
  double histomaxpart, histoentropy;
  long histolowestpos, histohighestpos, histohighpos, histolowpos;
  long * histogramfield;
  CCDLIB_RectangleStruct picrect;

  if( picbuffer == NULL )
    return( CCDLIB_NoPointerError );

  if( masklevel > 1. ) masklevel = 1.;
  if( masklevel < 0. ) masklevel = 0.;

  buflength = picparam->linelength*picparam->columnheight;
  if( **mask != NULL ) free( *mask );
  (*mask) = (char *)malloc( buflength * sizeof(char) );
  histogramfield = (long *)malloc( picparam->dynamik * sizeof(long) );

  if( histogramfield == NULL || (*mask) == NULL )
  {
    if( histogramfield != NULL ) free( histogramfield );
    if( (*mask) != NULL ) free( *mask );
    return( CCDLIB_OutOfMemoryError );
  }

  CCDLIB_InitRectangle(
    &picrect, 0, 0, picparam->linelength, picparam->columnheight );
  if( CCDLIB_RetrieveHistogram( &picrect, picparam,
       picbuffer, histogramfield ) != CCDLIB_Successful )
  {
    if( histogramfield != NULL ) free( histogramfield );
    if( (*mask) != NULL ) free( *mask );
    return( CCDLIB_NoPointerError );
  }

  CCDLIB_EvaluateHistogram(
    histogramfield,
    picparam->dynamik,
    CCDLIB_Histo99_8Const,
    0,
    0,
    &histolowestpos,
    &histohighestpos,
    &histolowpos,
    &histohighpos,
    &histomaxpos,
    &histomaximum,
    &histomaxpart,
    &histoentropy );

  fwhmlevel = (long)floor(((double)histohighpos - (double)histomaxpos) * masklevel
           + (double)histomaxpos + 0.5 );

  xstart = picrect.xpos;
  ystart = picrect.ypos;
  xstop  = picrect.xpos+picrect.xsize;
  ystop  = picrect.ypos+picrect.ysize;
  ystart *= picparam->linelength;
  ystop  *= picparam->linelength;
  ystep  = picparam->linelength;

  if( picparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    for( y = ystart; y < ystop; y += ystep )
    {
      for( x = xstart; x < xstop; x++ )
      {
        if( picbuffer[ y+x ] > fwhmlevel )
          (*mask)[ y+x ] = 1;
        else
          (*mask)[ y+x ] = 0;
      }
    }
  }
  else
  {
    ysstep = ystep*3;

    for( yd = ystart, ys = ystart*3;
        yd < ystop;
        yd += ystep, ys += ysstep )
    {
      for( x = xstart, xs = xstart*3;
          x < xstop;
          x++, xs += 3 )
      {
        brightval = (long)floor(0.5+
          0.299 * (double)picbuffer[ ys+xs+2 ] +
          0.587 * (double)picbuffer[ ys+xs+1 ] +
          0.114 * (double)picbuffer[ ys+xs ] );

        if( brightval > fwhmlevel )
          (*mask)[ yd+x ] = 1;
        else
          (*mask)[ yd+x ] = 0;
      }
    }
  }

  free( histogramfield );

  return( CCDLIB_Successful );
}

*/

/**************************************************************************
*                                                                         *
*  CCDLIB_EvaluatePlanetDisk bewertet Planetenaufnahmen nach diversen     *
*  Bewertungskriterien:                                                   *
*                                                                         *
*  Rauschanteil im 5x5 Quadrat ( Standardabweichung )                     *
*  Mittelwert des Sobeloperators in der Maske                             *
*  Maximum des Sobeloperators in der Maske                                *
*  Pruefung auf vollstaendige Abbildung des Planeten                      *
*  Schaerfeindikator nach Dr. Fleischmann                                 *
*  Belichtungsdynamik                                                     *
*                                                                         *
*  Return: TRUE, wenn Minimalbedingungen erfuellt werden                  *
*                                                                         *
*   Version:    1.0 (20.09.1998, Mariazell)                               *
*   Written by: Georg Dittie, 1998                                        *
*                                                                         *
**************************************************************************/

/** Liebe Leute, Hier wird noch entwickelt .... **/

/*
BOOL CCDLIB_EvaluatePlanetDisk(
  long *xmaxpos,
  long *ymaxpos,
  double *sobelmax,
  double *sobelmean,
  double *noiselevel,
  double *sharpindex,
  double *contrastval,
  char *mask,
  short *picbuffer,
  CCDLIB_PictureparamStruct *picparam,
  CCDLIB_RectangleStruct *excluderect,
  double maxnoise,
  double mincontrast )
{
  long x, xs, y, ys, xk, xks, yk, yks, xm, ym, n, i, maskcount,
      buflength, xstart, xstop, ysstart, ysstop, ysstep, ymstep;
  short mean, minimum, maximum;
  double sigma, sigmasum;

  BOOL result = FALSE;

  short *tempbuf;

  short sortarray[ 25 ], statarray[ 25 ];
  const long hindexhalf = 2, windexhalf = 2;
  const char conv2D[5][5] =
  { {0,1,1,1,0},
    {1,1,1,1,1},
    {1,1,1,1,1},
    {1,1,1,1,1},
    {0,1,1,1,0} };
*/
  /* Vorbereitungen */
/*
  buflength = picparam->linelength*picparam->columnheight;
  if( picparam->colorchannel == CCDLIB_RGBChannelColor )
    buflength *= 3;

  tempbuf = (short*)malloc( buflength*sizeof(short) );
  if( tempbuf == NULL )
    return( FALSE );

*/
  /* 1. Schritt Medianfilterung & parallele Rauschanalyse */
/*
  if( picparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    xstart  = 2;
    xstop   = picparam->linelength-2;
    ysstart = 2*picparam->linelength;
    ysstop  = (picparam->columnheight-2)*picparam->linelength;
    ysstep  = picparam->linelength;
    maskcount = 0;
    sigmasum  = 0.;
    for( ys = ysstart; ys < ysstop; ys += ysstep )
    {
      for( x = xstart; x < xstop; x++ )
      {
        if( mask[ ys+x ] == 1 )
        {
          for( yks = -hindexhalf*ysstep, n=0, yk=0;
              yks <= hindexhalf*ysstep;
              yks += ysstep, yk++ )
          {
            for( xks = -windexhalf, xk=0;
                xks <= windexhalf;
                xks ++, xk++ )
            {
              if( conv2D[ yk ][ xk ] != 0 )
              {
                sortarray[ n ] = picbuffer[ x+xks + ys+yks ];
                n++;
              }
            }
          }
          tempbuf[ ys+x ] = (short)(
            CCDLIB_GetMedianFromArray( sortarray, n-1 ) );

          CCDLIB_GetLineStatistics(
            &mean, &sigma, &minimum, &maximum, sortarray, n-1 );

          maskcount++;
          sigmasum += sigma;
        }
        else
        {
          tempbuf[ ys+x ] = 0;
        }
      }
    }
    *noiselevel = sigmasum / (double)maskcount;
  }
  else    
*/
  /* Medianfiltern von RGB-Bildern */
/*
  {
    xstart  = 6;
    xstop   = picparam->linelength*3-6;
    ysstart = 6*picparam->linelength;
    ysstop  = (picparam->columnheight-2)*picparam->linelength*3;
    ysstep  = picparam->linelength*3;
    ymstep  = picparam->linelength;
    maskcount = 0;
    sigmasum  = 0.;
    for( ys = ysstart, ym = ysstart/3;
        ys < ysstop;
        ys += ysstep, ym += ymstep )
    {
      for( x = xstart, xm = xstart/3;
          x < xstop;
          x += 3, xm++ )
      {
        if( mask[ ym+xm ] == 1 )
        {
          for( yks = -hindexhalf*ysstep, n=0, yk=0;
              yks <= hindexhalf*ysstep;
              yks += ysstep, yk++ )
          {
            for( xks = -windexhalf*3, xk=0;
                xks <= windexhalf*3;
                xks +=3, xk++ )
            {
              if( conv2D[ yk ][ xk ] != 0 )
              {
                sortarray[ n ] = picbuffer[ x+xks + ys+yks ];
                n++;
              }
            }
          }
          tempbuf[ ys+x ] = (short)(
            CCDLIB_GetMedianFromArray( sortarray, n-1 ) );

          for( yks = -hindexhalf*ysstep, n=0, yk=0;
              yks <= hindexhalf*ysstep;
              yks += ysstep, yk++ )
          {
            for( xks = -windexhalf*3+1, xk=0;
                xks <= windexhalf*3;
                xks +=3, xk++ )
            {
              if( conv2D[ yk ][ xk ] != 0 )
              {
                sortarray[ n ] = picbuffer[ x+xks + ys+yks ];
                statarray[ n ] =
                  (short)floor(0.5 + 0.587 * (double)picbuffer[ x+xks + ys+yks ]);
                n++;
              }
            }
          }
          tempbuf[ ys+x+1 ] = (short)(
            CCDLIB_GetMedianFromArray( sortarray, n-1 ) );

          for( yks = -hindexhalf*ysstep, n=0, yk=0;
              yks <= hindexhalf*ysstep;
              yks += ysstep, yk++ )
          {
            for( xks = -windexhalf*3+2, xk=0;
                xks <= windexhalf*3;
                xks +=3, xk++ )
            {
              if( conv2D[ yk ][ xk ] != 0 )
              {
                sortarray[ n ] = picbuffer[ x+xks + ys+yks ];
                statarray[ n ] +=
                  (short)floor(0.5 + 0.299 * (double)picbuffer[ x+xks + ys+yks ]);
                n++;
              }
            }
          }
          tempbuf[ ys+x+2 ] = (short)(
            CCDLIB_GetMedianFromArray( sortarray, n-1 ) );
        }
        else
        {
          tempbuf[ ys+x ]   = 0;
          tempbuf[ ys+x+1 ] = 0;
          tempbuf[ ys+x+2 ] = 0;
        }
      }
    }
    *noiselevel = sigmasum / (double)maskcount;
  }
*/
  /*** 2. Schritt: Auswertung des gefilterten Puffers */

   /* Oh wei, gibts hier noch viel zu hacken ... SEUFZ ! ***/
/*
  free( tempbuf );
  return( result );
}

*/

/*** Statistische Auswertung eines einfachen Puffers ****/

void CCDLIB_GetLineStatistics(
  short  *mean,
  double *sigma,
  short  *minimum,
  short  *maximum,
  short  *values,
  long   valnum
)
{
  long    n;

  double  meansum, variancesum, temporary;

  *mean     = 0;
  *minimum  = values[ 0 ];
  *maximum  = values[ 0 ];
  *sigma    = 0.;

  meansum = 0.;
  for( n=0; n<valnum; n++ )
  {
    meansum += (double)values[ n ];
    if( values[ n ] > *maximum ) *maximum = values[ n ];
    if( values[ n ] < *minimum ) *minimum = values[ n ];
  }
  *mean = (short)floor(0.5 + meansum / (double)valnum);

  variancesum = 0.;
  for( n=0; n<valnum; n++ )
  {
    temporary   = (double)( values[ n ] - *mean );
    variancesum += ( temporary * temporary );
  }

  if( variancesum == 0. )
    *sigma = 0.;
  else
    *sigma = sqrt( variancesum / (double)( valnum-1 ) );
}


/**************************************************************************
*                                                                         *
* The maximum and minimum function are not part of the standard           *
* C library! So we have to define them ourselfs, as we need them in the   *
* following function.                                                     *
*                                                                         *
**************************************************************************/

#ifndef max
  int max(int a, int b) {
    return ( a > b ) ? a : b;
  }
#endif

#ifndef min
  int min(int a, int b) {
    return ( a < b ) ? a : b;
  }
#endif


/*** Experimentierroutinen zum Rauschen in Bildern ***/

/**************************************************************************
*                                                                         *
*   CCDLIB_SetNoiseInPicture : Routine zum kuenstlichen Einfuegen von     *
*                              Rauschen in vorhandene Bilder              *
*                              zu Testzwecken                             *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *picture     : 32-Bit-Pointer auf die Bilddaten           *
*               noisetype    : Art des einzufuegenden Rauschens :         *
*                                                                         *
*               CCDLIB_NoiseWhiteConst:          Addiere weisses Rauschen *
*               CCDLIB_NoiseDenseWhiteConst:     Add starkes ws. Rauschen *
*               CCDLIB_NoiseWhiteFlatConst:      Nur weisses Rauschen     *
*               CCDLIB_NoiseDenseWhiteFlatConst: Nur int. weisses Rau.    *
*               CCDLIB_NoisePinkFlatConst:       Reines Rosa Rauschen     *
*               CCDLIB_NoiseBrightConst:         Fuege weisse Pixel ein   *
*               CCDLIB_NoiseDenseBrightConst:    Fuege viele weisse Pixel * 
*               CCDLIB_NoiseDarkConst:           Fuege schwarze Pixel ein *
*               CCDLIB_NoiseDenseDarkConst:      Fuege viele schwarze P.  *
*               CCDLIB_NoiseMixedConst:          Fuege w & s Pixel ein    *
*               CCDLIB_NoiseDenseMixedConst:     Fuege viele w & s Pixel  *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Das Resultatbild steht an derselben Stelle wie das Ausgangsbild.      *
*   ACHTUNG: Das Ausgangsbild wird dabei ueberschrieben !                 *
*                                                                         *
*   Version:    1.0 (05.06.1994)                                          *
*   Written by: Georg Dittie, 1994                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_SetNoiseInPicture
(
  CCDLIB_RectangleStruct * picturerect,
  CCDLIB_PictureparamStruct * pictureparam,
  short * picture,
  enum CCDLIB_NoiseTypeEnum noisetype
)
{
  long  x, ys, ystop;
  short randomnum;
  short zw;
  
  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL )
    return( CCDLIB_NoPicturePointerError );

  srand( time(NULL) );
  ystop = (picturerect->ypos + picturerect->ysize) * pictureparam->linelength;

  for(ys = picturerect->ypos * pictureparam->linelength;
      ys < ystop;
      ys += pictureparam->linelength)
  {
    for(x = picturerect->xpos;
        x < picturerect->xpos + picturerect->xsize;
        x++ )
    {
      randomnum = (short)rand();
    
      switch( noisetype )
      {
        case CCDLIB_NoiseWhiteConst:
          zw  = (short)(picture[ x+ys ] + ((randomnum & (short)(pictureparam->dynamik/4))
              - (short)(pictureparam->dynamik/8) ) );
          picture[ x+ys ] = (short)(max(min( zw,(short)(pictureparam->dynamik-1)), 0) );
          break;

        case CCDLIB_NoiseDenseWhiteConst:
          zw  = (short)(picture[ x+ys ] + ((randomnum & (short)(pictureparam->dynamik/2) )
              - (short)(pictureparam->dynamik/4) ) );
          picture[ x+ys ] = (short)(max(min( zw,(short)(pictureparam->dynamik-1) ), 0) );
          break;

        case CCDLIB_NoiseWhiteFlatConst:
          picture[ x+ys ] = (short)( (randomnum & (short)(pictureparam->dynamik/4) )
                          + (short)(pictureparam->dynamik/8*3) );
          break;

        case CCDLIB_NoiseDenseWhiteFlatConst:
          picture[ x+ys ] = (short)( (randomnum & (short)pictureparam->dynamik ) );
          break;

        case CCDLIB_NoisePinkFlatConst:
          picture[ x+ys]  = (short)CCDLIB_GaussDevRandom(
                              (short)(pictureparam->dynamik/2-1),
                              (short)(pictureparam->dynamik/2-1) );
          break;

        case CCDLIB_NoiseBrightConst:
          if( randomnum >= 0x7eff )
            picture[ x+ys ] = (short)(pictureparam->dynamik-1);
          break;

        case CCDLIB_NoiseDenseBrightConst:
          if( randomnum >= 0x77ff )
            picture[ x+ys ] = (short)(pictureparam->dynamik-1);
          break;
    
        case CCDLIB_NoiseDarkConst:
          if( randomnum <= 0x100 )
            picture[ x+ys ] = 0;
          break;
    
        case CCDLIB_NoiseDenseDarkConst:
          if( randomnum <= 0x800 )
            picture[ x+ys ] = 0;
          break;

        case CCDLIB_NoiseMixedConst:
          if( randomnum >= 0x7eff )
            picture[ x+ys ] = (short)(pictureparam->dynamik-1);
          else if( randomnum <= 0x100 )
            picture[ x+ys ] = 0;
          break;

        case CCDLIB_NoiseDenseMixedConst:
          if( randomnum >= 0x77ff )
            picture[ x+ys ] = (short)(pictureparam->dynamik-1);
          else if( randomnum <= 0x100 )
            picture[ x+ys ] = 0;
          break;
      }
    }
  }

  return( CCDLIB_Successful );
}


short CCDLIB_GaussDevRandom
(
  short mean,
  short dynamik
)
{
  static BOOL   iset = FALSE;
  static double gset;
  double        fac, rsq, v1, v2, res;

  if( !iset )
  {
    do
    {
      v1 = 2. * (double)rand() / (double)RAND_MAX - 1.;
      v2 = 2. * (double)rand() / (double)RAND_MAX - 1.;
      rsq = v1*v1 + v2*v2;
    } while ( rsq >= 1. || rsq == 0. );

    fac  = sqrt( -2. * log(rsq) / rsq );
    gset = v1*fac;
    iset = TRUE;
    res  = v2*fac;
  }
  else
  {
    iset = FALSE;
    gset = 0.;
    res  = gset;
  }

  return( (short)(mean + (short)( res * (double)(dynamik) + 0.5 )) );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_SeparatedMaskedFilterOp : Universelle Filteroperation fuer     *
*               separierbare Operationen. Die Maske ist quadratisch       *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *sourcepic   : 32-Bit-Pointer auf die Quellbilddaten      *
*               *destpic     : 32-Bit-Pointer auf die Zielbilddaten       *
*               maskdiameter : Durchmesser des Filtermaskenkreis          *
*               rectdiameter : Durchmesser des Rechteckanteils            *
*               filtdiameter : Durchmesser des Bandpassfilters            *
*               CCDLIB_OperationTypeEnum : Art der Filteroperation        *
*               CCDLIB_BandpassFiltEnum : Art der Bandpassfilterung       *
*                                                                         *
*               ACHTUNG: *diameter darf nur ungerade Werte annehmen!      *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0 (08.10.1998)                                          *
*   Written by: Georg Dittie, 1998                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_SeparatedMaskedFilterOp
(
  CCDLIB_RectangleStruct        *picturerect,
  CCDLIB_PictureparamStruct     *pictureparam,
  short                         *sourcepic,
  short                         *destpic,
  long                          maskdiameter,
  long                          rectdiameter,
  long                          filtdiameter,
  double                        effiency,
  enum CCDLIB_OperationTypeEnum operationtype,
  enum CCDLIB_BandpassFiltEnum  filtertype
)
{
  double  *mask, *line, *col;
  double  temporary, tempb, tempg, tempr, dynamikmone;

  long    x, xk, y, yk, ys, xc,
          xstart, xstartk, xstop, xstopk,
          ystart, ystartk, ystop, ystopk,
          ystep, ystartc, ystopc,
          indexhalf, xindexhalf;


  if( sourcepic == NULL || destpic == NULL )
    return( CCDLIB_NoPicturePointerError );

  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( effiency < 0.01 || effiency > 100. )
  {
    return( CCDLIB_CopyRegion(
      destpic, sourcepic, pictureparam, picturerect ) );
  }
  dynamikmone = (double)(pictureparam->dynamik - 1);
  indexhalf = maskdiameter/2;

  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    ystep   = pictureparam->linelength;
    xstart  = (long)(picturerect->xpos);
    xstartk = (long)(picturerect->xpos+indexhalf);
    ystart  = (long)(picturerect->ypos)*ystep;
    ystartk = (long)(picturerect->ypos+indexhalf);
    xstop   = (long)(picturerect->xpos+picturerect->xsize);
    xstopk  = (long)(picturerect->xpos+picturerect->xsize-indexhalf);
    ystop   = (long)(picturerect->ypos+picturerect->ysize)*ystep;
    ystopk  = (long)(picturerect->ypos+picturerect->ysize-indexhalf);
    ystartc = (long)(picturerect->ypos);
    ystopc  = (long)(picturerect->ypos+picturerect->ysize);
    xindexhalf = indexhalf;
  }
  else
  {
    ystep   = pictureparam->linelength*3;
    xstart  = (long)(picturerect->xpos)*3;
    xstartk = (long)(picturerect->xpos+indexhalf)*3;
    ystart  = (long)(picturerect->ypos)*ystep;
    ystartk = (long)(picturerect->ypos+indexhalf);
    xstop   = (long)(picturerect->xpos+picturerect->xsize)*3;
    xstopk  = (long)(picturerect->xpos+picturerect->xsize-indexhalf)*3;
    ystop   = (long)(picturerect->ypos+picturerect->ysize)*ystep;
    ystopk  = (long)(picturerect->ypos+picturerect->ysize-indexhalf);
    ystartc = (long)(picturerect->ypos);
    ystopc  = (long)(picturerect->ypos+picturerect->ysize);
    xindexhalf = indexhalf*3;
  }

  line = ( double * )malloc( ystep * sizeof( double ) );
  col  = ( double * )malloc( pictureparam->columnheight * sizeof( double ) );
  mask = ( double * )malloc( maskdiameter * sizeof( double ) );

  if( line == NULL || col == NULL || mask == NULL )
  {
    if( line != NULL )  free( line );
    if( col  != NULL )  free( col );
    if( mask != NULL )  free( mask );

    return( CCDLIB_OutOfMemoryError );
  }

  CCDLIB_GetSeparableMask(
    mask,
    maskdiameter,
    rectdiameter,
    filtdiameter,
    effiency,
    operationtype,
    filtertype );

  for( ys = ystart; ys < ystop; ys += ystep )
  {

    for( x = xstart; x < xstop; x++ )
      line[ x ] = (double)sourcepic[ x+ys ];

    if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
    {
      for( x = xstartk; x < xstopk; x++ )
      {
        temporary = 0.;
        for( xk = -xindexhalf; xk <= xindexhalf; xk++ )
        {
          temporary += (line[ x+xk ] * mask[ xk + indexhalf ]);
        }
        destpic[ x+ys ] = (short)floor(
          max( 0, min( dynamikmone, temporary ) ) + 0.5 );
      }
    }
    else
    {
      for( x = xstartk; x < xstopk; x+=3 )
      {
        tempb=0.; tempg=0.; tempr=0.;
        for( xk = -xindexhalf, xc=0; xk <= xindexhalf; xk += 3, xc++ )
        {
          tempb += (line[ x+xk   ] * mask[ xc ]);
          tempg += (line[ x+xk+1 ] * mask[ xc ]);
          tempr += (line[ x+xk+2 ] * mask[ xc ]);
        }
        destpic[ x+ys ] = (short)floor(
          max( 0, min( dynamikmone, tempb ) ) + 0.5 );
        destpic[ x+ys+1 ] = (short)floor(
          max( 0, min( dynamikmone, tempg ) ) + 0.5 );
        destpic[ x+ys+2 ] = (short)floor(
          max( 0, min( dynamikmone, tempr ) ) + 0.5 );
      }
    }
  }

  for( x = xstart; x < xstop; x++ )
  {
    for( y=ystartc, ys=ystart; y<ystopc; y++, ys+=ystep )
      col[ y ] = (double)destpic[ x+ys ];

    for( y = ystartk, ys = ystartk*ystep; y < ystopk; y++, ys += ystep )
    {
      temporary = 0.;
      for( yk = -indexhalf; yk <= indexhalf; yk++ )
      {
        temporary += (col[ y+yk ] * mask[ yk + indexhalf ]);
      }
      destpic[ x+ys ] = (short)floor(
        max( 0, min( dynamikmone, temporary ) ) + 0.5 );
    }
  }

  free( line );
  free( col );
  free( mask );

  return( CCDLIB_Successful );
}

void CCDLIB_GetSeparableMask(
  double                        *mask,
  long                          maskdiameter,
  long                          rectdiameter,
  long                          filtdiameter,
  double                        effiency,
  enum CCDLIB_OperationTypeEnum operationtype,
  enum CCDLIB_BandpassFiltEnum  filtertype
)
{
  long    x, xs, filthalf, filtdif, indexhalf;

  double  masksum, maskwidth, divfaktor, centralval, radius,
          maskradius, rectradius, filtradius, filtmasksum, filtfactor,
          arg, rectant;
  double  *filtmask;


  indexhalf  = maskdiameter/2;

  filtmask   = NULL;
  maskradius = (double)maskdiameter / 2.;
  filtradius = (double)filtdiameter / 2.;
  rectradius = (double)rectdiameter / 2.;
  masksum    = 0.;
  maskwidth  = 3./(double)(indexhalf);

  if( filtertype != CCDLIB_NoBandpassConst )
  {
    filtmask = (double*)malloc(filtdiameter*sizeof(double));
    if( filtmask == NULL )
    {
      filtertype = CCDLIB_NoBandpassConst;
    }
    else
    {
      filthalf  = filtdiameter / 2;
      filtdif   = indexhalf-filthalf;
      rectant   = filtradius/3.;
      for( x=0; x<filtdiameter; x++ )
      {
        radius      = (double)(x-filthalf);
        filtmasksum = 0.;
        switch( operationtype )
        {
        case CCDLIB_FiltRectangleConst:
          filtmask[ x ] = 1.;
          break;
        case CCDLIB_FiltTriangleConst:
          filtmask[ x ] = radius/filtradius;
          break;
        case CCDLIB_FiltGaussConst:
          filtmask[ x ] = -exp( -(radius*radius)/rectant );
          break;
        default:
          break; /* ??? This function has to return an error code! */
        }
        filtmasksum += fabs( filtmask[ x ] );
      }
    }
  }

  rectant = (maskradius-rectradius)/3.;
  for( x=0; x<maskdiameter; x++ )
  {
    radius = (double)(x-indexhalf);
    switch( operationtype )
    {
      case CCDLIB_SharpGaussConst:
        mask[ x ] = -exp( -(radius*radius)/(maskradius/3.) );
        break;
      case CCDLIB_SharpRectangleConst:
        mask[ x ] = -1.;
        break;
      case CCDLIB_SharpTriangleConst:
        mask[ x ] = -radius/(double)indexhalf;
        break;
      case CCDLIB_SharpGaussRectConst:
        if( radius < rectradius )
          mask[ x ] = -1.;
        else
        {
          arg = radius-rectradius;
          mask[ x ] = -exp( -(arg*arg)/rectant );
        }
        break;
      case CCDLIB_MeanValueConst:
        mask[ x ] = 1.;
        break;
      case CCDLIB_LorentzMaskConst:
        mask[ x ] = -1./(radius*radius*maskwidth+1.);
        break;
      default:
        break; /* ??? This function has to return an error code! */
    }
    masksum += fabs( mask[ x ] );
  }

  switch( operationtype )
  {
    case CCDLIB_SharpGaussConst:
    case CCDLIB_SharpRectangleConst:
    case CCDLIB_LorentzMaskConst:
    case CCDLIB_SharpTriangleConst:
    case CCDLIB_SharpGaussRectConst:
      if( filtertype == CCDLIB_NoBandpassConst )
      {
        centralval = masksum * (1. + 1./effiency);
        divfaktor = centralval - masksum;
        mask[ indexhalf ] += centralval;
      }
      else
      {
        filtfactor = masksum/filtmasksum * (1. + 1./effiency);
        divfaktor  = filtmasksum * filtfactor - masksum;
        for( x=0, xs=filtdif; x<maskdiameter; x++, xs++ )
        {
          mask[ xs ] += filtmask[ x ]*filtfactor;
        }
      }
      break;
    default:
      divfaktor = masksum;
      break;
  }

  for( x=0; x<maskdiameter; x++ )
    mask[ x ] /= divfaktor;

  if( filtmask != NULL ) free( filtmask );
}

/**************************************************************************
*                                                                         *
*  CCDLIB_MaskedFilterOperation : Universelle Filteroperation fuer  *
*               nicht separierbare Operationen. Die Maske ist echt        *
*               kreisfoermig !                                            *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*   *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *sourcepic   : 32-Bit-Pointer auf die Quellbilddaten      *
*               *destpic     : 32-Bit-Pointer auf die Zielbilddaten       *
*               maskdiameter : Durchmesser des Filtermaskenkreis          *
*               rectdiameter : Durchmesser des Rechteckanteils            *
*               filtdiameter : Durchmesser des Bandpassfilters            *
*               CCDLIB_OperationTypeEnum : Art der Filteroperation        *
*               CCDLIB_BandpassFiltEnum : Art der Bandpassfilterung       *
*                                                                         *
*               ACHTUNG: maskdiameter darf nur ungerade Werte annehmen!   *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.2 (08.10.1998)                                          *
*   Written by: Georg Dittie, 1994-1998                                   *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_MaskedFilterOperation
(
  CCDLIB_RectangleStruct        *picturerect,
  CCDLIB_PictureparamStruct     *pictureparam,
  short                         *sourcepic,
  short                         *destpic,
  long                          maskdiameter,
  long                          rectdiameter,
  long                          filtdiameter,
  double                        effiency,
  enum CCDLIB_OperationTypeEnum operationtype,
  enum CCDLIB_BandpassFiltEnum  filtertype
)
{
  long    x, xk, yk, ys, xks, yks, ystep, i, n,
          xstart, x1stop, ystart, y1stop,
          index, indexhalf, indexturn, bufferindex, xindexhalf, yindexhalf;

  double  temporary, dynamikmone, dynamikhalf, tempb, tempg, tempr;

  double  **mask;
  short   sortarray[ CCDLIB_ConvolutionWidthConst * CCDLIB_ConvolutionWidthConst ];

  short   *bufferline[ CCDLIB_ConvolutionWidthConst ];


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( sourcepic == NULL || destpic == NULL )
    return( CCDLIB_NoPicturePointerError );

  if( effiency < 0.01 || effiency > 100. )
  {
    return( CCDLIB_CopyRegion(
      destpic, sourcepic, pictureparam, picturerect ) );
  }

  if( maskdiameter > CCDLIB_ConvolutionWidthConst )
    return( CCDLIB_WrongParameterError );

  if( operationtype >= CCDLIB_OpNGradientConst )
    maskdiameter = 3;

  dynamikmone = (double)(pictureparam->dynamik - 1);
  dynamikhalf = (double)(pictureparam->dynamik)/2.;

  indexhalf  = maskdiameter / 2;
  indexturn  = indexhalf + 1;

  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    ystep   = pictureparam->linelength;
    xstart  = (long)(picturerect->xpos+indexhalf);
    ystart  = (long)(picturerect->ypos+indexhalf) * ystep;
    x1stop  = (long)(picturerect->xpos+picturerect->xsize-indexhalf);
    y1stop  = (long)(picturerect->ypos+picturerect->ysize-indexhalf) * ystep;
    xindexhalf = indexhalf;
    yindexhalf = indexhalf*ystep;
  }
  else
  {
    ystep   = pictureparam->linelength*3;
    xstart  = (long)(picturerect->xpos+indexhalf)*3;
    ystart  = (long)(picturerect->ypos+indexhalf)*ystep;
    x1stop  = (long)(picturerect->xpos+picturerect->xsize-indexhalf)*3;
    y1stop  = (long)(picturerect->ypos+picturerect->ysize-indexhalf)*ystep;
    xindexhalf = indexhalf*3;
    yindexhalf = indexhalf*ystep;
  }

  bufferline[ 0 ] = (short *)malloc(ystep* sizeof(short)*maskdiameter );

  if( bufferline[ 0 ] == NULL )
    return( CCDLIB_OutOfMemoryError );

  for( i=1; i<maskdiameter; i++ )
    bufferline[ i ] = bufferline[ 0 ] + i*ystep;

  if( !CCDLIB_Matrixalloc( &mask, maskdiameter ) )
  {
    free( bufferline[0] );
    return( CCDLIB_OutOfMemoryError );
  }

  CCDLIB_GetNotSeparableMask(
    mask, maskdiameter, rectdiameter, filtdiameter,
    effiency, operationtype, filtertype );

  index = 0;

  for( ys = ystart; ys < y1stop; ys += ystep )
  {
    bufferindex = index % indexturn;

#ifdef _PROGRESS_
    progresscounter++;
#endif

    if( index >= indexturn )
    {
      for( x = xstart; x < x1stop; x++ )
        destpic[ ys - indexturn*ystep + x ] = bufferline[ bufferindex ][ x ];
    }

    if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
    {
      for( x = xstart; x < x1stop; x++ )
      {
        switch( operationtype )
        {
          case CCDLIB_SharpRectangleConst:
          case CCDLIB_SharpTriangleConst:
          case CCDLIB_SharpGaussRectConst:
          case CCDLIB_MeanValueConst:
          case CCDLIB_LorentzMaskConst:
            temporary= 0.;
            for( yks = -yindexhalf, yk=0;
                 yks <= yindexhalf;
                 yks += ystep, yk++ )
            {
              for( xks = -xindexhalf, xk=0;
                   xks <= xindexhalf;
                   xks++, xk++ )
              {
                if( mask[ yk ][ xk ] != 0 )
                  temporary += (double)sourcepic[ x+xks + ys+yks ] * mask[ yk ][ xk ];
              }
            }
            bufferline[ bufferindex ][ x ] = (short)(
              max( 0, min( dynamikmone, temporary ) ) );
            break;

          case CCDLIB_OpNGradientConst:
          case CCDLIB_OpNEGradientConst:
          case CCDLIB_OpEGradientConst:
          case CCDLIB_OpSEGradientConst:
          case CCDLIB_OpSGradientConst:
          case CCDLIB_OpSWGradientConst:
          case CCDLIB_OpWGradientConst:
          case CCDLIB_OpNWGradientConst:
            temporary= 0.;
            for( yks = -yindexhalf, yk=0;
                 yks <= yindexhalf;
                 yks += ystep, yk++ )
            {
              for( xks = -xindexhalf, xk=0;
                   xks <= xindexhalf;
                   xks++, xk++ )
              {
                if( mask[ yk ][ xk ] != 0 )
                  temporary += (double)sourcepic[ x+xks + ys+yks ]*mask[ yk ][ xk ];
              }
            }
            bufferline[ bufferindex ][ x ] = (short)(
              max( 0, min( dynamikmone, temporary+dynamikhalf ) ) );
            break;

          case CCDLIB_MaskedMedianConst:
            for( yks = -yindexhalf, n=0, yk=0;
                 yks <= yindexhalf;
                 yks += ystep, yk++ )
            {
              for( xks = -xindexhalf, xk=0;
                   xks <= xindexhalf;
                   xks++, xk++ )
              {
                if( mask[ yk ][ xk ] != 0 )
                {
                  sortarray[ n ] = (short)sourcepic[ x+xks + ys+yks ];
                  n++;
                }
              }
            }
            bufferline[ bufferindex ][ x ] = (short)(
              CCDLIB_GetMedianFromArray( sortarray, n-1 ) );
            break;
          default:
            return( CCDLIB_WrongParameterError );
            break;
        }
      }
    }
    else  /** RGB-Bilder maskieren **/
    {
      for( x = xstart; x < x1stop; x+=3 )
      {
        switch( operationtype )
        {
          case CCDLIB_SharpRectangleConst:
          case CCDLIB_SharpTriangleConst:
          case CCDLIB_SharpGaussRectConst:
          case CCDLIB_MeanValueConst:
          case CCDLIB_LorentzMaskConst:
            tempb=0.; tempg=0.; tempr=0.;
            for( yks = -yindexhalf, yk=0;
                 yks <= yindexhalf;
                 yks += ystep, yk++ )
            {
              for( xks = -xindexhalf, xk=0;
                   xks <= xindexhalf;
                   xks+=3, xk++ )
              {
                if( mask[ yk ][ xk ] != 0 )
                {
                  tempb += (double)sourcepic[ x+xks   +ys+yks ]*mask[yk][xk];
                  tempg += (double)sourcepic[ x+xks+1 +ys+yks ]*mask[yk][xk];
                  tempr += (double)sourcepic[ x+xks+2 +ys+yks ]*mask[yk][xk];
                }
              }
            }
            bufferline[ bufferindex ][ x ] = (short)(
              max( 0, min( dynamikmone, tempb ) ) );
            bufferline[ bufferindex ][ x+1 ] = (short)(
              max( 0, min( dynamikmone, tempg ) ) );
            bufferline[ bufferindex ][ x+2 ] = (short)(
              max( 0, min( dynamikmone, tempr ) ) );
            break;
  
          case CCDLIB_OpNGradientConst:
          case CCDLIB_OpNEGradientConst:
          case CCDLIB_OpEGradientConst:
          case CCDLIB_OpSEGradientConst:
          case CCDLIB_OpSGradientConst:
          case CCDLIB_OpSWGradientConst:
          case CCDLIB_OpWGradientConst:
          case CCDLIB_OpNWGradientConst:
            tempb=0.; tempg=0.; tempr=0.;
            for( yks = -yindexhalf, yk=0;
                 yks <= yindexhalf;
                 yks += ystep, yk++ )
            {
              for( xks = -xindexhalf, xk=0;
                   xks <= xindexhalf;
                   xks+=3, xk++ )
              {
                if( mask[ yk ][ xk ] != 0 )
                {
                  tempb += (double)sourcepic[ x+xks   +ys+yks ]*mask[yk][xk];
                  tempg += (double)sourcepic[ x+xks+1 +ys+yks ]*mask[yk][xk];
                  tempr += (double)sourcepic[ x+xks+2 +ys+yks ]*mask[yk][xk];
                }
              }
            }
            bufferline[ bufferindex ][ x ] = (short)(
              max( 0, min( dynamikmone, tempb+dynamikhalf ) ) );
            bufferline[ bufferindex ][ x+1 ] = (short)(
              max( 0, min( dynamikmone, tempg+dynamikhalf ) ) );
            bufferline[ bufferindex ][ x+2 ] = (short)(
              max( 0, min( dynamikmone, tempr+dynamikhalf ) ) );
            break;
  
          case CCDLIB_MaskedMedianConst:
            for( yks = -yindexhalf, n=0, yk=0;
                 yks <= yindexhalf;
                 yks += ystep, yk++ )
            {
              for( xks = -xindexhalf, xk=0;
                   xks <= xindexhalf;
                   xks += 3, xk++ )
              {
                if( mask[ yk ][ xk ] != 0 )
                {
                  sortarray[ n ] = (short)sourcepic[ x+xks + ys+yks ];
                  n++;
                }
              }
            }
            bufferline[ bufferindex ][ x ] = (short)(
              CCDLIB_GetMedianFromArray( sortarray, n-1 ) );
  
            for( yks = -yindexhalf, n=0, yk=0;
                 yks <= yindexhalf;
                 yks += ystep, yk++ )
            {
              for( xks = -xindexhalf+1, xk=0;
                   xks <= xindexhalf+1;
                   xks += 3, xk++ )
              {
                if( mask[ yk ][ xk ] != 0 )
                {
                  sortarray[ n ] = (short)sourcepic[ x+xks + ys+yks ];
                  n++;
                }
              }
            }
            bufferline[ bufferindex ][ x+1 ] = (short)(
              CCDLIB_GetMedianFromArray( sortarray, n-1 ) );
  
            for( yks = -yindexhalf, n=0, yk=0;
                 yks <= yindexhalf;
                 yks += ystep, yk++ )
            {
              for( xks = -xindexhalf+2, xk=0;
                   xks <= xindexhalf+2;
                   xks += 3, xk++ )
              {
                if( mask[ yk ][ xk ] != 0 )
                {
                  sortarray[ n ] = (short)sourcepic[ x+xks + ys+yks ];
                  n++;
                }
              }
            }
            bufferline[ bufferindex ][ x+2 ] = (short)(
              CCDLIB_GetMedianFromArray( sortarray, n-1 ) );
            break;
          default:
            return( CCDLIB_WrongParameterError );
            break;
        }
      }
    }
    index++;
  }

  for( ys = y1stop; ys < y1stop + ystep; ys += ystep)
  {
    for( x = xstart; x < x1stop; x++ )
      destpic[ ys - indexturn*ystep + x ] =
        bufferline[ index%indexturn ][ x ];
    index++;
  }
  free( bufferline[ 0 ] );
  CCDLIB_Matrixfree( mask );

  return( CCDLIB_Successful );
}

void CCDLIB_GetNotSeparableMask(
  double                        **mask,
  long                          maskdiameter,
  long                          rectdiameter,
  long                          filtdiameter,
  double                        effiency,
  enum CCDLIB_OperationTypeEnum operationtype,
  enum CCDLIB_BandpassFiltEnum  filtertype
)
{
  long    x, y, xs, ys, xsqr, ysqr, filthalf, filtdif, indexhalf;

  double  masksum, maskwidth, divfaktor, centralval, radius,
          maskradius, rectradius, filtradius, filtmasksum, filtfactor,
          arg, rectant;
  double  **filtmask;


  indexhalf = maskdiameter / 2;

  filtmask    = NULL;
  maskradius  = (double)maskdiameter / 2.;
  filtradius  = (double)filtdiameter / 2.;
  rectradius  = (double)rectdiameter / 2.;
  maskwidth   = 9./(double)(indexhalf*indexhalf);
  masksum     = 0.;

  if( operationtype < CCDLIB_OpNGradientConst )
  {
    if( filtertype != CCDLIB_NoBandpassConst )
    {
      if( !CCDLIB_Matrixalloc( &filtmask, filtdiameter ) )
      {
        filtertype = CCDLIB_NoBandpassConst;
      }
      else
      {
        filthalf    = filtdiameter / 2;
        filtdif     = indexhalf-filthalf;
        rectant     = filtradius/3.;
        filtmasksum = 0.;
        for( y=0; y<filtdiameter; y++ )
        {
          ysqr = y-filthalf;
          ysqr = ysqr*ysqr;

          for( x=0; x<filtdiameter; x++ )
          {
            xsqr    = x-filthalf;
            xsqr    = xsqr*xsqr;
            radius  = sqrt( (double)(xsqr+ysqr) );
            if( radius <= filtradius )
            {
              switch( operationtype )
              {
                case CCDLIB_FiltRectangleConst:
                  filtmask[ y ][ x ] = 1.;
                  break;
                case CCDLIB_FiltTriangleConst:
                  filtmask[ y ][ x ] = radius/filtradius;
                  break;
                case CCDLIB_FiltGaussConst:
                  filtmask[ y ][ x ] = -exp( -(radius*radius)/rectant );
                  break;
                default:
                  break; /* ??? return an error value here */
              }
              filtmasksum += fabs( filtmask[ y ][ x ] );
            }
            else
              filtmask[ y ][ x ] = 0.;
          }
        }
      }
    }

    rectant = (maskradius-rectradius)/3.;
    for( y=0; y<maskdiameter; y++ )
    {
      ysqr = y-indexhalf;
      ysqr = ysqr*ysqr;

      for( x=0; x<maskdiameter; x++ )
      {
        xsqr    = x-indexhalf;
        xsqr    = xsqr*xsqr;
        radius  = sqrt( (double)(xsqr+ysqr) );

        if( radius <= maskradius )
        {
          switch( operationtype )
          {
            case CCDLIB_SharpRectangleConst:
              mask[ y ][ x ] = -1.;
              break;
            case CCDLIB_SharpTriangleConst:
              mask[ y ][ x ] = -radius/(double)indexhalf;
              break;
            case CCDLIB_SharpGaussRectConst:
              if( radius < rectradius )
                mask[ y ][ x ] = -1.;
              else
              {
                arg             = radius-rectradius;
                mask[ y ][ x ]  = -exp( -(arg*arg)/rectant );
              }
              break;
            case CCDLIB_MeanValueConst:
            case CCDLIB_MaskedMedianConst:
              mask[ y ][ x ] = 1.;
              break;
            case CCDLIB_LorentzMaskConst:
              mask[ y ][ x ] = -1./((double)(xsqr*ysqr)*maskwidth+1.);
              break;
            default:
              break; /* ??? This function has to return an error code! */
          }
          masksum += fabs( mask[ y ][ x ] );
        }
        else
          mask[ y ][ x ] = 0.;
      }
    }

    switch( operationtype )
    {
      case CCDLIB_SharpRectangleConst:
      case CCDLIB_LorentzMaskConst:
      case CCDLIB_SharpTriangleConst:
      case CCDLIB_SharpGaussRectConst:
        if( filtertype == CCDLIB_NoBandpassConst )
        {
          centralval  = masksum * (1. + 1./effiency);
          divfaktor   = centralval - masksum;
          mask[ indexhalf ][ indexhalf ] += centralval;
        }
        else
        {
          filtfactor = masksum/filtmasksum * (1. + 1./effiency);
          for( y=0, ys=filtdif; y<filtdiameter; y++, ys++ )
          {
            for( x=0, xs=filtdif; x<filtdiameter; x++, xs++ )
            {
              mask[ ys ][ xs ] += filtmask[ y ][ x ]*filtfactor;
            }
          }
          divfaktor = 0;
          for( y=0; y<maskdiameter; y++ )
          {
            for( x=0; x<maskdiameter; x++ )
            {
              divfaktor += mask[ y ][ x ];
            }
          }
        }
        break;
      default:
        divfaktor = masksum;
        break;
    }

    for( y=0; y<maskdiameter; y++ )
    {
      for( x=0; x<maskdiameter; x++ )
      {
        mask[ y ][ x ] /= divfaktor;
      }
    }
  }
  else
  {
    switch( operationtype )
    {
    case CCDLIB_OpNGradientConst:
      mask[ 0 ][ 0 ] = 1.;  mask[ 0 ][ 1 ] = 1.;  mask[ 0 ][ 2 ] = 1.;
      mask[ 1 ][ 0 ] = 0.;  mask[ 1 ][ 1 ] = 0.;  mask[ 1 ][ 2 ] = 0.;
      mask[ 2 ][ 0 ] = -1.; mask[ 2 ][ 1 ] = -1.; mask[ 2 ][ 2 ] = -1.;
      break;
    case CCDLIB_OpNEGradientConst:
      mask[ 0 ][ 0 ] = 0.;  mask[ 0 ][ 1 ] = 1.;  mask[ 0 ][ 2 ] = 1.;
      mask[ 1 ][ 0 ] = -1.; mask[ 1 ][ 1 ] = 0.;  mask[ 1 ][ 2 ] = 1.;
      mask[ 2 ][ 0 ] = -1.; mask[ 2 ][ 1 ] = -1.; mask[ 2 ][ 2 ] = 0.;
      break;
    case CCDLIB_OpEGradientConst:
      mask[ 0 ][ 0 ] = -1.; mask[ 0 ][ 1 ] = 0.;  mask[ 0 ][ 2 ] = 1.;
      mask[ 1 ][ 0 ] = -1.; mask[ 1 ][ 1 ] = 0.;  mask[ 1 ][ 2 ] = 1.;
      mask[ 2 ][ 0 ] = -1.; mask[ 2 ][ 1 ] = 0.;  mask[ 2 ][ 2 ] = 1.;
      break;
    case CCDLIB_OpSEGradientConst:
      mask[ 0 ][ 0 ] = -1.; mask[ 0 ][ 1 ] = -1.; mask[ 0 ][ 2 ] = 0.;
      mask[ 1 ][ 0 ] = -1.; mask[ 1 ][ 1 ] = 0.;  mask[ 1 ][ 2 ] = 1.;
      mask[ 2 ][ 0 ] = 0.;  mask[ 2 ][ 1 ] = 1.;  mask[ 2 ][ 2 ] = 1.;
      break;
    case CCDLIB_OpSGradientConst:
      mask[ 0 ][ 0 ] = -1.; mask[ 0 ][ 1 ] = -1.; mask[ 0 ][ 2 ] = -1.;
      mask[ 1 ][ 0 ] = 0.;  mask[ 1 ][ 1 ] = 0.;  mask[ 1 ][ 2 ] = 0.;
      mask[ 2 ][ 0 ] = 1.;  mask[ 2 ][ 1 ] = 1.;  mask[ 2 ][ 2 ] = 1.;
      break;
    case CCDLIB_OpSWGradientConst:
      mask[ 0 ][ 0 ] = 0.;  mask[ 0 ][ 1 ] = -1.; mask[ 0 ][ 2 ] = -1.;
      mask[ 1 ][ 0 ] = 1.;  mask[ 1 ][ 1 ] = 0.;  mask[ 1 ][ 2 ] = -1.;
      mask[ 2 ][ 0 ] = 1.;  mask[ 2 ][ 1 ] = 1.;  mask[ 2 ][ 2 ] = 0.;
      break;
    case CCDLIB_OpWGradientConst:
      mask[ 0 ][ 0 ] = 1.;  mask[ 0 ][ 1 ] = 0.;  mask[ 0 ][ 2 ] = -1.;
      mask[ 1 ][ 0 ] = 1.;  mask[ 1 ][ 1 ] = 0.;  mask[ 1 ][ 2 ] = -1.;
      mask[ 2 ][ 0 ] = 1.;  mask[ 2 ][ 1 ] = 0.;  mask[ 2 ][ 2 ] = -1.;
      break;
    case CCDLIB_OpNWGradientConst:
      mask[ 0 ][ 0 ] = 1.;  mask[ 0 ][ 1 ] = 1.;  mask[ 0 ][ 2 ] = 0.;
      mask[ 1 ][ 0 ] = 1.;  mask[ 1 ][ 1 ] = 0.;  mask[ 1 ][ 2 ] = -1.;
      mask[ 2 ][ 0 ] = 0.;  mask[ 2 ][ 1 ] = -1.; mask[ 2 ][ 2 ] = -1.;
      break;
    default:
      break; /* ??? This function has to return an error code! */
    }
  }
  if( filtmask != NULL ) CCDLIB_Matrixfree( filtmask );

}

short CCDLIB_GetMedianFromArray
(
  short *values_pd,
  long valuenumber_n
)
{
  short v,t;
  long  i,j, l,r, k;


  l = 0;
  r = valuenumber_n-1;
  k = valuenumber_n/2;

  while( r>l )
  {
    v = values_pd[ r ];
    i = l - 1;
    j = r;

    do
    {
      do
      {
        i++;
      }
      while( values_pd[ i ] < v );

      do
      {
        j--;
      }
      while( values_pd[ j ] > v );

      t              = values_pd[ i ];
      values_pd[ i ] = values_pd[ j ];
      values_pd[ j ] = t;
    }
    while( j > i );

    values_pd[ j ] = values_pd[ i ];
    values_pd[ i ] = values_pd[ r ];
    values_pd[ r ] = t;
    
    if( i >= k ) r=i-1;
    if( i <= k ) l=i+1;

  }
  v = values_pd[ k ];

  return( v );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_LineFilterOperation : Universelle Filteroperation fuer das     *
*                                zeilenweise filtern von Bildern          *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *sourcepic   : 32-Bit-Pointer auf die Quellbilddaten      *
*               *destpic     : 32-Bit-Pointer auf die Zielbilddaten       *
*               maskdiameter :  Durchmesser des Filtermaskenkreis         *
*               CCDLIB_TiefpassTypEnum : Art der Filteroperation          *
*                                                                         *
*               ACHTUNG: maskdiameter darf nur ungerade Werte annehmen!   *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Das Resultatbild steht an derselben Stelle wie das Ausgangsbild.      *
*   ACHTUNG: Das Ausgangsbild wird dabei ueberschrieben !                 *
*                                                                         *
*   Version:    1.1 (23.09.1997)                             32 Bit       *
*   Written by: Georg Dittie, 1994-1997                                   *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_LineFilterOperation
(
  CCDLIB_RectangleStruct        *picturerect,
  CCDLIB_PictureparamStruct     *pictureparam,
  short                         *sourcepic,
  short                         *destpic,
  unsigned long                 maskdiameter,
  enum CCDLIB_OperationTypeEnum operationtype
)
{
  long    x, yk, ys, yks, i,
          xstart, x1stop, y1stop, ystep,
          index, indexhalf, indexturn, bufferindex;

  double  temporary, dynamikmone, divfaktor;

  short   sortarray[ CCDLIB_ConvolutionWidthConst ];

  short   *bufferline[ CCDLIB_ConvolutionWidthConst ];


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( sourcepic == NULL || destpic == NULL )
    return( CCDLIB_NoPicturePointerError );

  if( maskdiameter > CCDLIB_ConvolutionWidthConst )
    return( CCDLIB_WrongParameterError );

  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
    ystep = pictureparam->linelength;
  else
    ystep = pictureparam->linelength*3;

  dynamikmone = (double)(pictureparam->dynamik - 1);
  divfaktor   = 1. / (double)maskdiameter;

  bufferline[ 0 ] = (short *)malloc(
    ystep * sizeof(short) * CCDLIB_ConvolutionWidthConst );

  if( bufferline[ 0 ] == NULL )
    return( CCDLIB_OutOfMemoryError );

  for( i=0; i<CCDLIB_ConvolutionWidthConst; i++ )
  {
    bufferline[ i ] = bufferline[ 0 ] + i * ystep;
  }

  indexhalf  = maskdiameter / 2;
  indexturn  = indexhalf + 1;
  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    x1stop = (long)(picturerect->xpos + picturerect->xsize);
    xstart = (long)(picturerect->xpos) + indexhalf;
  }
  else
  {
    x1stop = (long)(picturerect->xpos + picturerect->xsize)*3;
    xstart = (long)(picturerect->xpos + indexhalf)*3;
  }
  y1stop   = (long)(picturerect->ypos + picturerect->ysize - indexhalf) * ystep;

  index = 0;
  for( ys = (long)(picturerect->ypos + indexhalf) * ystep;
       ys < y1stop;
       ys += ystep )
  {
    bufferindex = index % indexturn;

    if( index >= indexturn )
    {
      for( x = xstart; x < x1stop; x++ )
        destpic[ ys - indexturn*ystep + x ] = bufferline[ bufferindex ][ x ];
    }

    for( x = xstart; x < x1stop; x++ )
    {
      switch( operationtype )
      {
        case CCDLIB_LPLinemeanConst:
          temporary= 0L;
          for( yks = -indexhalf*ystep;
               yks <= indexhalf*ystep;
               yks += ystep )
          {
            temporary += (double)sourcepic[ x + yks ] * divfaktor;
          }
          bufferline[ bufferindex ][ x ] = (short)(
            max( 0, min( dynamikmone, temporary ) ) );
          break;
  
        case CCDLIB_LPLinemedianConst:
          for( yks = -indexhalf*ystep, yk = 0;
               yks <= indexhalf*ystep;
               yks += ystep, yk++ )
          {
            sortarray[ yk ] = (short)sourcepic[ x + yks ];
          }
          bufferline[ bufferindex ][ x ] = (short)(
            CCDLIB_GetMedianFromArray( sortarray, maskdiameter ) );
          break;
        default:
          return( CCDLIB_WrongParameterError );
          break;
      }
    }
    index++;
  }

  for( ys = y1stop;
       ys < y1stop + indexhalf*ystep;
       ys += ystep)
  {
    for( x = xstart; x < x1stop; x++ )
    {
      destpic[ ys - indexturn*ystep + x ] =
        bufferline[ index%indexturn ][ x ];
    }
    index++;
  }

  free( bufferline[ 0 ] );

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_NoisefilterFunction : Routine zum Tiefpassfiltern von ver-     *
*               rauschten Bildern mit separierbaren und nichtseparier-    *
*               baren Filterfunktionen                                    *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *sourcepic   : 32-Bit-Pointer auf die Quellbilddaten      *
*               *destpic     : 32-Bit-Pointer auf die Zielbilddaten       *
*               filtertype   : Typ des separablen Tiefpass                *
*               pseudoflag   : Pseudoseparation nicht s. Filtertypen      *
*                                                                         *
*               ACHTUNG: filterwidth und filterheight darf nur ungerade   *
*                        Werte annehmen!                                  *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Das Resultatbild kann an derselben Stelle wie das Ausgangsbild        *
*   stehen, muss aber nicht: Beides geht.                                 *
*                                                                         *
*   Version:    1.2 (22.09.1997)                              32 Bit      *
*   Written by: Georg Dittie, 1994-1997                                   *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_NoisefilterFunction
(
  CCDLIB_RectangleStruct      *picturerect,
  CCDLIB_PictureparamStruct   *pictureparam,
  short                       *sourcepic,
  short                       *destpic,
  double                      sigmalimit,
  enum CCDLIB_NoiseFilterEnum filtertype,
  BOOL                        pseudoflag
)
{
  double  conv1Dh[ CCDLIB_ConvolutionWidthConst ];
  double  conv1Dv[ CCDLIB_ConvolutionWidthConst ];
  double  **conv2D;
  double  temporary, tempr, tempg, tempb, dynamikmone;

  short   *bufferline[ CCDLIB_ConvolutionWidthConst ];
  short   sortarray[ CCDLIB_ConvolutionWidthConst * CCDLIB_ConvolutionWidthConst ];

  long    x, xk, xc, y, yk, yc, ys, xks, yks, i, n,
          xstart, xstartk, ystart, ysstart, ystartk,
          xstop, ystop, ysstop, xstopk, ystopk, ystep,
          windexhalf, hindexhalf, hindexturn, index, bufferindex;
  long    filterwidth = 3, filterheight = 3;

  enum CCDLIB_ErrorEnum   result;

  double  *line;
  double  *col;


  if( sourcepic == NULL || destpic == NULL )
    return( CCDLIB_NoPicturePointerError );

  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  switch( filtertype )
  {
    case CCDLIB_Noise3x3MedianConst:
    case CCDLIB_Noise3x3SigmaConst:
    case CCDLIB_Noise3x3MittelConst:
      filterwidth=3; filterheight=3;
      break;
    case CCDLIB_Noise5x3MedianConst:
      filterwidth=5; filterheight=3;
      break;
    case CCDLIB_Noise5x5MedianConst:
    case CCDLIB_Noise5x5MittelConst:
    case CCDLIB_Noise5x5SigmaConst:
      filterwidth=5; filterheight=5;
      break;
    case CCDLIB_Noise7x5MedianConst:
      filterwidth=7; filterheight=5;
      break;
    case CCDLIB_NoiseZeilenMedianConst:
      return ( CCDLIB_LineFilterOperation( picturerect, pictureparam,
        sourcepic, destpic, 3, CCDLIB_LPLinemedianConst ) );
      break;
    case CCDLIB_NoiseZeilenMittelConst:
      return ( CCDLIB_LineFilterOperation( picturerect, pictureparam,
        sourcepic, destpic, 3, CCDLIB_LPLinemeanConst ) );
      break;
    default:
      return( CCDLIB_WrongParameterError );
      break;
  }

  windexhalf  = filterwidth / 2;
  hindexhalf  = filterheight / 2;
  dynamikmone = (double)(pictureparam->dynamik - 1);

  /* Zunaechst separable Verfahren */

  if( pseudoflag == TRUE &&
      filtertype != CCDLIB_Noise3x3SigmaConst &&
      filtertype != CCDLIB_Noise5x5SigmaConst )
  {
    if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
    {
      line = ( double * )malloc( pictureparam->linelength * sizeof( double ) );
      col  = ( double * )malloc( pictureparam->columnheight * sizeof( double ) );
    }
    else
    {
      line = ( double * )malloc( pictureparam->linelength * sizeof( double ) *3);
      col  = ( double * )malloc( pictureparam->columnheight * sizeof( double ));
    }

    if( line == NULL || col == NULL )
    {
      if( line != NULL ) free( line );
      if( col  != NULL ) free( col );

      return( CCDLIB_OutOfMemoryError );
    }

    /* Schleifengrenzen fuer separable Verfahren */

    xstart   = (long)(picturerect->xpos);
    ystart   = (long)(picturerect->ypos);
    ysstart  = (long)(picturerect->ypos * pictureparam->linelength);
    xstartk  = (long)(picturerect->xpos)+windexhalf;
    ystartk  = (long)(picturerect->ypos)+hindexhalf;
    xstop    = (long)(picturerect->xpos + picturerect->xsize);
    ystop    = (long)(picturerect->ypos + picturerect->ysize);
    ysstop   = (long)(picturerect->ypos + picturerect->ysize) * pictureparam->linelength;
    ystep    = (long)pictureparam->linelength;
    xstopk   = xstop-windexhalf;
    ystopk   = ystop-hindexhalf;

    if( pictureparam->colorchannel == CCDLIB_RGBChannelColor )
    {
      xstart*=3; xstartk*=3; xstop*=3; xstopk*=3;
      ysstart*=3; ysstop*=3; ystep*=3;
    }

    result = CCDLIB_GetSeparableNoiseFilterMask(
      filterwidth, filterheight, filtertype, conv1Dh, conv1Dv );

    if( result != CCDLIB_Successful )
    {
      free( line );
      free( col );
      return( result );
    }

    /* Horizontale Bearbeitung */

    for( ys = ysstart; ys < ysstop; ys += ystep )
    {
      for( x = xstart; x < xstop; x++ )
      {
        line[ x ] = (double)sourcepic[ x+ys ];
      }

      if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )   /* Schwarz-Weiss */
      {
        switch( filtertype )
        {
          case CCDLIB_Noise3x3MittelConst:
          case CCDLIB_Noise5x5MittelConst:
            for( x = xstartk; x < xstopk; x ++ )
            {
              temporary = 0L;
              for( xk=-windexhalf, xc=0; xc<filterwidth; xk++, xc++ )
              {
                temporary += line[ x+xk ] * conv1Dh[ xc ];
              }
              destpic[ x+ys ] = (short)floor( (temporary<0?0:temporary)+0.5 );
            }
            break;
          case CCDLIB_Noise3x3MedianConst:
          case CCDLIB_Noise5x3MedianConst:
          case CCDLIB_Noise5x5MedianConst:
          case CCDLIB_Noise7x5MedianConst:
            for( x = xstartk; x < xstopk; x ++ )
            {
              for( xk=-windexhalf, xc=0; xc<filterwidth; xk++, xc++ )
              {
                sortarray[ xc ] = line[ x+xk ];
              }
              destpic[ x+ys ] = (short)(
                CCDLIB_GetMedianFromArray( sortarray, filterwidth ) );
            }
            break;
          default:
            return( CCDLIB_WrongParameterError );
            break;
        }
      }
      else   /* RGB-Bilder */
      {
        switch( filtertype )
        {
          case CCDLIB_Noise3x3MittelConst:
          case CCDLIB_Noise5x5MittelConst:
            for( x = xstartk; x < xstopk; x +=3 )
            {
              tempr=0L; tempg=0L; tempb=0L;
              for( xk=-windexhalf*3, xc=0; xc<filterwidth; xk+=3, xc++ )
              {
                tempr += line[ x+xk+2 ] * conv1Dh[ xc ];
                tempg += line[ x+xk+1 ] * conv1Dh[ xc ];
                tempb += line[ x+xk   ] * conv1Dh[ xc ];
              }
              destpic[ x+ys+2 ] = (short)floor( (tempr<0?0:tempr)+0.5 );
              destpic[ x+ys+1 ] = (short)floor( (tempg<0?0:tempg)+0.5 );
              destpic[ x+ys   ] = (short)floor( (tempb<0?0:tempb)+0.5 );
            }
            break;
          case CCDLIB_Noise3x3MedianConst:
          case CCDLIB_Noise5x3MedianConst:
          case CCDLIB_Noise5x5MedianConst:
          case CCDLIB_Noise7x5MedianConst:
            for( x = xstartk; x < xstopk; x += 3 )
            {
              for( xk=-windexhalf*3, xc=0; xc<filterwidth; xk+=3, xc++ )
              {
                sortarray[ xc ] = line[ x+xk ];
              }
              destpic[ x+ys ] = (short)(
                CCDLIB_GetMedianFromArray( sortarray, filterwidth ) );
              for( xk=-windexhalf*3, xc=0; xc<filterwidth; xk+=3, xc++ )
              {
                sortarray[ xc ] = line[ x+xk+1 ];
              }
              destpic[ x+ys+1 ] = (short)(
                CCDLIB_GetMedianFromArray( sortarray, filterwidth ) );
              for( xk=-windexhalf*3, xc=0; xc<filterwidth; xk+=3, xc++ )
              {
                sortarray[ xc ] = line[ x+xk+2 ];
              }
              destpic[ x+ys+2 ] = (short)(
                CCDLIB_GetMedianFromArray( sortarray, filterwidth ) );
            }
            break;
          default:
            return( CCDLIB_WrongParameterError );
            break;
        }
      }
    }

    /* Vertikale Bearbeitung */

    for( x = xstart;
         x < xstop;
         x ++ )
    {
      for( y = ystart, ys = ysstart;
           y < ystop;
           y++, ys += ystep )
      {
        col[ y ] = (double)sourcepic[ x+ys ];
      }

      switch( filtertype )
      {
        case CCDLIB_Noise3x3MittelConst:
        case CCDLIB_Noise5x5MittelConst:
          for( y = ystartk, ys = ystartk * ystep;
               y < ystopk;
               y ++, ys += ystep )
          {
            temporary = 0L;
            for( yk=-hindexhalf, yc=0; yc<filterheight; yk++, yc++ )
            {
              temporary += col[ y+yk ] * conv1Dv[ yc ];
            }
            destpic[ x+ys ] = (short)floor( (temporary<0?0:temporary)+0.5 );
          }
          break;
        case CCDLIB_Noise3x3MedianConst:
        case CCDLIB_Noise5x3MedianConst:
        case CCDLIB_Noise5x5MedianConst:
        case CCDLIB_Noise7x5MedianConst:
          for( y = ystartk, ys = ystartk * ystep;
               y < ystopk;
               y ++, ys += ystep )
          {
            for( yk=-hindexhalf, yc=0; yc<filterheight; yk++, yc++ )
            {
              sortarray[ yc ] = col[ y+yk ];
            }
            destpic[ x+ys ] = (short)(
              CCDLIB_GetMedianFromArray( sortarray, filterheight ) );
          }
          break;
        default:
          return( CCDLIB_WrongParameterError );
          break;
      }
    }

    free( line );
    free( col );
  }
  else     /* Nicht separable Filterverfahren */
  {
    /* Schleifengrenzen fuer nicht separable Verfahren */

    xstart   = (long)(picturerect->xpos)+windexhalf;
    ysstart  = (long)(picturerect->ypos+hindexhalf) * pictureparam->linelength;
    xstop    = (long)(picturerect->xpos + picturerect->xsize - windexhalf);
    ysstop   = (long)(picturerect->ypos + picturerect->ysize - hindexhalf) * pictureparam->linelength;
    ystep    = (long)pictureparam->linelength;

    if( pictureparam->colorchannel == CCDLIB_RGBChannelColor )
    {
      xstart*=3;  xstop*=3;
      ysstart*=3; ysstop*=3; ystep*=3;
    }

    bufferline[ 0 ] = (short *)malloc(
      ystep * sizeof(short) * CCDLIB_ConvolutionWidthConst );

    if( bufferline[ 0 ] == NULL )
      return( CCDLIB_OutOfMemoryError );

    for( i = 0; i < CCDLIB_ConvolutionWidthConst; i ++ )
    {
      bufferline[ i ] = bufferline[ 0 ] + i * ystep;
    }

    hindexturn  = hindexhalf + 1;

    if( !CCDLIB_Matrixalloc( &conv2D, CCDLIB_ConvolutionWidthConst ) )
    {
      free( bufferline[0] );
      return( CCDLIB_OutOfMemoryError );
    }
    CCDLIB_GetNotSeparableNoiseFilterMask(
      filterwidth, filterheight, filtertype, conv2D );

    index = 0;
    for( ys = ysstart; ys < ysstop; ys += ystep)
    {
      bufferindex = index % hindexturn;

      if( index >= hindexturn )
      {
        for( x = xstart; x < xstop; x++ )
        {
          destpic[ ys - hindexturn*ystep + x ] =
            bufferline[ bufferindex ][ x ];
        }
      }

      if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
      {
        for( x = xstart; x < xstop; x++ )
        {
          switch( filtertype )
          {
            case CCDLIB_Noise3x3MittelConst:
            case CCDLIB_Noise5x5MittelConst:
              temporary= 0L;
              for( yks = -hindexhalf*ystep, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf, xk=0;
                     xks <= windexhalf;
                     xks++, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                    temporary += (double)sourcepic[ x+xks + ys+yks ] * conv2D[ yk ][ xk ];
                }
              }
              bufferline[ bufferindex ][ x ] = (short)(
                max( 0, min( dynamikmone, temporary ) ) );
              break;
            case CCDLIB_Noise3x3MedianConst:
            case CCDLIB_Noise5x3MedianConst:
            case CCDLIB_Noise5x5MedianConst:
            case CCDLIB_Noise7x5MedianConst:
              for( yks = -hindexhalf*ystep, n=0, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf, xk=0;
                     xks <= windexhalf;
                     xks ++, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                  {
                    sortarray[ n ] = sourcepic[ x+xks + ys+yks ];
                    n++;
                  }
                }
              }
              bufferline[ bufferindex ][ x ] = (short)(
                CCDLIB_GetMedianFromArray( sortarray, n-1 ) );
              break;
            case CCDLIB_Noise3x3SigmaConst:
            case CCDLIB_Noise5x5SigmaConst:
              for( yks = -hindexhalf*ystep, n=0, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf, xk=0;
                     xks <= windexhalf;
                     xks ++, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                  {
                    sortarray[ n ] = sourcepic[ x+xks + ys+yks ];
                    n++;
                  }
                }
              }
              bufferline[ bufferindex ][ x ] = (short)(
                CCDLIB_RemoveGreaterThanSigmaFromArray(
                sortarray, sourcepic[ x + ys ], n-1, sigmalimit ) );
              break;
            default:
              return( CCDLIB_WrongParameterError );
              break;
          }
        }
      }
      else    /* Nichtseparables Filtern von RGB-Bildern */
      {
        for( x = xstart; x < xstop; x+=3 )
        {
          switch( filtertype )
          {
            case CCDLIB_Noise3x3MittelConst:
            case CCDLIB_Noise5x5MittelConst:
              tempb=0.; tempg=0.; tempr=0.;
              for( yks = -hindexhalf*ystep, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf*3, xk=0;
                     xks <= windexhalf*3;
                     xks+=3, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                  {
                    tempb += (double)sourcepic[ x+xks + ys+yks   ] * conv2D[ yk ][ xk ];
                    tempg += (double)sourcepic[ x+xks + ys+yks+1 ] * conv2D[ yk ][ xk ];
                    tempr += (double)sourcepic[ x+xks + ys+yks+2 ] * conv2D[ yk ][ xk ];
                  }
                }
              }
              bufferline[ bufferindex ][ x ] = (short)(
                max( 0, min( dynamikmone, tempb ) ) );
              bufferline[ bufferindex ][ x+1 ] = (short)(
                max( 0, min( dynamikmone, tempg ) ) );
              bufferline[ bufferindex ][ x+2 ] = (short)(
                max( 0, min( dynamikmone, tempr ) ) );
  
              break;
  
            case CCDLIB_Noise3x3MedianConst:
            case CCDLIB_Noise5x3MedianConst:
            case CCDLIB_Noise5x5MedianConst:
            case CCDLIB_Noise7x5MedianConst:
              for( yks = -hindexhalf*ystep, n=0, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf*3, xk=0;
                     xks <= windexhalf*3;
                     xks +=3, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                  {
                    sortarray[ n ] = sourcepic[ x+xks + ys+yks ];
                    n++;
                  }
                }
              }
              bufferline[ bufferindex ][ x ] = (short)(
                CCDLIB_GetMedianFromArray( sortarray, n-1 ) );
  
              for( yks = -hindexhalf*ystep, n=0, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf*3+1, xk=0;
                     xks <= windexhalf*3;
                     xks +=3, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                  {
                    sortarray[ n ] = sourcepic[ x+xks + ys+yks ];
                    n++;
                  }
                }
              }
              bufferline[ bufferindex ][ x+1 ] = (short)(
                CCDLIB_GetMedianFromArray( sortarray, n-1 ) );
  
              for( yks = -hindexhalf*ystep, n=0, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf*3+2, xk=0;
                     xks <= windexhalf*3;
                     xks +=3, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                  {
                    sortarray[ n ] = sourcepic[ x+xks + ys+yks ];
                    n++;
                  }
                }
              }
              bufferline[ bufferindex ][ x+2 ] = (short)(
                CCDLIB_GetMedianFromArray( sortarray, n-1 ) );
  
              break;
            case CCDLIB_Noise3x3SigmaConst:
            case CCDLIB_Noise5x5SigmaConst:
              for( yks = -hindexhalf*ystep, n=0, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf*3, xk=0;
                     xks <= windexhalf*3;
                     xks +=3, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                  {
                    sortarray[ n ] = sourcepic[ x+xks + ys+yks ];
                    n++;
                  }
                }
              }
              bufferline[ bufferindex ][ x ] = (short)(
                CCDLIB_RemoveGreaterThanSigmaFromArray(
                sortarray, sourcepic[ x + ys ], n-1, sigmalimit ) );
  
              for( yks = -hindexhalf*ystep, n=0, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf*3+1, xk=0;
                     xks <= windexhalf*3;
                     xks +=3, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                  {
                    sortarray[ n ] = sourcepic[ x+xks + ys+yks ];
                    n++;
                  }
                }
              }
              bufferline[ bufferindex ][ x+1 ] = (short)(
                CCDLIB_RemoveGreaterThanSigmaFromArray(
                sortarray, sourcepic[ x+1 + ys ], n-1, sigmalimit ) );
  
              for( yks = -hindexhalf*ystep, n=0, yk=0;
                   yks <= hindexhalf*ystep;
                   yks += ystep, yk++ )
              {
                for( xks = -windexhalf*3+2, xk=0;
                     xks <= windexhalf*3;
                     xks +=3, xk++ )
                {
                  if( conv2D[ yk ][ xk ] != 0 )
                  {
                    sortarray[ n ] = sourcepic[ x+xks + ys+yks ];
                    n++;
                  }
                }
              }
              bufferline[ bufferindex ][ x+2 ] = (short)(
                CCDLIB_RemoveGreaterThanSigmaFromArray(
                sortarray, sourcepic[ x+2 + ys ], n-1, sigmalimit ) );
  
              break;
            default:
              return( CCDLIB_WrongParameterError );
              break;
          }
        }
      }
      index++;
    }

    for( ys = ysstop; ys < ysstop + hindexhalf*ystep; ys += ystep)
    {
      for( x = xstart; x < xstop; x++ )
        destpic[ ys - hindexturn*ystep + x ] =
          bufferline[ index%hindexturn ][ x ];
      index++;
    }

    CCDLIB_Matrixfree( conv2D );
    free( bufferline[ 0 ] );
  }

  return( CCDLIB_Successful );
}

enum CCDLIB_ErrorEnum CCDLIB_GetSeparableNoiseFilterMask(
  long                        filterwidth,
  long                        filterheight,
  enum CCDLIB_NoiseFilterEnum filtertype,
  double                      *conv1Dh,
  double                      *conv1Dv
)
{
  long i;


  switch( filtertype )
  {
    case CCDLIB_Noise3x3MedianConst:
    case CCDLIB_Noise5x3MedianConst:
    case CCDLIB_Noise5x5MedianConst:
    case CCDLIB_Noise7x5MedianConst:
      for( i=0; i<filterwidth; i++ )
        conv1Dh[ i ] = 1.;
      for( i=0; i<filterheight; i++ )
        conv1Dv[ i ] = 1.;
      break;
    case CCDLIB_Noise3x3MittelConst:
    case CCDLIB_Noise5x5MittelConst:
      for( i=0; i<filterwidth; i++ )
        conv1Dh[ i ] = 1. / (double)filterwidth;
      for( i=0; i<filterheight; i++ )
        conv1Dv[ i ] = 1. / (double)filterheight;
      break;
    default:
      return( CCDLIB_WrongParameterError );
      break;
  }

  return( CCDLIB_Successful );
}

enum CCDLIB_ErrorEnum CCDLIB_GetNotSeparableNoiseFilterMask(
  long                        filterwidth,
  long                        filterheight,
  enum CCDLIB_NoiseFilterEnum filtertype,
  double                      **conv2D
)
{
  long    x, y, hh, wh, xdif, ydif, xsqr, ysqr;
  double  areasum;


  hh = filterheight/2;
  wh = filterwidth/2;

  switch( filtertype )
  {
    case CCDLIB_Noise3x3MedianConst:
    case CCDLIB_Noise3x3SigmaConst:
    case CCDLIB_Noise3x3MittelConst:
      for( y=0; y<filterheight; y++ )
      {
        ydif = y-hh;
        ysqr = ydif*ydif;
        for( x=0; x<filterwidth; x++ )
        {
          xdif = x-wh;
          xsqr = xdif*xdif;
  
          if( ysqr+xsqr < 2 )
            conv2D[y][x] = 1.;
          else
            conv2D[y][x] = 0.;
        }
      }
      break;
    case CCDLIB_Noise5x3MedianConst:
    case CCDLIB_Noise5x5SigmaConst:
    case CCDLIB_Noise5x5MedianConst:
    case CCDLIB_Noise5x5MittelConst:
      for( y=0; y<filterheight; y++ )
      {
        ydif = y-hh;
        ysqr = ydif*ydif;
        for( x=0; x<filterwidth; x++ )
        {
          xdif = x-wh;
          xsqr = xdif*xdif;
  
          if( ysqr+xsqr < 4 )
            conv2D[y][x] = 1.;
          else
            conv2D[y][x] = 0.;
        }
      }
      break;
    case CCDLIB_Noise7x5MedianConst:
      for( y=0; y<filterheight; y++ )
      {
        ydif = y-hh;
        ysqr = ydif*ydif;
        for( x=0; x<filterwidth; x++ )
        {
          xdif = x-wh;
          xsqr = xdif*xdif;
          if( ysqr+xsqr < 6 )
            conv2D[y][x] = 1.;
          else
            conv2D[y][x] = 0.;
        }
      }
      break;
    default:
      return( CCDLIB_WrongParameterError );
      break;
  }

  switch( filtertype )
  {
    case CCDLIB_Noise3x3MittelConst:
    case CCDLIB_Noise5x5MittelConst:
      areasum = 0.;
      for( y=0; y<filterheight; y++ )
      {
        for( x=0; x<filterwidth; x++ )
        {
          areasum += conv2D[y][x];
        }
      }
      for( y=0; y<filterheight; y++ )
      {
        for( x=0; x<filterwidth; x++ )
        {
          conv2D[y][x] /= areasum;
        }
      }
      break;
    default:
      return( CCDLIB_WrongParameterError );
      break;
  }

  return( CCDLIB_Successful );
}

short CCDLIB_RemoveGreaterThanSigmaFromArray(
  short   *sortarray,
  short   greyvalue,
  long    arraylength,
  double  sigmalimit
)
{
  double mean = 0., sigmasum = 0., dif, sigma;
  long i;


  for( i=0; i<arraylength; i++ )
    mean += (double)sortarray[ i ];

  for( i=0; i<arraylength; i++ )
  {
    dif       = (double)sortarray[ i ] - mean;
    sigmasum += (dif*dif);
  }

  if( sigmasum != 0. )
    sigma = sqrt( sigmasum / (double)(arraylength-1) );
  else
    return( (short)floor(mean+0.5) );

  if( fabs( (double)greyvalue-mean ) > sigma*sigmalimit )
    return( (short)floor(mean+0.5) );
  else
    return( greyvalue );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_ExtractLuminance : Routine zum Berechnen des Luminanzanteils   *
*               nach ISO-Norm aus einem RGB-Farbbild                      *
*                                                                         *
*   Parameter:  *sourcepic    : 32-Bit-Pointer auf die Quellbilddaten     *
*               * pictureparam: Systemparameter des zu bearbeitenden      *
*               *destpic      : 32-Bit-Pointer auf die Luminanzdaten      *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*   CCDLIB_NoPointerError:        Nicht initialisierte Ptr.               *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Das Resultatbild kann nicht an derselben Stelle wie das               *
*   Ausgangsbild stehen !                                                 *
*                                                                         *
*   Version:    1.0 (12.05.1996)                                          *
*   Written by: Georg Dittie, 1996                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_ExtractLuminance
(
  short                     *sourcepicture,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *destpicture
)
{
   long   i, ic, istop;
   double lumval;


   if( destpicture == NULL || sourcepicture == NULL )
     return( CCDLIB_NoPicturePointerError );

  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor ||
      destpicture == sourcepicture )
    return( CCDLIB_WrongParameterError );

  istop = pictureparam->linelength*pictureparam->columnheight;

  ic = 0;
  for( i=0; i<istop; i++ )
  {
    lumval  = 0.299*(double)sourcepicture[ ic + 2 ]
            + 0.587*(double)sourcepicture[ ic + 1 ]
            + 0.114*(double)sourcepicture[ ic ];
    destpicture[ i ] = (short)floor( lumval + 0.5 );
    ic += 3;
  }

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_FlatfieldCorrection: Routine zur Flatfieldkorrektur eines      *
*               Bildbereichs                                              *
*                                                                         *
*   Parameter:  *destpicture  : 32-Bit-Pointer auf das Zielbild           *
*               *sourcepicture: 32-Bit-Pointer auf das Quellbild          *
*               *flatfield    : 32-Bit-Pointer auf das Flatfield          *
*               *pictureparam : Systemparameter des zu bearbeitenden      *
*                               Bildes                                    *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0 (11.05.1996)                                          *
*   Written by: Georg Dittie, 1996                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_FlatfieldCorrection
(
  short                     *destpicture,
  short                     *sourcepicture,
  short                     *flatfield,
  CCDLIB_PictureparamStruct *pictureparam
)
{
  long   i, buflength;
  double flatmean;


  if( destpicture == NULL || sourcepicture == NULL || flatfield == NULL )
    return( CCDLIB_NoPicturePointerError );

  if( pictureparam->colorchannel == CCDLIB_RGBChannelColor )
    buflength = pictureparam->linelength*pictureparam->columnheight*3L;
  else
    buflength = pictureparam->linelength*pictureparam->columnheight;

  flatmean = 0.;
  for( i=0; i<buflength; i++ )
    flatmean += (double)sourcepicture[ i ];
  flatmean /= (double)buflength;

  for( i=0; i<buflength; i++ )
    destpicture[ i ] = (short)floor(
      (double)sourcepicture[ i ] * flatmean / (double)flatfield[ i ] + 0.5 );

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_DarkframeCorrection: Routine zur Darkframekorrektur eines      *
*               Bildbereichs                                              *
*                                                                         *
*   Parameter:  *destpicture  : 32-Bit-Pointer auf das Zielbild           *
*               *sourcepicture: 32-Bit-Pointer auf das Quellbild          *
*               *darkframe    : 32-Bit-Pointer auf das Flatfield          *
*               *pictureparam : Systemparameter des zu bearbeitenden      *
*                               Bildes                                    *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0 (11.05.1996)                                          *
*   Written by: Georg Dittie, 1996                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_DarkframeCorrection
(
  short                     *destpicture,
  short                     *sourcepicture,
  short                     *darkframe,
  CCDLIB_PictureparamStruct *pictureparam
)
{
  long  i, buflength;
  short temp;


  if( destpicture == NULL || sourcepicture == NULL || darkframe == NULL )
    return( CCDLIB_NoPicturePointerError );

  if( pictureparam->colorchannel == CCDLIB_RGBChannelColor )
    buflength = pictureparam->linelength*pictureparam->columnheight*3L;
  else
    buflength = pictureparam->linelength*pictureparam->columnheight;

  for( i=0; i<buflength; i++ )
  {
    temp = (short)(sourcepicture[ i ] - darkframe[ i ]);
    destpicture[ i ] = (short)( temp < 0 ? 0 : temp );
  }
  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_CountBrightsteps: Routine zum Ermitteln der Anzahl von Grau-   *
*               bzw. Luminanz und Farbstufen eines Bildbereichs           *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*   *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *picture     : 32-Bit-Pointer auf die Bilddaten           *
*               *greycount   : Anzahl der Grauwerte                       *
*               *redcount    : Anzahl der Rotwerte                        *
*               *greencount  : Anzahl der Gruenwerte                      *
*               *bluecount   : Anzahl der Blauwerte                       *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0 (11.10.1998)                                          *
*   Written by: Georg Dittie, 1998                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_CountBrightsteps
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  long                      *greycount,
  long                      *redcount,
  long                      *greencount,
  long                      *bluecount
)
{
  long x, ys, i, xstop, ystop, ystep;
  long *greyhist, *redhist, *greenhist, *bluehist;


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL )
    return( CCDLIB_NoPicturePointerError );

  *greycount  = 0;
  *redcount   = 0;
  *greencount = 0;
  *bluecount  = 0;

  xstop = (long)(picturerect->xpos + picturerect->xsize);
  ystop = (long)(picturerect->ypos + picturerect->ysize) * pictureparam->linelength;

  greyhist = (long*)malloc( pictureparam->dynamik*sizeof(long) );
  if( greyhist == NULL )
    return( CCDLIB_OutOfMemoryError );
  memset( greyhist, 0, pictureparam->dynamik*sizeof(long) );

  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    for(ys = (long)(picturerect->ypos * pictureparam->linelength);
      ys < ystop;
      ys += pictureparam->linelength)
    {
      for(x = (long)(picturerect->xpos);
        x < xstop;
        x++ )
      {
        greyhist[ picture[ ys+x ] ] ++;
      }
    }
    for( i=0; i<pictureparam->dynamik; i++ )
      if( greyhist[ i ] > 0 )
        (*greycount)++;
  }
  else  /* Luminanz-Histogram nach ISO-Norm */
  {
    redhist   = (long*)malloc( pictureparam->dynamik*sizeof(long) );
    greenhist = (long*)malloc( pictureparam->dynamik*sizeof(long) );
    bluehist  = (long*)malloc( pictureparam->dynamik*sizeof(long) );
    if( redhist == NULL || greenhist == NULL || bluehist == NULL )
    {
      if( greyhist != NULL )  free( greyhist );
      if( redhist != NULL )   free( redhist );
      if( greenhist != NULL ) free( greenhist );
      if( bluehist != NULL )  free( bluehist );
      return( CCDLIB_OutOfMemoryError );
    }
    memset( greyhist, 0, pictureparam->dynamik*sizeof(long) );
    memset( greyhist, 0, pictureparam->dynamik*sizeof(long) );
    memset( greyhist, 0, pictureparam->dynamik*sizeof(long) );

    xstop *= 3;
    ystop *= 3;
    ystep = pictureparam->linelength*3;
    for( ys = (long)(picturerect->ypos * pictureparam->linelength);
         ys < ystop;
         ys += ystep )
    {
      for( x = (long)(picturerect->xpos);
           x < xstop;
           x+=3 )
      {
        greyhist[ (long)floor(
          0.114*(double)picture[ ys+x ] +
          0.587*(double)picture[ ys+x+1 ] +
          0.299*(double)picture[ ys+x+2 ] + 0.5) ] ++;
        bluehist[ picture[ ys+x ] ] ++;
        greenhist[ picture[ ys+x+1 ] ] ++;
        redhist[ picture[ ys+x+2 ] ] ++;
      }
    }
    for( i=0; i<pictureparam->dynamik; i++ )
    {
      if( greyhist[ i ] > 0 )
        (*greycount)++;
      if( redhist[ i ] > 0 )
        (*redcount)++;
      if( greenhist[ i ] > 0 )
        (*greencount)++;
      if( bluehist[ i ] > 0 )
        (*bluecount)++;
    }
  }

  if( greyhist != NULL )  free( greyhist );
  if( redhist != NULL )   free( redhist );
  if( greenhist != NULL ) free( greenhist );
  if( bluehist != NULL )  free( bluehist );

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_RetrieveHistogram: Routine zum Ermitteln eines Grauwertver-    *
*               teilungshistogramms eines Bildbereichs                    *
*                                                                         *
*   Parameter:  *picturerect    : Koordinate der oberen linken Ecke des   *
*                                 Arbeitsbereiches im Bild                *
*               *pictureparam   : Systemparameter des zu bearbeitenden    *
*                                 Bildes                                  *
*               *picture        : 32-Bit-Pointer auf die Bilddaten        *
*               *histogramfield : Pointer auf Histogrammwerte             *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Das Resultat steht an der Stelle, wo histogramfield hin pointet.      *
*                                                                         *
*   Version:    1.1 (22.09.1997)                                          *
*   Written by: Georg Dittie, 1994-1997                                   *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_RetrieveHistogram
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  long                      *histogramfield
)
{
  long  x, ys, i, xstop, ystop, ystep;


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL )
    return( CCDLIB_NoPicturePointerError );

  if( histogramfield == NULL )
    return( CCDLIB_NoPointerError );

  for( i=0; i<(long)pictureparam->dynamik; i++ )
    histogramfield[ i ] = 0;

  xstop = (long)(picturerect->xpos + picturerect->xsize);
  ystop = (long)(picturerect->ypos + picturerect->ysize) * pictureparam->linelength;

  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    for( ys = (long)(picturerect->ypos * pictureparam->linelength);
         ys < ystop;
         ys += pictureparam->linelength)
    {
      for( x = (long)(picturerect->xpos); x < xstop; x++ )
      {
        histogramfield[ picture[ ys+x ] ] ++;
      }
    }
  }
  else  /* Luminanz-Histogram nach ISO-Norm */
  {
    xstop *= 3;
    ystop *= 3;
    ystep = pictureparam->linelength*3;
    for( ys = (long)(picturerect->ypos * pictureparam->linelength);
         ys < ystop;
         ys += ystep )
    {
      for(x = (long)(picturerect->xpos); x < xstop; x+=3 )
      {
        histogramfield[ (long)floor(
          0.114*(double)picture[ ys+x ] +
          0.587*(double)picture[ ys+x+1 ] +
          0.299*(double)picture[ ys+x+2 ] + 0.5) ] ++;
      }
    }
  }

  return( CCDLIB_Successful );
}

/**************************************************************************
*                                                                         *
*   CCDLIB_GetMaximumInHistogram: Routine zum Adhoc-Auswerten eines       *
*               Grauwertverteilungshistogramms eines Bildbereichs.        *
*                                                                         *
*   Parameter:  *histogramfield: Pointer auf Histogrammwerte              *
*               dynamik        : Anzahl der Graustufen                    *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*               * lowestposition  : Kleinster besetzter Grauwert          *
*               * highestposition : Groesster besetzter Grauwert          *
*               * maximumposition : Haeufgster besetzter Grauwert         *
*               * maximalvalue    : Anzahl der Pixel mit haeuf. Grauw.    *
*               * maximumpart     : Anteil des haeuf. Grauwert am Bild    *
*               * entropy         : Grauwert-Entropie des Bildes          *
*                                                                         *
*   Version:    1.0 (05.06.1994)                                          *
*   Written by: Georg Dittie, 1994                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_GetMaximumInHistogram
(
  long    *histogramfield,
  long    dynamik,
  long    *lowestposition,
  long    *highestposition,
  long    *maximumposition,
  long    *maximalvalue,
  double  *maximumpart,
  double  *entropy
)
{
  long i, histogramsum;
  
  BOOL minfound, maxfound;

  
  if( histogramfield == NULL )
    return( CCDLIB_NoPointerError );
  
  minfound = FALSE;
  maxfound = FALSE;
                        
  *maximumposition  = 0;
  *maximalvalue     = histogramfield[ 0 ];
  histogramsum      = 0;
  
  *lowestposition = 0;
  for( i=0; i<dynamik; i++ )
  {
    histogramsum += histogramfield[ i ];
    
    if( histogramfield[ i ] > 0 && minfound == FALSE )
    {
      minfound = TRUE;
      *lowestposition = i;
    }
    
    if( histogramfield[ i ] > *maximalvalue )
    {
      *maximalvalue = histogramfield[ i ];
      *maximumposition = i;
    }                       
  }
  
  *highestposition = dynamik-1;
  for( i=dynamik-1; i>=0; i-- )
  {
    if( histogramfield[ i ] > 0 && maxfound == FALSE )
    {
      *highestposition  = i;
      maxfound          = TRUE;
    }
  }  

  *maximumpart = (double)*maximalvalue/(double)histogramsum;
  *entropy     = 1./log( 2. ) 
               * log( (double)labs( *highestposition - *lowestposition + 1 ) );  

  return( CCDLIB_Successful );
}                                                           

/**************************************************************************
*                                                                         *
*   CCDLIB_EvaluateHistogram: Routine zum Adhoc-Auswerten eines           *
*               Grauwertverteilungshistogramms eines Bildbereichs.        *
*                                                                         *
*   Parameter:  *histogramfield : Pointer auf Histogrammwerte             *
*               dynamik         : Anzahl der Graustufen                   *
*   interval        : Enum zur Intervalwahl                   *
*               hotpixelcount   : Anzahl auszuschliessender Hotpixel      *
*               deadpixelcount  : Anzahl auszuschliessender Deadpixel     *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*               * lowestposition  : Kleinster besetzter Grauwert          *
*               * highestposition : Groesster besetzter Grauwert          *
*               * lowposition     : Kleinster Grauwert des Intervals      *
*               * highposition    : Groesster Grauwert des Intervals      *
*               * maximumposition : Haeufgster besetzter Grauwert         *
*               * maximalvalue    : Anzahl der Pixel mit haeuf. Grauw.    *
*               * maximumpart     : Anteil des haeuf. Grauwert am Bild    *
*               * entropy         : Grauwert-Entropie des Bildes          *
*                                                                         *
*   Version:    1.2 (08.10.1998)                                          *
*   Written by: Georg Dittie, 1994-1998                                   *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_EvaluateHistogram
(
  long                            *histogramfield,
  long                            dynamik,
  enum CCDLIB_HistogramrangeEnum  interval,
  long                            hotpixelnum,
  long                            deadpixelnum,
  long                            *lowestposition,
  long                            *highestposition,
  long                            *lowposition,
  long                            *highposition,
  long                            *maximumposition,
  long                            *maximalvalue,
  double                          *maximumpart,
  double                          *entropy
)
{
  long i, histogramsum, histolowsum, histohighsum;

  BOOL minfound, absmaxfound, maxfound;


  if( histogramfield == NULL )
    return( CCDLIB_NoPointerError );

  absmaxfound = FALSE;
  minfound    = FALSE;
  maxfound    = FALSE;

  histogramsum  = 0;
  histolowsum   = 0;
  histohighsum  = 0;

  *lowestposition = 0;
  for( i=0; i<dynamik; i++ )
  {
    histogramsum += histogramfield[ i ];

    if( histogramsum > deadpixelnum )
    {
      *lowestposition = i;
      break;
    }
  }

  *highestposition = dynamik-1;
  *maximumposition = 0;
  *maximalvalue    = histogramfield[ dynamik-1 ];
  histogramsum     = 0;
  for( i=dynamik-1; i>=0; i-- )
  {
    histogramsum += histogramfield[ i ];
    if( histogramsum > hotpixelnum && absmaxfound == FALSE )
    {
      *highestposition  = i;
      absmaxfound       = TRUE;
    }

    if( histogramfield[ i ] > *maximalvalue )
    {
      *maximalvalue     = histogramfield[ i ];
      *maximumposition  = i;
    }
  }

  histogramsum = histogramsum-hotpixelnum-deadpixelnum;
  *maximumpart = (double)*maximalvalue/(double)histogramsum;

  switch( interval )
  {
    case CCDLIB_HistoExaktConst:
      histohighsum = 0;
      histolowsum  = histogramsum;
      break;
    case CCDLIB_Histo99_8Const:
      histohighsum = (long)floor(histogramsum*0.001+0.5);
      histolowsum  = (long)floor(histogramsum*0.999+0.5);
      break;
    case CCDLIB_Histo99_6Const:
      histohighsum = (long)floor(histogramsum*0.002+0.5);
      histolowsum  = (long)floor(histogramsum*0.998+0.5);
      break;
    case CCDLIB_Histo99Const:
      histohighsum = (long)floor(histogramsum*0.005+0.5);
      histolowsum  = (long)floor(histogramsum*0.995+0.5);
      break;
    case CCDLIB_Histo98Const:
      histohighsum = (long)floor(histogramsum*0.01+0.5);
      histolowsum  = (long)floor(histogramsum*0.99+0.5);
      break;
    case CCDLIB_Histo96Const:
      histohighsum = (long)floor(histogramsum*0.02+0.5);
      histolowsum  = (long)floor(histogramsum*0.98+0.5);
      break;
    case CCDLIB_Histo90Const:
      histohighsum = (long)floor(histogramsum*0.05+0.5);
      histolowsum  = (long)floor(histogramsum*0.95+0.5);
      break;
    case CCDLIB_Histo80Const:
      histohighsum = (long)floor(histogramsum*0.1+0.5);
      histolowsum  = (long)floor(histogramsum*0.9+0.5);
      break;
    default:
      return( CCDLIB_WrongParameterError );
      break;
  }

  *highposition    = dynamik-1;
  histogramsum     = 0;
  for( i=*highestposition; i>=*lowestposition; i-- )
  {
    histogramsum += histogramfield[ i ];
    if( histogramsum >= histohighsum && maxfound == FALSE )
    {
      *highposition = i;
      maxfound = TRUE;
    }
    if( histogramsum >= histolowsum && minfound == FALSE )
    {
      *lowposition = i;
      minfound = TRUE;
    }
  }

  *entropy  = 1./log( 2. )
            * log( (double)labs( *highestposition - *lowestposition + 1 ) );

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_GetLocalStatistics: Routine zum Ermitteln von Mittelwert       *
*               und Standardabweichung in einer Bildregion                *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *picture     : 32-Bit-Pointer auf die Bilddaten           *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*               * mean  :           arithmetischer Mittelwert             *
*               * sigma :           N-1 Standardabweichung                *
*                                                                         *
*   Version:    1.0 (05.06.1994)                                          *
*   Written by: Georg Dittie, 1994                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_GetLocalStatistics
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  double                    *mean,
  double                    *sigma
)
{
  long    x, ys, xstop, ystop;
       
  long    meansum, meanvalue, variance, temporary, pixelnum;
  double  variancesum;


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL )
  {
    *mean = 0.;
    *sigma = 0.;
    return( CCDLIB_NoPicturePointerError );
  }

  xstop = (long)(picturerect->xpos + picturerect->xsize);
  ystop = (long)(picturerect->ypos + picturerect->ysize) * pictureparam->linelength;
  
  pixelnum = (long)picturerect->xsize * (long)picturerect->ysize;

  *mean = 0;
  for( ys = (long)(picturerect->ypos * pictureparam->linelength);
       ys < ystop;
       ys += pictureparam->linelength)
  {
    meansum = 0;
    for( x = (long)(picturerect->xpos);
         x < xstop;
         x++ )
    {
      meansum += (long)picture[ ys+x ];
    }
    *mean += (double)meansum;
  }                                          
  if( *mean == 0. )
  {
    *mean   = 0.;
    *sigma  = 0.;
    return( CCDLIB_Successful );
  }
  
  *mean /= (double)pixelnum;
  
  meanvalue   = (long)floor( *mean + 0.5 );
  variancesum = 0.;
  for( ys = (long)(picturerect->ypos * pictureparam->linelength);
       ys < ystop;
       ys += pictureparam->linelength)
  {
    variance = 0;
    for( x = (long)(picturerect->xpos); x < xstop; x++ )
    {
      temporary = (long)picture[ ys+x ] - meanvalue;
      variance += ( temporary * temporary );
    }                                                                      
    variancesum += (double)variance;
  }

  if( variancesum == 0. )
    *sigma = 0.;
  else
    *sigma = sqrt( variancesum / (double)( pixelnum-1 ) );

  return( CCDLIB_Successful );
}



/**************************************************************************
*                                                                         *
*   CCDLIB_CalcHistogramLUT: Routine zum Berechnen einer Look Up Table    *
*                            zur Kontrastkorrektur. Nicht gebrauchte      *
*                            Pointer koennen auf NULL gesetzt werden.     *
*                                                                         *
*   Parameter:  *histogramLUT  : berechnete LUT                           *
*               dynamik        : Anzahl der Graustufen                    *
*               kontart        : Methode der LUT-Berechnung               *
*               gamma          : Gammaeinstellung                         *
*               wendepunkt     : Wendepunkt bei S-Kurve                   *
*               highgamma      : Gamma oberer Abschnitt                   *
*               lowgamma       : Gamma unterer Abschnitt                  *
*               equalize       : Equalizing mit ausfuehren                *
*               interval       : Interval fuer Equalizing                 *
*               *histogramfield: Pointer auf Histogrammwerte              *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0 (20.09.1997)                        32 Bit            *
*   Written by: Georg Dittie, 1997                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_CalcHistogramLUT
(
  short                             *histogramLUT,
  long                              dynamik,
  enum CCDLIB_HistogramModifierEnum modifier,
  double                            gamma,
  long                              wendepunkt,
  double                            highgamma,
  double                            lowgamma,
  BOOL                              equalize,
  enum CCDLIB_HistogramrangeEnum    interval,
  long                              *histogramfield
)
{
  double  xnormfaktor, ynormfaktor,
          xhighnormfak, yhighnormfak, xlownormfak, ylownormfak, ywendepkt;
  long    i, histodif, histosum, aktsum;
  long    lowestposition, highestposition,
          lowposition, highposition,
          maximumposition, maximalvalue;
  double  maximumpart, entropy;


  if( histogramLUT == NULL ) return( CCDLIB_NoPointerError );

  if( equalize == TRUE || modifier == CCDLIB_HistogramConst )
  {
    if( histogramfield == NULL ) return( CCDLIB_NoPointerError );

    CCDLIB_EvaluateHistogram(
      histogramfield, dynamik, interval, 0, 0,
      &lowestposition, &highestposition,
      &lowposition, &highposition,
      &maximumposition, &maximalvalue,
      &maximumpart, &entropy );
    histodif = highposition-lowposition;
    if( wendepunkt <= lowposition )  wendepunkt = lowposition+1;
    if( wendepunkt >= highposition ) wendepunkt = highposition-1;
  }
  else
  {
    lowestposition  = 0; highestposition = dynamik-1;
    lowposition     = 0;    highposition = dynamik-1;
    histodif  = highposition-lowposition;
  }


  switch( modifier )
  {
    case CCDLIB_LinearConst:
      ynormfaktor = (double)(dynamik-1)/(double)(histodif);
      break;
    case CCDLIB_S_SchlagConst:
      ywendepkt     = (long)floor( (double)((wendepunkt-lowposition)*(dynamik-1))
                    / (double)histodif + 0.5);
      xlownormfak   = (double)(wendepunkt-lowposition);
      ylownormfak   = (double)ywendepkt;
      xhighnormfak  = (double)(highposition-wendepunkt);
      yhighnormfak  = (double)(dynamik-1-ywendepkt);
      break;
    case CCDLIB_GammaKorrekturConst:
      xnormfaktor = (double)histodif;
      ynormfaktor = (double)dynamik-1.;
      break;
    case CCDLIB_LogarithmusConst:
      xnormfaktor = (double)(histodif-1)/(MC_E*MC_E-1.);
      ynormfaktor = (double)(dynamik-1)/2.;
      break;
    case CCDLIB_ExponentConst:
      xnormfaktor = (double)(histodif-1)/2.;
      ynormfaktor = (double)(dynamik-1)/(MC_E*MC_E-1.);
      break;
    case CCDLIB_HistogramConst:
      histosum = 0;
      for( i=lowposition; i<highposition; i++ )                                  
        histosum += histogramfield[i];
      ynormfaktor = (double)(dynamik-1)/(double)(histosum);
    default:
      return( CCDLIB_WrongParameterError );
      break;
  }

  for( i=0; i<lowposition; i++ )
    histogramLUT[ i ] = 0;
  for( i=lowposition, aktsum=0; i<highposition; i++ )
  {
    switch( modifier )
    {
      case CCDLIB_LinearConst:
        histogramLUT[ i ] = (short)floor
          (ynormfaktor * (double)(i-lowposition) + 0.5);
        break;
      case CCDLIB_S_SchlagConst :
        if( i<wendepunkt )
          histogramLUT[ i ] = (short)floor
            (ylownormfak * pow( (double)(i-lowposition)/xlownormfak, lowgamma ) + 0.5);
        else
          histogramLUT[ i ] = ywendepkt + (short)floor
            (yhighnormfak * pow( (double)(i-wendepunkt)/xhighnormfak, highgamma ) + 0.5);
        break;
      case CCDLIB_GammaKorrekturConst :
        histogramLUT[ i ] = (short)floor
          (ynormfaktor * pow( (double)(i-lowposition)/xnormfaktor, gamma ) + 0.5);
        break;
      case CCDLIB_LogarithmusConst :
        histogramLUT[ i ] = (short)floor
          (ynormfaktor * log( (double)(i-lowposition)/xnormfaktor + 1. ) + 0.5);
        break;
      case CCDLIB_ExponentConst :
        histogramLUT[ i ] = (short)floor
          (ynormfaktor * ( exp( (double)(i-lowposition)/xnormfaktor ) - 1.) + 0.5);
        break;
      case CCDLIB_HistogramConst :
        histogramLUT[ i ] = (short)floor
          (ynormfaktor * (double)aktsum + 0.5);
        aktsum += histogramfield[i];
        break;
      default:
        return( CCDLIB_WrongParameterError );
        break;
    }
  }
  for( i=highposition; i<dynamik; i++ )
    histogramLUT[ i ] = (short)(dynamik-1);

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_ModifyHistogram: Routine zum Veraendern der Grauwertvertei-    *
*               lung eines Bildausschnitts. Die Routine arbeitet mit      *
*               einer vorher zu fuellenden HistogramLUT.                  *
*                                                                         *
*   Parameter:  *picturerect :  Koordinate der oberen linken Ecke des     *
*                               Arbeitsbereiches im Bild                  *
*               *pictureparam:  Systemparameter des zu bearbeitenden      *
*                               Bildes                                    * 
*               *sourcepic   :  32-Bit-Pointer auf die Bildquelle         *
*               *destpic     :  32-Bit-Pointer auf das Bildziel           *
*               *histogramLUT:  LUT zur Grauwerttransformation            *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesC: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerErrorConst:   Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.2 (21.09.1997)                              32 Bit      *
*   Written by: Georg Dittie, 1994-1997                                   *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_ModifyHistogram
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *sourcepic,
  short                     *destpic,
  short                     *histogramLUT )
{
  long  x, ys, xstop, ystop, ystep;


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( sourcepic == NULL || destpic == NULL)
    return( CCDLIB_NoPicturePointerError );

  if( histogramLUT == NULL )
    return( CCDLIB_NoPointerError );

  xstop = (long)(picturerect->xpos + picturerect->xsize);
  ystop = (long)(picturerect->ypos + picturerect->ysize) * pictureparam->linelength;

  if( pictureparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    for( ys = (long)(picturerect->ypos * pictureparam->linelength);
         ys < ystop;
         ys += pictureparam->linelength)
    {
      for( x = (long)(picturerect->xpos); x < xstop; x++ )
      {
        destpic[ x+ys ] = histogramLUT[ sourcepic[ x+ys ] ];
      }
    }
  }
  else
  {
    xstop *= 3;
    ystop *= 3;
    ystep = pictureparam->linelength*3;
    for( ys = (long)(picturerect->ypos * pictureparam->linelength);
         ys < ystop;
         ys += ystep )
    {
      for( x = (long)(picturerect->xpos); x < xstop; x += 3 )
      {
        destpic[ x+ys ]   = histogramLUT[ sourcepic[ x+ys ] ];
        destpic[ x+ys+1 ] = histogramLUT[ sourcepic[ x+ys+1 ] ];
        destpic[ x+ys+2 ] = histogramLUT[ sourcepic[ x+ys+2 ] ];
      }
    }
  }

  return( CCDLIB_Successful );
}

/**************************************************************************
*                                                                         *
*   CCDLIB_SubtractBackground: Routine zum Abzug eines erhoehten Unter-   *
*               grunds. Dabei wird davon ausgegangen, das der Unter-      *
*               flaechenmaessig das Bild beherrscht.                      * 
*                                                                         *
*   Parameter:  *picturerect :  Koordinate der oberen linken Ecke des     *
*                               Arbeitsbereiches im Bild                  *
*               *pictureparam:  Systemparameter des zu bearbeitenden      *
*                               Bildes                                    *
*               *sourcepic   :  32-Bit-Pointer auf die Bildquelle         *
*               *destpic     :  32-Bit-Pointer auf das Bildziel           *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Das Resultatbild steht an derselben Stelle wie das Ausgangsbild.      *
*   ACHTUNG: Das Ausgangsbild wird dabei ueberschrieben !                 *
*                                                                         *
*   Version:    1.1 (21.09.1997)                                          *
*   Written by: Georg Dittie, 1997                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_SubtractBackground
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *sourcepic,
  short                     *destpic
)
{
  long    x, ys, i, xstop, ystop;

  long    histomaximum, histomaxpos;
  double  histomaxpart, histoentropy, yfak;
  long    histolowestpos, histohighestpos;

  enum CCDLIB_ErrorEnum result;

  long    *histogramfield;
  short   *translatetab;

  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( sourcepic == NULL || destpic == NULL)
    return( CCDLIB_NoPicturePointerError );

  histogramfield = (long *)malloc( pictureparam->dynamik * sizeof(long) );
  translatetab   = (short *)malloc( pictureparam->dynamik * sizeof(short) );

  if( histogramfield == NULL || translatetab == NULL )
  {
    if( histogramfield != NULL )  free( histogramfield );
    if( translatetab   != NULL )  free( translatetab );

    return( CCDLIB_OutOfMemoryError );
  }

  result = CCDLIB_RetrieveHistogram(
    picturerect,
    pictureparam,
    sourcepic,
    histogramfield );

  if( result != CCDLIB_Successful )
  {
    free( histogramfield );
    free( translatetab );
    return( result );
  }

  result = CCDLIB_GetMaximumInHistogram(
    histogramfield,
    pictureparam->dynamik,
    &histolowestpos,
    &histohighestpos,
    &histomaxpos,
    &histomaximum,
    &histomaxpart,
    &histoentropy );

  if( result != CCDLIB_Successful )
  {
    free( histogramfield );
    free( translatetab );
    return( result );
  }

  yfak = (double)(pictureparam->dynamik-1)/(double)(pictureparam->dynamik-histomaxpos);

  for( i=0; i<histomaxpos; i++ )
  {
   translatetab[ i ] = 0;
  }
  for( i=histomaxpos; i<pictureparam->dynamik; i++ )
  {
   translatetab[ i ] = (short)
    floor( (double)(i - histomaxpos) * yfak );
  }

  xstop = (long)(picturerect->xpos + picturerect->xsize);
  ystop = (long)(picturerect->ypos + picturerect->ysize) * pictureparam->linelength;

  for( ys = (long)(picturerect->ypos * pictureparam->linelength);
       ys < ystop;
       ys += pictureparam->linelength)
  {

    for( x = (long)(picturerect->xpos); x < xstop; x++ )
    {
      destpic[ x+ys ] = translatetab[ sourcepic[ x+ys ] ];
    }
  }

  free( histogramfield );
  free( translatetab );

  return( CCDLIB_Successful );
}



/**************************************************************************
*                                                                         *
*   CCDLIB_SubtractShiftDifference: Abziehen eines verschobenen Teils     *
*               von einem Originalbild: Sichtbarmachung von Strukturen    *
*                                                                         *
*   Parameter:  *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *sourcepic   : 32-Bit-Pointer auf die Quellbilddaten      *
*               *destpic     : 32-Bit-Pointer auf die Zielbilddaten       *
*               xcenter      : X-Koordinate des Drehpunktes               *
*               ycenter      : Y-Koordinate des Drehpunktes               *
*               shiftval     : Verschiebungsstrecke (Pixel / Grad)        *
*               subintensity : Anteil des zu subtrahierenden Bildes       *
*                                                                         *
*               CCDLIB_ShiftTypeEnum : Art der Shiftoperation             *
*                                                                         *
*               CCDLIB_RotConstAngle   -> Rotation um einen konst. Win.   *
*               CCDLIB_RotConstTangent -> Rot. mit konst. Tangente        *
*               CCDLIB_ShiftConstRad   -> Shift mit konst. Radius         *
*                                                                         *
*               ACHTUNG: sourcepic darf nicht destpic sein !              *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0 (02.10.1997)                              32 Bit      *
*   Written by: Georg Dittie, 1997                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_SubtractShiftDifference
(
  CCDLIB_PictureparamStruct *picparam,
  short                     *sourcepic,
  short                     *destpic,
  long                      xcenter,
  long                      ycenter,
  double                    shiftval,
  double                    subintensity,
  enum CCDLIB_ShiftTypeEnum shifttype
)
{
  long    xs, xs1, xd, xf, yf, yss, yss1, yd, yds, xstop, ystop, ystep,
          xcent, ycent, xlimit, ylimit, xneu, yneu,
          greyval, blueval, greenval, redval,
          xm0, xm1, xm2, xm3;
  double  xdist, ydist, rdist, angle, platformval, tempval;

  if( sourcepic == NULL || destpic == NULL )
    return( CCDLIB_NoPicturePointerError );

  if( sourcepic == destpic || subintensity < 0. || subintensity > 1. )
    return( CCDLIB_WrongParameterError );

  xstop   = picparam->linelength;
  ystop   = picparam->linelength*picparam->columnheight;
  ystep   = picparam->linelength;
  xcent   = xcenter<<8;
  ycent   = ycenter<<8;
  xlimit  = xstop<<8;
  ylimit  = (picparam->columnheight)<<8;

  platformval = (double)picparam->dynamik/2.*subintensity;
  if( shifttype == CCDLIB_RotConstAngleConst )  shiftval *= MC_DEG;

  if( picparam->colorchannel != CCDLIB_RGBChannelColor )
  {
    for( yd=0, yds=0; yds<ystop; yd++, yds+=ystep )
    {
      ydist = (double)(yd-ycenter);
      for( xd=0; xd<xstop; xd++ )
      {
        xdist = (double)(xd-xcenter);
        rdist = sqrt( xdist*xdist + ydist*ydist );

        if( rdist <= 0.5 )
        {
          destpic[ yds+xd ] = sourcepic[ yds+xd ];
          continue;
        }

        switch( shifttype )
        {
          case CCDLIB_RotConstAngleConst:
            rdist *= 256.;
            angle = atan2( ydist, xdist )+shiftval;
            break;
          case CCDLIB_RotConstTangentConst:
            if( rdist <= shiftval )
              angle = atan2( ydist, xdist )+MC_PId2;
            else
              angle = atan2( ydist, xdist )+MC_PId2*shiftval/rdist;
            rdist *= 256.;
            break;
          case CCDLIB_ShiftConstRadConst:
            rdist = (rdist+shiftval) * 256.;
            if( rdist < 0. )  rdist = 0.;
            angle = atan2( ydist, xdist );
            break;
          default:
            return( CCDLIB_WrongParameterError );
            break;
        }
        xneu  = (long)(rdist*cos(angle))+xcent;
        yneu  = (long)(rdist*sin(angle))+ycent;

        if( xneu >= 0 && xneu < xlimit &&
            yneu >= 0 && yneu < ylimit )
        {
          yf = yneu & 0xFF;
          xf = xneu & 0xFF;

          xs  = xneu >> 8;
          xs1 = xs+1;

          yss  = (yneu >> 8) * ystep;
          yss1 = yss + ystep;

          /* Bilineare Interpolation aus George Wolberg "Digital Image Warping" */

          xm0 = (long)sourcepic[ yss +xs  ];
          xm1 = (long)sourcepic[ yss +xs1 ];
          xm2 = (long)sourcepic[ yss1+xs  ];
          xm3 = (long)sourcepic[ yss1+xs1 ];

          greyval = (
            ( (xm0<<16) + (xm1-xm0)*(xf<<8) + (((xm2-xm0)<<8) +
            (xm3-xm2-xm1+xm0)*xf)*yf ) >> 16 );
        }
        else
          greyval = 0;

        tempval = (double)sourcepic[ yds+xd ]
                - subintensity*(double)greyval + platformval;
        if( tempval < 0. )
          destpic[ yds+xd ] = 0;
        else if( tempval >= (double)picparam->dynamik )
          destpic[ yds+xd ] = (short)(picparam->dynamik-1);
        else
          destpic[ yds+xd ] = (short)(tempval);
      }
    }
  }
  else
  {
    xstop   *= 3;
    ystop   *= 3;
    ystep   *= 3;
    xlimit  *= 3;
    ylimit  *= 3;

    for( yd=0, yds=0; yds<ystop; yd++, yds+=ystep )
    {
      ydist = (double)(yd-ycenter);
      for( xd=0; xd<xstop; xd+=3 )
      {
        xdist = (double)(xd/3-xcenter);
        rdist = sqrt( xdist*xdist + ydist*ydist );

        if( rdist <= 0.5 )
        {
          destpic[ yds+xd   ] = sourcepic[ yds+xd   ];
          destpic[ yds+xd+1 ] = sourcepic[ yds+xd+1 ];
          destpic[ yds+xd+2 ] = sourcepic[ yds+xd+2 ];
          continue;
        }
        switch( shifttype )
        {
          case CCDLIB_RotConstAngleConst:
            rdist *= 256.;
            angle = atan2( ydist, xdist )+shiftval;
            break;
          case CCDLIB_RotConstTangentConst:
            if( rdist <= shiftval )
              angle = atan2( ydist, xdist )+MC_PId2;
            else
              angle = atan2( ydist, xdist )+MC_PId2*shiftval/rdist;
            rdist *= 256.;
            break;
          case CCDLIB_ShiftConstRadConst:
            rdist = (rdist+shiftval) * 256.;
            if( rdist < 0. )  rdist = 0.;
            angle = atan2( ydist, xdist );
            break;
          default:
            return( CCDLIB_WrongParameterError );
            break;
        }
        xneu  = (long)(rdist*cos(angle))+xcent;
        yneu  = (long)(rdist*sin(angle))+ycent;

        if( xneu >= 0 && xneu < xlimit &&
            yneu >= 0 && yneu < ylimit )
        {
          yf = yneu & 0xFF;
          xf = xneu & 0xFF;

          xs  = (xneu >> 8) * 3;
          xs1 = xs + 3;

          yss  = (yneu >> 8) * ystep;
          yss1 = yss + ystep;

          /* Bilineare Interpolation aus George Wolberg "Digital Image Warping" */

          xm0 = (long)sourcepic[ yss +xs  ];
          xm1 = (long)sourcepic[ yss +xs1 ];
          xm2 = (long)sourcepic[ yss1+xs  ];
          xm3 = (long)sourcepic[ yss1+xs1 ];

          blueval = (
            ( (xm0<<16) + (xm1-xm0)*(xf<<8) + (((xm2-xm0)<<8) +
            (xm3-xm2-xm1+xm0)*xf)*yf ) >> 16 );

          xm0 = (long)sourcepic[ yss +xs +1 ];
          xm1 = (long)sourcepic[ yss +xs1+1 ];
          xm2 = (long)sourcepic[ yss1+xs +1 ];
          xm3 = (long)sourcepic[ yss1+xs1+1 ];

          greenval = (
            ( (xm0<<16) + (xm1-xm0)*(xf<<8) + (((xm2-xm0)<<8) +
            (xm3-xm2-xm1+xm0)*xf)*yf ) >> 16 );

          xm0 = (long)sourcepic[ yss +xs +2 ];
          xm1 = (long)sourcepic[ yss +xs1+2 ];
          xm2 = (long)sourcepic[ yss1+xs +2 ];
          xm3 = (long)sourcepic[ yss1+xs1+2 ];

          redval = (
            ( (xm0<<16) + (xm1-xm0)*(xf<<8) + (((xm2-xm0)<<8) +
            (xm3-xm2-xm1+xm0)*xf)*yf ) >> 16 );
        }
        else
        {
          blueval  = 0;
          greenval = 0;
          redval   = 0;
        }
        tempval = (double)sourcepic[ yds+xd ]
                - subintensity*(double)blueval +platformval;
        if( tempval < 0. )
          destpic[ yds+xd ] = 0;
        else if( tempval >= (double)picparam->dynamik )
          destpic[ yds+xd ] = (short)(picparam->dynamik-1);
        else
          destpic[ yds+xd ] = (short)(tempval);

        tempval = (double)sourcepic[ yds+xd+1 ]
                - subintensity*(double)greenval +platformval;
        if( tempval < 0. )
          destpic[ yds+xd+1 ] = 0;
        else if( tempval >= (double)picparam->dynamik )
          destpic[ yds+xd+1 ] = (short)(picparam->dynamik-1);
        else
          destpic[ yds+xd+1 ] = (short)(tempval);

        tempval = (double)sourcepic[ yds+xd+2 ]
                - subintensity*(double)redval +platformval;
        if( tempval < 0. )
          destpic[ yds+xd+2 ] = 0;
        else if( tempval >= (double)picparam->dynamik )
          destpic[ yds+xd+2 ] = (short)(picparam->dynamik-1);
        else
          destpic[ yds+xd+2 ] = (short)(tempval);
      }
    }
  }
  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_CalcXYGradientFromPic : Routine zum Berechnen des volstaen-    *
*               digen Gradientenvektors aus einem Grauwertbild            *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *picture     : 32-Bit-Pointer auf die Bilddaten           *
*                                                                         *
*               ACHTUNG: maskdiameter darf nur ungerade Werte annehmen!   *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*               *gradient    :  32-Bit-Pointer auf die Gradientenwerte    *
*               Ein Gradientenwert ist eine Union aus einem 32-Bit-Wert   *
*               und dem X- und Y-Anteil des Vektors als short             *
*                                                                         *
*   Version:    1.0 (05.06.1994)                                          *
*   Written by: Georg Dittie, 1994                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_CalcXYGradientFromPic
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  CCDLIB_GradientUnion      *gradient
)
{
  long  x, ys, ysm1, ysp1, xstart, xstop, ystart, ystop;
  
  short grad;


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL || gradient == NULL )
    return( CCDLIB_NoPicturePointerError );

  xstart = picturerect->xpos + 1;
  xstop  = picturerect->xpos + picturerect->xsize - 1;
  ystart = ( picturerect->ypos + 1 ) * pictureparam->linelength;
  ystop  = ( picturerect->ypos + picturerect->ysize - 1) * pictureparam->linelength;

  ysm1 = ystart - pictureparam->linelength;
  ysp1 = ystart + pictureparam->linelength;
  for( ys = ystart; ys < ystop; ys += pictureparam->linelength )
  {
    for( x = xstart; x < xstop; x++ )
    {
      grad = (short)( ( (long)picture[ ys+x+1 ] - (long)picture[ ys+x-1 ] ) >> 1 );

      gradient[ ys+x ].value.xgrad = (short)(
        max( 0, min( pictureparam->dynamik-1, grad ) ) );

      grad = (short)( ( (long)picture[ ysp1+x ] - (long)picture[ ysm1+x ] ) >> 1 );
      
      gradient[ ys+x ].value.ygrad = (short)(
        max( 0, min( pictureparam->dynamik-1, grad ) ) );
    }

    ysm1 += pictureparam->linelength;
    ysp1 += pictureparam->linelength;
  }

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_SobelGradientFromPic : Routine zum Berechnen des volstaen-     *
*               digen Gradientenvektors aus einem Grauwertbild mit        *
*               Hilfe des rauschmindernden Sobel-Operators                *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *picture     : 32-Bit-Pointer auf die Bilddaten           *
*                                                                         *
*               ACHTUNG: maskdiameter darf nur ungerade Werte annehmen!   *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*               *gradient    :  32-Bit-Pointer auf die Gradientenwerte    *
*               Ein Gradientenwert ist eine Union aus einem 32-Bit-Wert   *
*               und dem X- und Y-Anteil des Vektors als short             *
*                                                                         *
*   Version:    1.0 (05.06.1994)                                          *
*   Written by: Georg Dittie, 1994                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_SobelGradientFromPic
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  CCDLIB_GradientUnion      *gradient
)
{
  long  x, xm1, xp1, ys, ysm1, ysp1,
        xstart, xstop, ystart, ystop;
  
  short grad;
  long  leftvalue, rightvalue, upvalue, downvalue;


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL || gradient == NULL )
    return( CCDLIB_NoPicturePointerError );

  xstart = picturerect->xpos + 1;
  xstop  = picturerect->xpos + picturerect->xsize - 1;
  ystart = ( picturerect->ypos + 1 ) * pictureparam->linelength;
  ystop  = ( picturerect->ypos + picturerect->ysize - 1) * pictureparam->linelength;

  ysm1 = ystart - pictureparam->linelength;
  ysp1 = ystart + pictureparam->linelength;
  for( ys = ystart; ys < ystop; ys += pictureparam->linelength )
  {
    xm1 = xstart - 1;
    xp1 = xstart + 1;
    for( x = xstart; x < xstop; x++ )
    {
      leftvalue = ( (long)picture[ ysm1+xm1 ]
                + (long)picture[ ys+xm1 ]
                + (long)picture[ ysp1+xm1 ] ) / 3;

      rightvalue  = ( (long)picture[ ysm1+xp1 ]
                  + (long)picture[ ys+xp1 ]
                  + (long)picture[ ysp1+xp1 ] ) / 3;

      grad = (short)( ( rightvalue - leftvalue ) >> 1 );
      
      gradient[ ys+x ].value.xgrad = (short)(
        max( 0, min( pictureparam->dynamik-1, grad ) ) );

      upvalue = ( (long)picture[ ysm1+xp1 ]
              + (long)picture[ ysm1+x ]
              + (long)picture[ ysm1+xm1 ] ) / 3;

      downvalue = ( (long)picture[ ysp1+xp1 ]                       
                + (long)picture[ ysp1+x ]
                + (long)picture[ ysp1+xm1 ] ) / 3;

      grad = (short)( ( upvalue - downvalue ) >> 1 );
      
      gradient[ ys+x ].value.ygrad = (short)(
        max( 0, min( pictureparam->dynamik-1, grad ) ) );

      xm1++;
      xp1++;
    }

    ysm1 += pictureparam->linelength;
    ysp1 += pictureparam->linelength;
  }

  return( CCDLIB_Successful );
}
                                       

/**************************************************************************
*                                                                         *
*   CCDLIB_CalcAbsGradientFromPic : Routine zum Berechnen der Gradien-    *
*               tenamplitude ( Betragswert ) Aus Geschwindigkeitsgruen-   *
*               den kann optional eine auf 4% genaue Naeherungsmethode    *
*               benutzt werden.                                           *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *picture     : 32-Bit-Pointer auf die Bilddaten           *
*               *gradient    : 32-Bit-Pointer auf die Gradientenwerte     *
*                                                                         *
*               Ein Gradientenwert ist eine Union aus einem 32-Bit-Wert   *
*               und dem X- und Y-Anteil des Vektors als short             *
*               CCDLIB_GradientOperationEnum : Methodenauswahl            *
*                                                                         *
*               ACHTUNG: maskdiameter darf nur ungerade Werte annehmen!   *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0 (05.06.1994)                                          *
*   Written by: Georg Dittie, 1994                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_CalcAbsGradientFromPic
(
  CCDLIB_RectangleStruct            *picturerect,
  CCDLIB_PictureparamStruct         *pictureparam,
  short                             *picture,
  CCDLIB_GradientUnion              *gradient,
  enum CCDLIB_GradientOperationEnum operationtype
)
{
  long    avalue, bvalue, value;
  double  avaluef, bvaluef, valuef;
  long    afactor, bfactor, halfdynamikbits;
  
  long    x, ys, xstart, xstop, ystart, ystop;


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL || gradient == NULL )
    return( CCDLIB_NoPicturePointerError );

  afactor = (long)floor( (double)pictureparam->dynamik * 0.960 + 0.5 );
  bfactor = (long)floor( (double)pictureparam->dynamik * 0.398 + 0.5 );
  halfdynamikbits = (long)floor( log( pictureparam->dynamik ) / log( 2. ) + 0.5 ) - 1;

  xstart = picturerect->xpos + 1;
  xstop  = picturerect->xpos + picturerect->xsize - 1;
  ystart = ( picturerect->ypos + 1 ) * pictureparam->linelength;
  ystop  = ( picturerect->ypos + picturerect->ysize - 1) * pictureparam->linelength;

  for( ys = ystart; ys < ystop; ys += pictureparam->linelength )
  {
    switch( operationtype )
    {
      case CCDLIB_AproximationConst:
        for( x = xstart; x < xstop; x++ )
        {
          avalue = (long)abs( gradient[ ys+x ].value.xgrad );
          bvalue = (long)abs( gradient[ ys+x ].value.ygrad );
        
          /* Naeherung geometrische Addition aus Bronstein Semendjajew */
        
          if( avalue > bvalue )
          {
            value = ( afactor*avalue + bfactor*bvalue ) >> halfdynamikbits;
          }
          else
          {
            value = ( afactor*bvalue + bfactor*avalue ) >> halfdynamikbits;
          }
  
          picture[ x+ys ] = (short)(
          max( 0, min( (long)pictureparam->dynamik-1, value ) ) );
        }
        break;

      case CCDLIB_ExactConst:
        for( x = xstart; x < xstop; x++ )
        {
          avaluef = (double)( gradient[ ys+x ].value.xgrad );
          bvaluef = (double)( gradient[ ys+x ].value.ygrad );
          valuef = (short)floor( sqrt( avaluef*avaluef + bvaluef*bvaluef ) + 0.5 );
          picture[ x+ys ] = ( max( 0, min( pictureparam->dynamik-1, valuef ) ) );
        }
        break;

      default:
        return( CCDLIB_WrongParameterError );
        break;
    }
  }

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_BinarizePicture : Routine zum Binarisieren eines Grauwert-     *
*               bildes. Dabei koennen die beiden zuzuordnenden Grau-      *
*               werte gewaehlt werden.                                    *
*                                                                         *
*   Parameter:  *picturerect : Koordinate der oberen linken Ecke des      *
*                              Arbeitsbereiches im Bild                   *
*               *pictureparam: Systemparameter des zu bearbeitenden       *
*                              Bildes                                     *
*               *picture     : 32-Bit-Pointer auf die Bilddaten           *
*               treshold     : Binarisierungsschwelle                     *
*               logreyval    : Grauwert fuer Werte unter der Schwelle     *
*               higreyval    : Grauwert fuer Werte ueber der Schwelle     *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Das Resultatbild steht an derselben Stelle wie das Ausgangsbild.      *
*   ACHTUNG: Das Ausgangsbild wird dabei ueberschrieben !                 *
*                                                                         *
*   Version:    1.0 (05.06.1994)                                          *
*   Written by: Georg Dittie, 1994                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_BinarizePicture
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam,
  short                     *picture,
  short                     treshold,
  short                     logreyval,
  short                     higreyval
)
{
  long  x, ys, xstart, ystart, xstop, ystop;
  
  short value;


  if( CCDLIB_CheckRectangleValues( picturerect, pictureparam ) == FALSE )
    return( CCDLIB_WrongRectangleValuesError );

  if( picture == NULL )
    return( CCDLIB_NoPicturePointerError );

  xstart = picturerect->xpos;
  xstop  = picturerect->xpos + picturerect->xsize;
  ystart = ( picturerect->ypos ) * pictureparam->linelength;
  ystop  = ( picturerect->ypos + picturerect->ysize ) * pictureparam->linelength;

  for( ys = ystart; ys < ystop; ys += pictureparam->linelength)
  {
    for( x = xstart; x < xstop; x++ )
    {
      value = picture[ x+ys ];
      picture[ x+ys ] = ( value > treshold ? higreyval : logreyval );
    }
  }

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_CheckRectangleValues :  checks correctness of rectangular      *
*                                  values and imagesize                   *
*                                                                         *
*   parameters: *picturerect  :  rectangular values                       *
*               *pictureparam :  image parameters                         *
*                                                                         *
*   results:    TRUE (0)    : values are ok                               *
*               FALSE (!=0) : values are incorrect                        *
*                                                                         *
*   Version:    1.0 (18.05.1996)                                          *
*   Written by: Georg Dittie, 1996                                        *
*                                                                         *
**************************************************************************/

BOOL CCDLIB_CheckRectangleValues
(
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam
)
{
  /* check pointers */
  if( (picturerect == NULL) || (pictureparam == NULL) )   return( FALSE );

  /* find out if upper left corner is inside the image */
  if( picturerect->xpos < 0 ) return( FALSE );
  if( picturerect->ypos < 0 ) return( FALSE );
  if( picturerect->xpos >= pictureparam->linelength ) return( FALSE );
  if( picturerect->ypos >= pictureparam->columnheight ) return( FALSE );

  /* check if sizes are correct */
  if( picturerect->xsize < 0 ) return( FALSE );
  if( picturerect->ysize < 0 ) return( FALSE );
  if( picturerect->xpos + picturerect->xsize > pictureparam->linelength ) 
    return( FALSE );
  if( picturerect->ypos + picturerect->ysize > pictureparam->columnheight ) 
    return( FALSE );

  return( TRUE );
}

/*** Funktionen zur Matrizen- und Vectorrechnung ***/

BOOL CCDLIB_Matrixalloc
(
  double ***matrix,
  int size
)
{
  int i,j;


  (*matrix) = (double **)malloc( size*sizeof( double* ) );
  if( (*matrix) == NULL )
    return( FALSE );

  (*matrix)[ 0 ] = (double *)malloc( size*size*sizeof( double ) );
  if( (*matrix)[ 0 ] == NULL )
  {
    free( *matrix );
    return( FALSE );
  }

  for( i = 1; i < size; i++ )
    (*matrix)[ i ] = (*matrix)[ 0 ] + size*i;

  for( i = 0; i < size; i++ )
    for( j = 0; j < size; j++ )
      (*matrix)[ i ][ j ] = 0.;

  return( TRUE );
}


void CCDLIB_Matrixfree
(
   double **matrix
)
{
  free( matrix[ 0 ] );
  free( matrix );
}


void CCDLIB_Matrixcopy
(
  double **destmat,
  double **srcmat,
  int    size
)
{
  int x, y;

  for( y=0; y<size; y++ )
    for( x=0; x<size; x++ )
      destmat[x][y] = srcmat[x][y];
}


void CCDLIB_Matrixmul
(
  double **result,
  double **mat1,
  double **mat2,
  int    size
)
{
  int     x, y, k;
  double  sum, A, B;

  for( y=0; y<size; y++ )
  {
    for( x=0; x<size; x++ )
    {
      sum = 0.;
      for( k=0; k<size; k++ )
      {
        A = mat1[k][y];
        B = mat2[x][k];
        if( A != 0. || B != 0. )
          sum += A * B;
      }
      result[x][y] = sum;
    }
  }
}


/* Sieht idiotisch aus, aber der Compiler macht prozessoroptimalen
  Code draus ! Geschwindigkeitssteigerung etwa 50 % */

void CCDLIB_FastMatVecMul
(
  double *result,
  double **matrix,
  double *vector
)
{
  double sum, M, V0, V1, V2, V3, *mx;

  mx = matrix[0];
  V0 = vector[0];
  V1 = vector[1];
  V2 = vector[2];
  V3 = vector[3];

  sum = 0.;
  M = mx[0];
  if( M != 0. )
    if( V0 != 0. )
      sum += M * V0;
  M = mx[4];
  if( M != 0. )
    if( V1 != 0. )
      sum += M * V1;
  M = mx[8];
  if( M != 0. )
    if( V2 != 0. )
      sum += M * V2;
  M = mx[12];
  if( M != 0. )
    if( V3 != 0. )
      sum += M * V3;
  result[0] = sum;

  sum = 0.;
  M = mx[1];
  if( M != 0. )
    if( V0 != 0. )
      sum += M * V0;
  M = mx[5];
  if( M != 0. )
    if( V1 != 0. )
      sum += M * V1;
  M = mx[9];
  if( M != 0. )
    if( V2 != 0. )
      sum += M * V2;
  M = mx[13];
  if( M != 0. )
    if( V3 != 0. )
      sum += M * V3;
  result[1] = sum;

  sum = 0.;
  M = mx[2];
  if( M != 0. )
    if( V0 != 0. )
      sum += M * V0;
  M = mx[6];
  if( M != 0. )
    if( V1 != 0. )
      sum += M * V1;
  M = mx[10];
  if( M != 0. )
    if( V2 != 0. )
      sum += M * V2;
  M = mx[14];
  if( M != 0. )
    if( V3 != 0. )
      sum += M * V3;
  result[2] = sum;

  sum = 0.;
  M = mx[3];
  if( M != 0. )
    if( V0 != 0. )
      sum += M * V0;
  M = mx[7];
  if( M != 0. )
    if( V1 != 0. )
      sum += M * V1;
  M = mx[11];
  if( M != 0. )
    if( V2 != 0. )
      sum += M * V2;
  M = mx[15];
  if( M != 0. )
    if( V3 != 0. )
      sum += M * V3;
  result[3] = sum;
}


void CCDLIB_Matrixvectormul
(
  double  *result,
  double  **matrix,
  double  *vector,
  int     size
)
{
  int     x, y;
  double  sum, M, V;

  for( y=0; y<size; y++ )
  {
    sum = 0.;
    for( x=0; x<size; x++ )
    {
      M = matrix[x][y];
      V = vector[x];
      if( M != 0. || V != 0. )
        sum += M * V;
    }
    result[y] = sum;
  }
}


/*** Passende Ma"se berechnen ***/

void CCDLIB_GetMatchingScale
(
  CCDLIB_RectangleStruct    *matchingrect,
  CCDLIB_RectangleStruct    *picturerect,
  CCDLIB_PictureparamStruct *pictureparam
)
{
  if( (double)picturerect->xsize/(double)pictureparam->linelength <
      (double)picturerect->ysize/(double)pictureparam->columnheight )
  {
    matchingrect->xpos = 0;
    matchingrect->ypos = 0;
    matchingrect->xsize = (int)floor(
      (double)picturerect->xsize
      / (double)picturerect->ysize
      * (double)pictureparam->columnheight);
    matchingrect->ysize = pictureparam->columnheight;
  }
  else
  {
    matchingrect->xpos = 0;
    matchingrect->ypos = 0;
    matchingrect->xsize = pictureparam->linelength;
    matchingrect->ysize = (int)floor(
      (double)picturerect->ysize
      / (double)picturerect->xsize
      * (double)pictureparam->linelength);
  }
}

/*** Ortstransformationen ***/

BOOL CCDLIB_GetProjectionPoint
(
  double *xp,
  double *yp,
  double xobj,
  double yobj,
  double zobj,
  double **projmatrix,
  long   linelength,
  long   columnheight,
  long   srclength,
  long   srcheight,
  double limitwidth
)
{
  double  vector[ 4 ],
          result[ 4 ] = { 0., 0., 0., 0. };
  double  xmid, ymid, xsrc, ysrc, xlimit, ylimit;
  BOOL    visible;

  xmid  = (double)(linelength >> 1);
  ymid  = (double)(columnheight >> 1);
  xsrc  = (double)(srclength >> 1);
  ysrc  = (double)(srcheight >> 1);

  vector[ 0 ] = xobj - xmid;
  vector[ 1 ] = ymid - yobj;
  vector[ 2 ] = zobj;
  vector[ 3 ] = 1.;

  CCDLIB_FastMatVecMul( result, (double**)projmatrix, vector );

  if( result[ 3 ] == 0. )
    return( FALSE );
  *xp = result[ 0 ] / result[ 3 ];
  *yp = result[ 1 ] / result[ 3 ];

  xlimit = xsrc-limitwidth;
  ylimit = ysrc-limitwidth;

  if( *xp < -xlimit || *xp >= xlimit ||
      *yp < -ylimit || *yp >= ylimit )
    visible = FALSE;
  else
    visible = TRUE;

  *xp = *xp + xsrc;
  *yp = ysrc - *yp;

  return( visible );
}

/*** Berechnung der normierten Koordinatenmatrix fuer CalcRotScale ***/

enum CCDLIB_ErrorEnum CCDLIB_CalcRotScalematrix
(
  double **projmatrix,
  double rotation,
  double magnification,
  double xpixelsize,
  double ypixelsize
)
{
  double **Ps = NULL, **P = NULL;

  if( magnification <= 0. || xpixelsize == 0. || ypixelsize == 0. )
    return( CCDLIB_WrongParameterError );

  if( projmatrix == NULL )
    return( CCDLIB_NoPointerError );

  if( !CCDLIB_Matrixalloc( &Ps, 4 ) )
  {
    return( CCDLIB_OutOfMemoryError );
  }

  if( !CCDLIB_Matrixalloc( &P, 4 ) )
  {
    CCDLIB_Matrixfree( Ps );
    return( CCDLIB_OutOfMemoryError );
  }

  Ps[ 0 ][ 0 ] = cos( -rotation );
  Ps[ 1 ][ 0 ] = sin( -rotation );
  Ps[ 0 ][ 1 ] = -sin( -rotation );
  Ps[ 1 ][ 1 ] = cos( -rotation );
  Ps[ 2 ][ 2 ] = 1.;
  Ps[ 3 ][ 3 ] = 1.;

  P[ 0 ][ 0 ] = xpixelsize/ypixelsize;
  P[ 1 ][ 1 ] = 1.;
  P[ 2 ][ 3 ] = 1.;
  P[ 3 ][ 3 ] = magnification;

  CCDLIB_Matrixmul( projmatrix, P, Ps, 4 );

  CCDLIB_Matrixfree( Ps );
  CCDLIB_Matrixfree( P );

  return( CCDLIB_Successful );
}


/**************************************************************************
*                                                                         *
*   CCDLIB_CalcRotScale : Routine zur Entzerrung, Skalierung und          *
*               Rotation eines beliebigen Bildausschnitts                 *
*                                                                         *
*   Parameter:  *picturerect : Zu entzerrender Ausschnitt im Zielbild     *
*               *pictureparam: Systemparameter des Quellbilds             *
*               *picture     : 32-Bit-Pointer auf die Bilddaten           *
*               rotation     : Positionswinkel des Ausschnitts            *
*               magnification: Vergroesserung des Ausschnitts             *
*                                                                         *
*   Ergebnis:   CCDLIB_ErrorEnum : Detaillierte Fehler/Erfolgsmeldung     *
*                                                                         *
*               CCDLIB_Successful:            Alles OK !                  *
*               CCDLIB_WrongParameterError:   Unsinnige Werte bekommen    *
*               CCDLIB_WrongRectangleValuesE: Unsinnige Bildmasse         *
*               CCDLIB_NoPointerError:        Nicht initialisierte Ptr.   *
*               CCDLIB_NoPicturePointerError: Keine Bilddaten bekommen    *
*               CCDLIB_OutOfMemoryError:      Zu wenig Speicher           *
*                                                                         *
*   Version:    1.0 (24.12.1997)                               32 Bit     *
*   Written by: Georg Dittie, 1997                                        *
*                                                                         *
**************************************************************************/

enum CCDLIB_ErrorEnum CCDLIB_CalcRotScale
(
  CCDLIB_PictureparamStruct     *sourcepicpar,
  CCDLIB_PictureparamStruct     *destpicpar,
  short                         *sourcepic,
  short                         *destpic,
  long                          turnxpos,
  long                          turnypos,
  double                        rotation,
  double                        magnification,
  enum CCDLIB_EnlargeMethodEnum method
)
{
  double  **projmatrix;

  double  xp, yp;
  double  xm00, xm01, xm02, xm03,
          xm10, xm11, xm12, xm13,
          xm20, xm21, xm22, xm23,
          xm30, xm31, xm32, xm33;
  long    v00, v01, v10, v11, xfrac, yfrac;
  double  *sintab0, *sintab1, *sintab2, *sintab3, sinsum, nenner, angle, limitwidth;
  int     x, xq, ys, yq, xqstart, yqstart,
          xstart, xstop, ysstart, ysstop, ystep, yqstep,
          xq0, xq1, xq2, xq3, yq0, yq1, yq2, yq3;


  if( sourcepic == NULL || destpic == NULL )
    return( CCDLIB_NoPicturePointerError );

  if( sourcepic == destpic )
    return( CCDLIB_WrongParameterError );

  if( !CCDLIB_Matrixalloc( &projmatrix, 4 ) )
    return( CCDLIB_OutOfMemoryError );

  xqstart = (long)floor((turnxpos-sourcepicpar->linelength/2)*magnification
          + sourcepicpar->linelength/2.
          * (1.-sourcepicpar->ypixelsize/sourcepicpar->xpixelsize)
          * magnification+0.5)
          - (destpicpar->linelength-sourcepicpar->linelength)/2.;
  xstart  = 0;
  xstop   = destpicpar->linelength;
  yqstart = (long)floor((turnypos-sourcepicpar->columnheight/2)*magnification+0.5)
          - (destpicpar->columnheight-sourcepicpar->columnheight)/2.;
  ysstart = 0;
  ysstop  = destpicpar->linelength * destpicpar->columnheight;
  ystep   = destpicpar->linelength;
  yqstep  = sourcepicpar->linelength;
  limitwidth = 0.;

  if( method == CCDLIB_Lanczos2Const )
  {
    limitwidth = 1.;

    sintab0 = (double *)malloc( 256*sizeof( double ));
    sintab1 = (double *)malloc( 256*sizeof( double ));
    sintab2 = (double *)malloc( 256*sizeof( double ));
    sintab3 = (double *)malloc( 256*sizeof( double ));

    if( sintab0 == NULL || sintab0 == NULL || sintab0 == NULL || sintab0 == NULL )
    {
      if( sintab0 != NULL ) free( sintab0 );
      if( sintab1 != NULL ) free( sintab1 );
      if( sintab2 != NULL ) free( sintab2 );
      if( sintab3 != NULL ) free( sintab3 );
      return( CCDLIB_OutOfMemoryError );
    }

    for( xfrac = 0; xfrac < 256; xfrac++ )
    {
      nenner = MC_PId2*( (double)xfrac/256. - 2. );
      angle  = MC_PI*(double)xfrac/256. - MC_2PI;
      if( nenner != 0. )
        sintab0[ xfrac ] = sin(nenner)/nenner*sin(angle)/angle;
      else
        sintab0[ xfrac ] = 1.;

      nenner = MC_PId2*( (double)xfrac/256. - 1. );
      angle  = MC_PI*(double)xfrac/256. - MC_PI;
      if( nenner != 0. )
        sintab1[ xfrac ] = sin(nenner)/nenner*sin(angle)/angle;
      else
        sintab1[ xfrac ] = 1.;

      nenner = MC_PId2*( (double)xfrac/256. );
      angle  = MC_PI*(double)xfrac/256.;
      if( nenner != 0. )
        sintab2[ xfrac ] = sin(nenner)/nenner*sin(angle)/angle;
      else
        sintab2[ xfrac ] = 1.;

      nenner = MC_PId2*( (double)xfrac/256. + 1. );
      angle  = MC_PI*(double)xfrac/256. + MC_PI;
      if( nenner != 0. )
        sintab3[ xfrac ] = sin(nenner)/nenner*sin(angle)/angle;
      else
        sintab3[ xfrac ] = 1.;

      sinsum = sintab0[ xfrac ]+sintab1[ xfrac ]+sintab2[ xfrac ]+sintab3[ xfrac ];
      sinsum = sinsum;

      sintab0[ xfrac ] /= sinsum;
      sintab1[ xfrac ] /= sinsum;
      sintab2[ xfrac ] /= sinsum;
      sintab3[ xfrac ] /= sinsum;
    }
  }

  if( CCDLIB_CalcRotScalematrix(
      (double **)projmatrix,
      rotation,
      magnification,
      sourcepicpar->xpixelsize,
      sourcepicpar->ypixelsize ) != CCDLIB_Successful )
    return( CCDLIB_WrongParameterError );

  if( sourcepicpar->colorchannel != CCDLIB_RGBChannelColor )
  {
    for( ys = ysstart, yq = yqstart; ys < ysstop; ys += ystep, yq++ )
    {
      for( x = xstart, xq = xqstart; x < xstop; x++, xq++ )
      {
        if( CCDLIB_GetProjectionPoint(
              &xp, &yp, (double)xq, (double)yq, 0., (double**)projmatrix,
              destpicpar->linelength, destpicpar->columnheight,
              sourcepicpar->linelength, sourcepicpar->columnheight, limitwidth ) == FALSE )
        {
          destpic[ x+ys ] = destpicpar->blankvalue.word;
        }
        else
        {
          switch( method )
          {
            case CCDLIB_GetSampleConst:               /* Verkleinerung durch Undersamplen */
            case CCDLIB_NearestNeighborConst:         /* Naechster Nachbar-Kloetzelgrafik */
              xq0 = (int)floor( xp+0.5 );
              yq0 = (int)floor( yp+0.5 ) * yqstep;
              destpic[ x+ys ] = sourcepic[ xq0+yq0 ];
              break;

            case CCDLIB_MeanOfVicinityConst:          /* Verkleinerung durch Mitteln */
            case CCDLIB_LinearIntpolConst:            /* Lineare Interpolation */
              xq0 = (int)floor( xp );
              xq1 = xq0+1;
              yq0 = (int)floor( yp ) * yqstep;
              yq1 = yq0 + yqstep;
  
              xfrac = (long)( frac( xp )*256. );
              yfrac = (long)( frac( yp )*256. );
  
              v00 = (long)sourcepic[ xq0+yq0 ];
              v10 = (long)sourcepic[ xq1+yq0 ];
              v01 = (long)sourcepic[ xq0+yq1 ];
              v11 = (long)sourcepic[ xq1+yq1 ];
  
              destpic[ x+ys ] = (short)(
                ( (v00<<16) + (v10-v00)*(xfrac<<8) + (((v01-v00)<<8) +
                (v11-v01-v10+v00)*xfrac)*yfrac ) >> 16 );
              break;

            case CCDLIB_Lanczos2Const:
              xq1 = (int)floor( xp );
              xq0 = --xq1;
              xq2 = ++xq0;
              xq3 = ++xq1;
              yq1 = (int)floor( yp ) * yqstep;
              yq0 = yq1 - yqstep;
              yq2 = yq1 + yqstep;
              yq3 = yq2 + yqstep;
  
              xfrac = (long)( frac( xp )*256. );
              yfrac = (long)( frac( yp )*256. );
  
              xm00 = (double)sourcepic[ yq0+xq0 ];
              xm10 = (double)sourcepic[ yq0+xq1 ];
              xm20 = (double)sourcepic[ yq0+xq2 ];
              xm30 = (double)sourcepic[ yq0+xq3 ];
              xm01 = (double)sourcepic[ yq1+xq0 ];
              xm11 = (double)sourcepic[ yq1+xq1 ];
              xm21 = (double)sourcepic[ yq1+xq2 ];
              xm31 = (double)sourcepic[ yq1+xq3 ];
              xm02 = (double)sourcepic[ yq2+xq0 ];
              xm12 = (double)sourcepic[ yq2+xq1 ];
              xm22 = (double)sourcepic[ yq2+xq2 ];
              xm32 = (double)sourcepic[ yq2+xq3 ];
              xm03 = (double)sourcepic[ yq3+xq0 ];
              xm13 = (double)sourcepic[ yq3+xq1 ];
              xm23 = (double)sourcepic[ yq3+xq2 ];
              xm33 = (double)sourcepic[ yq3+xq3 ];
  
              destpic[ ys+x ] = (short)floor(
                ( xm00 * sintab0[ xfrac ] * sintab0[ yfrac ]
                + xm10 * sintab1[ xfrac ] * sintab0[ yfrac ]
                + xm20 * sintab2[ xfrac ] * sintab0[ yfrac ]
                + xm30 * sintab3[ xfrac ] * sintab0[ yfrac ]
                + xm01 * sintab0[ xfrac ] * sintab1[ yfrac ]
                + xm11 * sintab1[ xfrac ] * sintab1[ yfrac ]
                + xm21 * sintab2[ xfrac ] * sintab1[ yfrac ]
                + xm31 * sintab3[ xfrac ] * sintab1[ yfrac ]
                + xm02 * sintab0[ xfrac ] * sintab2[ yfrac ]
                + xm12 * sintab1[ xfrac ] * sintab2[ yfrac ]
                + xm22 * sintab2[ xfrac ] * sintab2[ yfrac ]
                + xm32 * sintab3[ xfrac ] * sintab2[ yfrac ]
                + xm03 * sintab0[ xfrac ] * sintab3[ yfrac ]
                + xm13 * sintab1[ xfrac ] * sintab3[ yfrac ]
                + xm23 * sintab2[ xfrac ] * sintab3[ yfrac ]
                + xm33 * sintab3[ xfrac ] * sintab3[ yfrac ] ) );
              break;

            default:
              return( CCDLIB_WrongParameterError );
              break;
          }
        }
      }
    }
  }
  else
  {
    ystep   *= 3;
    yqstep  *= 3;
    xstart  *= 3;
    xstop   *= 3;
    ysstart *= 3;
    ysstop  *= 3;
    for( ys = ysstart, yq = yqstart; ys < ysstop; ys += ystep, yq++ )
    {
      for( x = xstart, xq = xqstart; x < xstop; x+=3, xq++ )
      {
        if( CCDLIB_GetProjectionPoint(
              &xp, &yp, (double)xq, (double)yq, 0., (double**)projmatrix,
              destpicpar->linelength, destpicpar->columnheight,
              sourcepicpar->linelength, sourcepicpar->columnheight, limitwidth ) == FALSE )
        {
          destpic[ x+ys+2 ] = destpicpar->blankvalue.word;
          destpic[ x+ys+1 ] = destpicpar->gblankvalue.word;
          destpic[ x+ys   ] = destpicpar->bblankvalue.word;
        }
        else
        {
          switch( method )
          {
            case CCDLIB_GetSampleConst:               /* Verkleinerung durch Undersamplen */
            case CCDLIB_NearestNeighborConst:         /* Naechster Nachbar-Kloetzelgrafik */
              xq0 = (int)floor( xp+0.5 ) * 3;
              yq0 = (int)floor( yp+0.5 ) * yqstep;
  
              destpic[ x+ys   ] = sourcepic[ xq0+yq0 ];
              destpic[ x+ys+1 ] = sourcepic[ xq0+yq0+1 ];
              destpic[ x+ys+2 ] = sourcepic[ xq0+yq0+2 ];
              break;
  
            case CCDLIB_MeanOfVicinityConst:          /* Verkleinerung durch Mitteln */
            case CCDLIB_LinearIntpolConst:            /* Lineare Interpolation */
              xq0 = (int)floor( xp ) * 3;
              xq1 = xq0 + 3;
              yq0 = (int)floor( yp ) * yqstep;
              yq1 = yq0 + yqstep;
  
              xfrac = (long)( frac( xp )*256. );
              yfrac = (long)( frac( yp )*256. );
  
              v00 = (long)sourcepic[ xq0+yq0 ];
              v10 = (long)sourcepic[ xq1+yq0 ];
              v01 = (long)sourcepic[ xq0+yq1 ];
              v11 = (long)sourcepic[ xq1+yq1 ];
  
              destpic[ x+ys ] = (short)(
                ( (v00<<16) + (v10-v00)*(xfrac<<8) + (((v01-v00)<<8) +
                (v11-v01-v10+v00)*xfrac)*yfrac ) >> 16 );
  
              v00 = (long)sourcepic[ xq0+yq0+1 ];
              v10 = (long)sourcepic[ xq1+yq0+1 ];
              v01 = (long)sourcepic[ xq0+yq1+1 ];
              v11 = (long)sourcepic[ xq1+yq1+1 ];
  
              destpic[ x+ys+1 ] = (short)(
                ( (v00<<16) + (v10-v00)*(xfrac<<8) + (((v01-v00)<<8) +
                (v11-v01-v10+v00)*xfrac)*yfrac ) >> 16 );
  
              v00 = (long)sourcepic[ xq0+yq0+2 ];
              v10 = (long)sourcepic[ xq1+yq0+2 ];
              v01 = (long)sourcepic[ xq0+yq1+2 ];
              v11 = (long)sourcepic[ xq1+yq1+2 ];
  
              destpic[ x+ys+2 ] = (short)(
                ( (v00<<16) + (v10-v00)*(xfrac<<8) + (((v01-v00)<<8) +
                (v11-v01-v10+v00)*xfrac)*yfrac ) >> 16 );
              break;
  
            case CCDLIB_Lanczos2Const:
              xq1 = (int)floor( xp ) * 3;
              xq0 = xq1-3;
              xq2 = xq0+3;
              xq3 = xq1+6;
              yq1 = (int)floor( yp ) * yqstep;
              yq0 = yq1 - yqstep;
              yq2 = yq1 + yqstep;
              yq3 = yq2 + yqstep;
  
              xfrac = (long)( frac( xp )*256. );
              yfrac = 255-(long)( frac( yp )*256. );
  
              xm00 = (double)sourcepic[ yq0+xq0 ];
              xm10 = (double)sourcepic[ yq0+xq1 ];
              xm20 = (double)sourcepic[ yq0+xq2 ];
              xm30 = (double)sourcepic[ yq0+xq3 ];
              xm01 = (double)sourcepic[ yq1+xq0 ];
              xm11 = (double)sourcepic[ yq1+xq1 ];
              xm21 = (double)sourcepic[ yq1+xq2 ];
              xm31 = (double)sourcepic[ yq1+xq3 ];
              xm02 = (double)sourcepic[ yq2+xq0 ];
              xm12 = (double)sourcepic[ yq2+xq1 ];
              xm22 = (double)sourcepic[ yq2+xq2 ];
              xm32 = (double)sourcepic[ yq2+xq3 ];
              xm03 = (double)sourcepic[ yq3+xq0 ];
              xm13 = (double)sourcepic[ yq3+xq1 ];
              xm23 = (double)sourcepic[ yq3+xq2 ];
              xm33 = (double)sourcepic[ yq3+xq3 ];
  
              destpic[ ys+x ] = (short)floor(
                ( xm00 * sintab0[ xfrac ] * sintab0[ yfrac ]
                + xm10 * sintab1[ xfrac ] * sintab0[ yfrac ]
                + xm20 * sintab2[ xfrac ] * sintab0[ yfrac ]
                + xm30 * sintab3[ xfrac ] * sintab0[ yfrac ]
                + xm01 * sintab0[ xfrac ] * sintab1[ yfrac ]
                + xm11 * sintab1[ xfrac ] * sintab1[ yfrac ]
                + xm21 * sintab2[ xfrac ] * sintab1[ yfrac ]
                + xm31 * sintab3[ xfrac ] * sintab1[ yfrac ]
                + xm02 * sintab0[ xfrac ] * sintab2[ yfrac ]
                + xm12 * sintab1[ xfrac ] * sintab2[ yfrac ]
                + xm22 * sintab2[ xfrac ] * sintab2[ yfrac ]
                + xm32 * sintab3[ xfrac ] * sintab2[ yfrac ]
                + xm03 * sintab0[ xfrac ] * sintab3[ yfrac ]
                + xm13 * sintab1[ xfrac ] * sintab3[ yfrac ]
                + xm23 * sintab2[ xfrac ] * sintab3[ yfrac ]
                + xm33 * sintab3[ xfrac ] * sintab3[ yfrac ] ) );
  
              xq0++; xq1++; xq2++; xq3++;
  
              xm00 = (double)sourcepic[ yq0+xq0 ];
              xm10 = (double)sourcepic[ yq0+xq1 ];
              xm20 = (double)sourcepic[ yq0+xq2 ];
              xm30 = (double)sourcepic[ yq0+xq3 ];
              xm01 = (double)sourcepic[ yq1+xq0 ];
              xm11 = (double)sourcepic[ yq1+xq1 ];
              xm21 = (double)sourcepic[ yq1+xq2 ];
              xm31 = (double)sourcepic[ yq1+xq3 ];
              xm02 = (double)sourcepic[ yq2+xq0 ];
              xm12 = (double)sourcepic[ yq2+xq1 ];
              xm22 = (double)sourcepic[ yq2+xq2 ];
              xm32 = (double)sourcepic[ yq2+xq3 ];
              xm03 = (double)sourcepic[ yq3+xq0 ];
              xm13 = (double)sourcepic[ yq3+xq1 ];
              xm23 = (double)sourcepic[ yq3+xq2 ];
              xm33 = (double)sourcepic[ yq3+xq3 ];
  
              destpic[ ys+x+1 ] = (short)floor(
                ( xm00 * sintab0[ xfrac ] * sintab0[ yfrac ]
                + xm10 * sintab1[ xfrac ] * sintab0[ yfrac ]
                + xm20 * sintab2[ xfrac ] * sintab0[ yfrac ]
                + xm30 * sintab3[ xfrac ] * sintab0[ yfrac ]
                + xm01 * sintab0[ xfrac ] * sintab1[ yfrac ]
                + xm11 * sintab1[ xfrac ] * sintab1[ yfrac ]
                + xm21 * sintab2[ xfrac ] * sintab1[ yfrac ]
                + xm31 * sintab3[ xfrac ] * sintab1[ yfrac ]
                + xm02 * sintab0[ xfrac ] * sintab2[ yfrac ]
                + xm12 * sintab1[ xfrac ] * sintab2[ yfrac ]
                + xm22 * sintab2[ xfrac ] * sintab2[ yfrac ]
                + xm32 * sintab3[ xfrac ] * sintab2[ yfrac ]
                + xm03 * sintab0[ xfrac ] * sintab3[ yfrac ]
                + xm13 * sintab1[ xfrac ] * sintab3[ yfrac ]
                + xm23 * sintab2[ xfrac ] * sintab3[ yfrac ]
                + xm33 * sintab3[ xfrac ] * sintab3[ yfrac ] ) );
  
              xq0++; xq1++; xq2++; xq3++;
  
              xm00 = (double)sourcepic[ yq0+xq0 ];
              xm10 = (double)sourcepic[ yq0+xq1 ];
              xm20 = (double)sourcepic[ yq0+xq2 ];
              xm30 = (double)sourcepic[ yq0+xq3 ];
              xm01 = (double)sourcepic[ yq1+xq0 ];
              xm11 = (double)sourcepic[ yq1+xq1 ];
              xm21 = (double)sourcepic[ yq1+xq2 ];
              xm31 = (double)sourcepic[ yq1+xq3 ];
              xm02 = (double)sourcepic[ yq2+xq0 ];
              xm12 = (double)sourcepic[ yq2+xq1 ];
              xm22 = (double)sourcepic[ yq2+xq2 ];
              xm32 = (double)sourcepic[ yq2+xq3 ];
              xm03 = (double)sourcepic[ yq3+xq0 ];
              xm13 = (double)sourcepic[ yq3+xq1 ];
              xm23 = (double)sourcepic[ yq3+xq2 ];
              xm33 = (double)sourcepic[ yq3+xq3 ];
  
              destpic[ ys+x+2 ] = (short)floor(
                ( xm00 * sintab0[ xfrac ] * sintab0[ yfrac ]
                + xm10 * sintab1[ xfrac ] * sintab0[ yfrac ]
                + xm20 * sintab2[ xfrac ] * sintab0[ yfrac ]
                + xm30 * sintab3[ xfrac ] * sintab0[ yfrac ]
                + xm01 * sintab0[ xfrac ] * sintab1[ yfrac ]
                + xm11 * sintab1[ xfrac ] * sintab1[ yfrac ]
                + xm21 * sintab2[ xfrac ] * sintab1[ yfrac ]
                + xm31 * sintab3[ xfrac ] * sintab1[ yfrac ]
                + xm02 * sintab0[ xfrac ] * sintab2[ yfrac ]
                + xm12 * sintab1[ xfrac ] * sintab2[ yfrac ]
                + xm22 * sintab2[ xfrac ] * sintab2[ yfrac ]
                + xm32 * sintab3[ xfrac ] * sintab2[ yfrac ]
                + xm03 * sintab0[ xfrac ] * sintab3[ yfrac ]
                + xm13 * sintab1[ xfrac ] * sintab3[ yfrac ]
                + xm23 * sintab2[ xfrac ] * sintab3[ yfrac ]
                + xm33 * sintab3[ xfrac ] * sintab3[ yfrac ] ) );
              break;

            default:
              return( CCDLIB_WrongParameterError );
              break;
          }
        }
      }
    }
  }

  CCDLIB_Matrixfree( projmatrix );
  if( method == CCDLIB_Lanczos2Const )
  {
    free( sintab0 );
    free( sintab1 );
    free( sintab2 );
    free( sintab3 );
  }

  return( CCDLIB_Successful );
}



/*** Routine zur Erzeugung eines leeren Bildes ***/

CCDLIB_PictureparamStruct CCDLIB_Getnullpicture
(
  void
)
{
  CCDLIB_PictureparamStruct nullpicturestruct =
  {
    0L,                         /* Breite des gesamten Bildpuffers in Pixel */
    0L,                         /* Hoehe des gesamten Bildpuffers in Pixel */
    0x8000L,                    /* Umfang der moeglichen Graustufen */
    16,                         /* Bits pro Pixel */
    1.0,                        /* Verhaeltnis von Pixelbreite/Pixelhoehe (X/Y) */
    1.0,                        /* Pixelbreite */
    1.0,                        /* Pixelhoehe */
    CCDLIB_ShortPixels,         /* Daten-Type eines Pixels */
    CCDLIB_BlackWhiteColorType, /* Zugehoerigkeit zu einem SW / Farbbild */
    CCDLIB_EmptyColor,          /* Evtl. Farbkanal */
    CCDLIB_CartesianType,       /* Projektionsart der Bilddaten */
    0,                          /* Fuellwert fuer unsichtbare Pixel */
    0,                          /* Fuellwert fuer unsichtbare Pixel */
    0,                          /* Fuellwert fuer unsichtbare Pixel */
    1,                          /* Grauskala */
    0,                          /* Offset fuer Grau-Nullpunkt i.e. 0 */
    "Counts",                   /* Grauwertskaleneinheit */
    0.,                         /* Nullpunktkoordinate in Pixeln, bez. auf oben links */
    0.,                         /* Nullpunktkoordinate in Pixeln,,bez. auf oben links */
    1.,                         /* X-Achsenskaleneinheit pro Pixel */
    1.,                         /* Y-Achsenskaleneinheit pro Pixel */
    0.,                         /* X-Achsenskaleneinheit beim Nullpunkt */
    0.,                         /* Y-Achsenskaleneinheit beim Nullpunkt */
    "Pixel",                    /* Skaleneinheit der X-Achse */
    "Pixel",                    /* Skaleneinheit der Y-Achse */
    1,                          /* Bildnummer des Bildes in einer Serie (1-n) */
    1                           /* Laenge einer Aufnahmeserie */
  };

  return( nullpicturestruct );
}


